// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>

// #define OLD_SYSTEM

#include "audiorecorder.h"

//===========================================================================
// constructor for the audio recorder
//===========================================================================

AudioRecorder::AudioRecorder(const char * cp_DeviceName)
{
   strncpy(m_ca_DeviceName,cp_DeviceName,99);
   m_ca_DeviceName[99] ='\0';
   m_b_ArtsdUsed       = false;
   
   // no regular audio device open
   m_i_AudioHandle     = -1;
   i_Error =0;

#ifdef HAS_ARTSD
   if( strcmp( cp_DeviceName, "artsd" ) == 0 )
   {
       m_b_ArtsdUsed = true;

       arts_init();
       stream     = arts_record_stream( 44100, 16, 2, "khdrecord" );
       packetsize = arts_stream_get( stream, ARTS_P_PACKET_SIZE );
       if(stream == 0)
       {
          fprintf(stderr,"could not connect to artsd!\n");
          i_Error = 1;
          return;
       }
       if( packetsize >= SAVEBLOCKBUFFERSIZE )
       {
          fprintf(stderr,"arts packetsize(%d) >= SaveblockBufferSize(%d)!\n",
                         packetsize,
			 SAVEBLOCKBUFFERSIZE);
	  i_Error = 1;
       }
       m_i_AudioHandle = 0;
   }
#endif // HAS_ARTSD

   if( ! m_b_ArtsdUsed)
   {
      // Attempt to open the audio device for read only
      m_i_AudioHandle = open(m_ca_DeviceName, O_RDONLY  );
      if (m_i_AudioHandle == -1)
      {
         fprintf(stderr,"error open the device\n");
         i_Error = 1;
      }


      if (ioctl(m_i_AudioHandle, SNDCTL_DSP_GETFMTS, &i_AudioFormatMask) < 0)
      {
         i_Error =2;
      }
   } // no artsd


   pc_SaveBlockArray = NULL;
   
   StopRecording = false;
   
   
} // AudioRecorder()

//===========================================================================
//
//===========================================================================

int AudioRecorder::SetDevice(const char * cp_DeviceName)
{
   i_Error =0;

   strncpy(m_ca_DeviceName,cp_DeviceName,99);
   m_ca_DeviceName[99] ='\0';

   // if opened before, close it
   if(m_i_AudioHandle >=0)
   {
      close(m_i_AudioHandle);
      m_i_AudioHandle = -1;
   }

#ifdef HAS_ARTSD
   if( stream != 0 )
   {
      arts_close_stream( stream );
      arts_free();
      m_b_ArtsdUsed = false;
   }

   if( strcmp( cp_DeviceName, "artsd" ) == 0 )
   {
      m_b_ArtsdUsed = true;
      arts_init();
      stream = arts_record_stream( 44100, 16, 2, "khdrecord" );
      packetsize = arts_stream_get( stream, ARTS_P_PACKET_SIZE );
      if( packetsize >= SAVEBLOCKBUFFERSIZE )
      {
         fprintf( stderr, "arts packetsize(%d) >= SAVEBLOCKBUFFERSIZE(%d)!\n",
			packetsize, SAVEBLOCKBUFFERSIZE );
         i_Error = 1;
      }
      m_i_AudioHandle = 0; // trick to avoid problems
   }
   return 0 ;
#endif


   // Attempt to open the audio device for read only
   m_i_AudioHandle = open(m_ca_DeviceName, O_RDONLY  );
   if (m_i_AudioHandle == -1)
   {
      fprintf(stderr,"error open the device\n");
      i_Error = 1;
   }


   if (ioctl(m_i_AudioHandle, SNDCTL_DSP_GETFMTS, &i_AudioFormatMask) < 0)
   {
      i_Error =2;
   }

   return i_Error;
} // AudioRecorder()


//===========================================================================
// destructor
//===========================================================================

AudioRecorder::~AudioRecorder()
{
   if(m_i_AudioHandle>0)
   {
      close(m_i_AudioHandle);
   }
#ifdef HAS_ARTSD
   if(   m_b_ArtsdUsed
      && stream != NULL )
   {
      arts_close_stream( stream );
      arts_free();
      stream = NULL;
      m_b_ArtsdUsed =false;
   }
#endif
} // ~AudioRecorder()

//===========================================================================
// the the sampling speed
//===========================================================================


bool AudioRecorder::SetSpeed(int &i_Speed)
{
   int i_Bits;
   int i_Stereo;
   int i_Return;

   i_Bits   = 16;
   i_Stereo = 2;


   if(m_b_ArtsdUsed)
   {
      i_Speed = 44100;
      return true;
   }

   if(m_i_AudioHandle <0)
   {
      return false;
   }

   SyncAudio();

   /* Set the sample speed, size and stereoness */
   if (ioctl(m_i_AudioHandle, SNDCTL_DSP_SAMPLESIZE, &i_Bits) < 0)
   {
      return false;
   }

   if (ioctl(m_i_AudioHandle, SNDCTL_DSP_STEREO, &i_Stereo) < 0)
   {
      return false;
   }

   i_Return = ioctl(m_i_AudioHandle, SNDCTL_DSP_SPEED, &i_Speed);
   if (i_Return  < 0)
   {
      return false;
   }

   return true;


} // SetSpeed()

//===========================================================================
// sync the audio card
//===========================================================================


bool AudioRecorder::SyncAudio()
{
   if(m_b_ArtsdUsed)
   {
      return true;
   }

   if(m_i_AudioHandle <0)
   {
      return false;
   }

    if (ioctl (m_i_AudioHandle, SNDCTL_DSP_SYNC, NULL) < 0)
    {

	return false;
    }
    return true;

} // SyncAudio()

//===========================================================================
// read some date from the device
//===========================================================================

int AudioRecorder::ReadAudio()
{
   int count;

   if(pc_SaveBlockArray == NULL)
   {
      return 0;
   }

   if(m_i_AudioHandle <0)
   {
      return 0;
   }

   while ( true )
   {
      SaveBlock *pc_ActualBlock;

      // we should get an empty block
      pc_ActualBlock = pc_SaveBlockArray->GetNextRecordBlock();
      if( pc_ActualBlock == NULL)
      {
          return 0;
      }     
#ifdef OLD_SYSTEM
      {
       // on an old system read the buffer in 16 portions
         int i; 
         int i_Size;
         unsigned char * pc_Buffer;

         i_Size = pc_ActualBlock->BufferSize / 16;
         pc_Buffer = (unsigned char *) pc_ActualBlock->Buffer;
         for(i=0; i<16; i++)
         {
            // do the real read
            count = read(m_i_AudioHandle, 
                   pc_Buffer,
                   i_Size);
            // next portion of buffer
            pc_Buffer += i_Size;   
         }
         // set the count for read all
         count = pc_ActualBlock->BufferSize;
      }
#else
      // do the real read
      if( m_b_ArtsdUsed )
      {
#ifdef HAS_ARTSD
         if(stream != NULL)
         {
            count = arts_read( stream,
                           pc_ActualBlock->Buffer,
                           pc_ActualBlock->BufferSize );
         }
         else
         {
            count =0;
         }
#endif
      }
      else
      {
         count = read(m_i_AudioHandle,
                      pc_ActualBlock->Buffer,
                      pc_ActualBlock->BufferSize);
      }

#endif // ! OLD_SYSTEM
      // did we get some bytes
      if(count >0)
      {
         pc_ActualBlock->BufferFilled=count;
         // put that block into the writing list

      }
      else if( count <=0)
      {
         pc_ActualBlock->SetFinalBlock();
      }

      // should we stop the recording
      if(StopRecording)
      {
         pc_ActualBlock->SetFinalBlock();
      }
      pc_SaveBlockArray->ReleaseRecordBlock();

      if(StopRecording)
      {
          break;
      }
   } // while true

   return 1;
} // ReadAudio()


//===========================================================================
// the the saveblock pointer into this structure
//===========================================================================

void AudioRecorder::SetSaveBlockArray(SaveBlockArray *pc_SaveBlockArray)
{

   this ->pc_SaveBlockArray = pc_SaveBlockArray;

} // SetSaveBlockArray

//===========================================================================
//
//===========================================================================

int AudioRecorder::GetError()
{
   return i_Error;
} // GetError()

//===========================================================================
//
//===========================================================================


void AudioRecorder::Abort()
{
   SaveBlock *pc_ActualBlock;

   if(pc_SaveBlockArray == NULL)
   {
      return;
   }
   // we should get an empty block
   pc_ActualBlock = pc_SaveBlockArray->GetNextRecordBlock();
   if(pc_ActualBlock ==NULL)
   {
      return;
   }
   pc_ActualBlock->BufferFilled= 0;
   pc_ActualBlock->SetFinalBlock();
   pc_SaveBlockArray->ReleaseRecordBlock();

} // Abort()

//===========================================================================
//
//===========================================================================




