// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// CommandLine
// analyze the command line parameters
//
//===========================================================================

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <stdio.h>
#include <string.h>
#include <time.h>
#include "commandline.h"


CommandLineParameter::CommandLineParameter(int argc, char *argv[])
{
   // initialize with default values
   m_i_InputLevel       = 60;
   m_i_InputLevelMaster = 50;
   m_i_Bitrate          = 128;
   m_i_ServerPort       = DEFAULT_SERVER_PORT;
   m_qs_Outputname      = "test";
   m_qs_InputDevice     = "line";
   m_qs_SoundDevice     = "/dev/dsp";
   m_qs_MixerDevice     = "/dev/mixer";
   m_qs_RecorderFile    = "";	      // no recorder file
   m_t_StartTime        = 0;             // no start time
   m_t_StopTime         = 0;             // no stop time
   m_b_ParseOK          = true;
   m_b_TestMode         = false;
   m_b_ServerMode       = false;
   m_b_RemoteMode       = false;
   m_e_RecordMode       = ModeMp3;	// default is mp3
   m_i_AnalyzeLevel     = 5;
   m_i_AnalyzeTimeout   = 5;
   m_b_AnalyzeMode      = false ;


   // if there are arguments, just check them
   if(argc > 1 && argv != NULL)
   {
      ParseParameter(argc,argv);
   }

   // check for valid values
   if(m_b_ParseOK)
   {
      CheckValidParameters();
   }

} // CommandLineParameter() - constructor

//===========================================================================
//
//===========================================================================

CommandLineParameter::~CommandLineParameter()
{


} // ~CommandLineParameter() - destructor


//===========================================================================
// parse all parameters
//===========================================================================

int CommandLineParameter::ParseParameter(int argc, char *argv[])
{
   int i;

   for(i =1; i<argc; i++)
   {
      ParseASingleParameter(argv[i]);
   }

   return 0;
} // ParseParameter()

//===========================================================================
// parse a single parameter and look whether it fits
//===========================================================================

int CommandLineParameter::ParseASingleParameter( char *cp_Parameter)
{
   QString qs_Parameter;


   qs_Parameter = cp_Parameter;
   CheckParameter(qs_Parameter,"-command",ParameterType_Empty,NULL);

   CheckParameter(qs_Parameter,
                  "-test",
                  ParameterType_Bool,
                  &m_b_TestMode);

   CheckParameter(qs_Parameter,
                  "-outputfile=",
                  ParameterType_String,
                  &m_qs_Outputname);

   CheckParameter(qs_Parameter,
                  "-input=",
                  ParameterType_String,
                  &m_qs_InputDevice);

   CheckParameter(qs_Parameter,
                  "-sounddevice=",
                  ParameterType_String,
                  &m_qs_SoundDevice);

   CheckParameter(qs_Parameter,
                  "-mixerdevice=",
                  ParameterType_String,
                  &m_qs_MixerDevice);

   CheckParameter(qs_Parameter,
                  "-recorderfile=",
                  ParameterType_String,
                  &m_qs_RecorderFile);

   CheckParameter(qs_Parameter,
                  "-level=",
                  ParameterType_Integer,
                  &m_i_InputLevel);

   CheckParameter(qs_Parameter,
                  "-master=",
                  ParameterType_Integer,
                  &m_i_InputLevelMaster);

   CheckParameter(qs_Parameter,
                  "-bitrate=",
                  ParameterType_Integer,
                  &m_i_Bitrate);

   CheckParameter(qs_Parameter,
                  "-start=",
                  ParameterType_Time,
                  &m_t_StartTime);

   CheckParameter(qs_Parameter,
                  "-stop=",
                  ParameterType_Time,
                  &m_t_StopTime);

   CheckParameter(qs_Parameter,
                  "-server",
                  ParameterType_Bool,
                  &m_b_ServerMode);

   CheckParameter(qs_Parameter,
                  "-remote",
                  ParameterType_Bool,
                  &m_b_RemoteMode);

   CheckParameter(qs_Parameter,
                  "-port=",
                  ParameterType_Integer,
                  &m_i_ServerPort);

   CheckParameter(qs_Parameter,
                  "-recordmode=",
                  ParameterType_RecordMode,
                  &m_e_RecordMode);

   CheckParameter(qs_Parameter,
                  "-analyze",
                  ParameterType_Bool,
                  &m_b_AnalyzeMode);

   CheckParameter(qs_Parameter,
                  "-analyzelevel=",
                  ParameterType_Integer,
                  &m_i_AnalyzeLevel);

   CheckParameter(qs_Parameter,
                  "-analyzetimeout=",
                  ParameterType_Integer,
                  &m_i_AnalyzeTimeout);


   return 0;

} // ParseASingleParameter()

//===========================================================================
// test a single parameter for the type and correct value
//===========================================================================


bool CommandLineParameter::CheckParameter(QString qs_Parameter,
                            QString qs_StartChars,
                            ParameterResultType e_ResultType,
                            void *p_Result)
{
   QString qs_RealParameter;
   bool    b_Result;


   if( ! qs_Parameter.startsWith(qs_StartChars) )
   {
      return false;
   }
   
   b_Result = false;

   qs_RealParameter = qs_Parameter.mid(qs_StartChars.length());


   switch(e_ResultType)
   {
      case ParameterType_Empty:
         // do nothing
         b_Result = true;
         break;
      case ParameterType_Integer:
      {
         char c_Dummy;
         int  i_ScanfResult;
         int  i_Number;
   
         if( qs_RealParameter.isEmpty())
         {
            break;
         }
         // get a valid integer
         i_ScanfResult = sscanf((const char *)qs_RealParameter,
                                "%d%c",
                                &i_Number,
                                &c_Dummy);
         if(i_ScanfResult ==1)
         {
            // we got the number
            * ((int *) p_Result)  = i_Number;
            b_Result = true;
         }
         break;
      }
      case ParameterType_Bool:
      {
   
         // we got the parameter
         * ((bool *) p_Result)  = true;
         b_Result = true;
         break;
      }
      case ParameterType_String:
      {
         if( qs_RealParameter.isEmpty())
         {
            break;
         }
         * ((QString *) p_Result)  = qs_RealParameter;
         b_Result = true;
        
         break;
      }
      case ParameterType_Time:
      {
         struct tm t_Time;
         time_t    t_UnixTime;
         char      ca_TimeString[50];
         char      *cp_StrptimeReturn;
         QChar      c_FirstChar;

         if( qs_RealParameter.isEmpty())
         {
            break;
         }
         c_FirstChar =  (qs_RealParameter[0]);
         if(c_FirstChar == QChar('+'))
         {
            QString qs_Number;
            char    c_Dummy;
            int     i_ScanResult;
            int     i_Offset;

            // get number without the '+' char
            qs_Number = qs_RealParameter.mid(1);
            i_ScanResult = sscanf((const char *) qs_Number,
                                  "%d%c",
                                  &i_Offset,
                                  &c_Dummy);
            if(i_ScanResult = 1)
            {
               time_t t_CurrentTime;
               time(&t_CurrentTime);
               t_CurrentTime += i_Offset;
               * ((time_t *) p_Result)  = t_CurrentTime;
               b_Result = true;
            }

         }
         else // no offset, but real time
         {
            // copy into a real c-string
            strncpy(ca_TimeString,(const char *) qs_RealParameter,49);
            ca_TimeString[49] = 0;
            cp_StrptimeReturn = strptime( ca_TimeString,
                                 "%Y-%m-%d/%H:%M:%S",
   		                 &t_Time);

            if( cp_StrptimeReturn != NULL)
            {
   
               t_UnixTime = mktime(&t_Time);
  
               if(t_UnixTime != ( (time_t) -1) )
               {
                  // we got the number
                  * ((time_t *) p_Result)  = t_UnixTime;
                  b_Result = true;
               }
            }
         } // if no offset time

         break;
      }
      case ParameterType_RecordMode:
      {
         if( qs_RealParameter.isEmpty())
         {
            break;
         }
         if( qs_RealParameter .compare("wav")==0)
         {
            * ((RecordMode *) p_Result) = ModeWave;
            b_Result = true;
         }
         else if( qs_RealParameter .compare("mp3")==0)
         {
            * ((RecordMode *) p_Result) = ModeMp3;
            b_Result = true;
         }
         else if( qs_RealParameter .compare("ogg")==0)
         {
            * ((RecordMode *) p_Result) = ModeOgg;
            b_Result = true;
         }
         else
         {
            b_Result = false;
         }

         break;
      }
      default:
         break;

   } // switch 

   // did we get the parameter?
   if(b_Result != true)
   {
      // no, then show error
      ShowError(qs_StartChars,qs_RealParameter,e_ResultType);
      m_b_ParseOK = false;
   }


   return b_Result;

} // CheckParameter

//===========================================================================
// error in parameter. display it on the console
//===========================================================================

void CommandLineParameter::ShowError(QString qs_Command,
                                     QString qs_RealParameter,
                                     ParameterResultType e_ResultType
                                    )
{
   fprintf(stderr,"Error at command \"%s\"\n",(const char *) qs_Command);
   fprintf(stderr,"Parameter \"%s\" has not the correct type.\n",
                        (const char *) qs_RealParameter);
   
   switch(e_ResultType)
   {
      case ParameterType_Empty:
         fprintf(stderr,"Parameter should be empty\n");
         break;
      case ParameterType_Integer:
         fprintf(stderr,"Parameter should be a number like 5\n");
         break;
      case ParameterType_String:
         fprintf(stderr,"Parameter should be a string\n");
         break;
      case ParameterType_Time:
         fprintf(stderr,"Parameter should be a time like 2002-05-23/15:42:47\n");
         break;
      default:
         break;

   } // switch 

} // ShowError



//===========================================================================
// all parameters are read. check whether they are in the
// valid range
//===========================================================================

bool CommandLineParameter::CheckValidParameters()
{
   bool b_Valid ;


   b_Valid =true;

   if(   m_i_InputLevel <0
      || m_i_InputLevel >100)
   {
      fprintf(stderr,"Wrong level. Must be between 0..100\n");
      b_Valid = false;
   }

   if(   m_i_InputLevelMaster <0 
      || m_i_InputLevelMaster >100)
   {
      fprintf(stderr,"Wrong master level. Must be between 0..100\n");
      b_Valid = false;
   }

   if(
#ifdef LOWFI
       m_i_Bitrate != 8
       && m_i_Bitrate != 16
#else
       m_i_Bitrate != 40
       && m_i_Bitrate != 128
       && m_i_Bitrate != 160
       && m_i_Bitrate != 192
       && m_i_Bitrate != 256
       && m_i_Bitrate != 320
#endif
     )
   {
#ifdef LOWFI
      fprintf(stderr,"Wrong bitrate. Must be one 8 or 16\n");
#else
      fprintf(stderr,"Wrong bitrate. Must be one of 40,128,160,192,256,320\n");
#endif
      b_Valid = false;
   }

   if(   m_i_ServerPort <0 
      || m_i_ServerPort >= 0x8000)
   {
      fprintf(stderr,"Wrong portnumber. Must be between 0..32767\n");
      b_Valid = false;
   }

   if(m_qs_InputDevice.compare("line") ==0)
   {
      m_i_InputDevice = SOUND_MIXER_LINE;
   }
   else if(m_qs_InputDevice.compare("mic") ==0)
   {
      m_i_InputDevice = SOUND_MIXER_MIC;
   }
   else if(m_qs_InputDevice.compare("cd") ==0)
   {
      m_i_InputDevice = SOUND_MIXER_CD;
   }
   else
   {
      fprintf(stderr,"Wrong input device. Must be one of line, mic or cd\n");
      m_i_InputDevice = SOUND_MIXER_LINE;
      b_Valid = false;
   }
   if(  ! m_b_ServerMode &&
        ! m_b_RemoteMode &&
        ! m_b_AnalyzeMode &&
          m_qs_RecorderFile.isEmpty() )
   {
      if(    m_t_StartTime ==0 
          || m_t_StopTime  == 0
        )
      {
         fprintf(stderr,"Missing start or stop times and no recoder file\n");
         b_Valid = false;
      }
      else if(m_t_StartTime >= m_t_StopTime)
      {
         fprintf(stderr,"Stop time before start time\n");
         b_Valid = false;
      }
      else if(m_t_StartTime < time(NULL))
      {
         fprintf(stderr,"Start time was in the past\n");
         b_Valid = false;
      }


   } // if no recorder file

   if(   m_i_AnalyzeLevel <0
      || m_i_AnalyzeLevel >10)
   {
      fprintf(stderr,"Wrong AnalyzeLevel. Must be between 0..10\n");
      b_Valid = false;
   }

   if(   m_i_AnalyzeTimeout <0
      || m_i_AnalyzeTimeout >10)
   {
      fprintf(stderr,"Wrong AnalyzeTimeout. Must be between 0..10\n");
      b_Valid = false;
   }



   // in the test mode everything is valid
   if(m_b_TestMode)
   {
      b_Valid = true;
   }

   if( ! b_Valid )
   {
      m_b_ParseOK =false;
   }

   return b_Valid;

} // CheckValidParameters()

//===========================================================================
//
//===========================================================================










