// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// filewriter
// write sampleblocks into a file
//
//===========================================================================

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <linux/in.h>
#include <signal.h>


extern "C"
{
#include "lame/lame.h"
}

#include "filewriter.h"
#include "saveblock.h"
#include "mp3stream.h"

static  FileWriter *pc_FileWriterThisPointer = NULL;


//===========================================================================
// got it from somewhere
//===========================================================================

typedef struct _WaveHeaderStructure 
{
  u_long	main_chunk;	/* 'RIFF' */
  u_long	length;		/* Length of rest of file */
  u_long	chunk_type;	/* 'WAVE' */

  u_long	sub_chunk;	/* 'fmt ' */
  u_long	sc_len;		/* length of sub_chunk, =16 (rest of chunk) */
  u_short	format;		/* should be 1 for PCM-code */
  u_short	modus;		/* 1 Mono, 2 Stereo */
  u_long	sample_fq;	/* frequence of sample */
  u_long	byte_p_sec;
  u_short	byte_p_spl;	/* samplesize; 1 or 2 bytes */
  u_short	bit_p_spl;	/* 8, 12 or 16 bit */

  u_long	data_chunk;	/* 'data' */
  u_long	data_length;	/* samplecount (lenth of rest of block?)*/
} WaveHeaderStructure;


//===========================================================================
//
//===========================================================================

void SignalHandler(int i)
{
   //   fprintf(stderr,"SignalHandler %d\n",i);
   if(pc_FileWriterThisPointer != NULL)
   {
      pc_FileWriterThisPointer -> SignalHandler();
   }
} // SignalHandler()


//===========================================================================
//
//===========================================================================

FileWriter::FileWriter()
{
   // save ptr for the signal handler
   pc_FileWriterThisPointer = this;

   m_i_UsedSampleRate = 44100; // assume default

   // file unused
   m_i_FileHandle = -1;

   // disable the server mode
   m_b_ServerMode      = false;
   m_b_ClientConnected = false;

   // allocate a destination buffer
   m_cp_DestinationBuffer      = new char[0x20000];
   m_i_DestinationBufferLength = 0x20000;

   m_i_SingleBufferSize = 0x4000;
   sip_LeftBuffer  = new short int [m_i_SingleBufferSize];  // 16 K
   sip_RightBuffer = new short int [m_i_SingleBufferSize];



   // clean up the lame global flags
   m_ps_LameGlobalFlags = NULL;

   // use filename without a date
   m_b_FilenameWithDate = false;
   // start with a zero counter
   m_i_FileUpCounter    = 0;

   m_i_ActualMp3StreamBuffer = 0;
   m_i_StreamBufferCount     = 0;
   m_c_StreamBuffer          = NULL;

   m_i_ServerPort = DEFAULT_SERVER_PORT;
  
   f_OggBuffer = NULL;
   
} // constructor

//===========================================================================
//
//===========================================================================


FileWriter::~FileWriter()
{
   // switch off the server mode 
   if(m_b_ServerMode )
   {
      SetServerMode (false);
   }

   // delete the destination buffer
   if(m_cp_DestinationBuffer != NULL)
   {
      delete m_cp_DestinationBuffer ;
      m_cp_DestinationBuffer = NULL;
   }



} // destructor

//===========================================================================
//
//===========================================================================

void FileWriter::SetUsedSampleRate(int i_UsedSampleRate)
{
   m_i_UsedSampleRate = i_UsedSampleRate;

} // SetUsedSampleRate

//===========================================================================
//
//===========================================================================

void FileWriter::AdjustFileAtEnd()
{
   if(m_i_FileHandle < 0)
   {
      // no file currently open
      return;
   }

   if(m_Destination == ModeWave)
   {
      WaveHeaderStructure  s_Header;
      unsigned long        ul_NewLength;
      struct stat          statbuffer;
      long                 l_ReadResult;
	
      // go to position 0
 	
      lseek(m_i_FileHandle,0,SEEK_SET);

      // clean the header
      memset(&s_Header,0,sizeof(s_Header));

      // read the current header
      l_ReadResult = read(m_i_FileHandle,
                          &s_Header,
                          sizeof(WaveHeaderStructure));
	
      if( l_ReadResult != sizeof(WaveHeaderStructure) )
      {
         return;
      }
	
      // get the actual file size
      fstat(m_i_FileHandle,& statbuffer);

      // calculate the new sample (data) length
      ul_NewLength = statbuffer.st_size - sizeof(WaveHeaderStructure);

      // write that length into the header structure
      s_Header.data_length = ul_NewLength;


      // again back to the front
      lseek(m_i_FileHandle,0,SEEK_SET);
	
      // write the new header to the file
      write(m_i_FileHandle,&s_Header,sizeof(WaveHeaderStructure));

   } // if mode wave

   else if(m_Destination == ModeMp3)
   {
      int i_EncodeResult;
      
      i_EncodeResult = lame_encode_finish(m_ps_LameGlobalFlags,
		         (unsigned char *) m_cp_DestinationBuffer,
		         m_i_DestinationBufferLength );

      //fprintf(stderr,"i_EncodeResult = %d\n",i_EncodeResult);				
      if( i_EncodeResult>0 )
      {
         write(m_i_FileHandle,
               m_cp_DestinationBuffer,
               i_EncodeResult
               );
      }			

   } // if mode mp3

   else if(m_Destination == ModeOgg)
   {
      OggCode(0);

      ogg_stream_clear(&s_OggStreamState);

      vorbis_block_clear(&s_VorbisBlock);
      vorbis_dsp_clear(&s_VorbisDspState);
      vorbis_info_clear(&s_VorbisInfo);

      f_OggBuffer = NULL;
   } // if mode ogg


   // close the output file
   close(m_i_FileHandle);


   // did we have an empty file
   if(m_l_SampleDataWritten ==0 )
   {
       // delete empty files
       remove(m_qs_TotalFileName);
       
   }


   // remeber, that the file is unused      
   m_i_FileHandle = -1;
   


} // AdjustFileAtEnd()


//===========================================================================
//
//===========================================================================

void FileWriter::SetEncodeRate(int i_EncodeRate)
{
   m_i_MP3Bitrate = i_EncodeRate;

} // SetEncodeRate()


//===========================================================================
//
//===========================================================================

void FileWriter::SetDestination(RecordMode e_Destination)
{
   m_Destination = e_Destination;
} // SetDestination()


//===========================================================================
// open a file
//===========================================================================

int FileWriter::OpenFile()
{
   char s_Extension[10];

   // make sure, that there is at least a name

   if(m_qs_FileNamePrefix.isEmpty())
   {
      m_qs_FileNamePrefix = "test";
   }

   m_i_FileHandle = -1;

   // a new file contains no bytes
   m_l_SampleDataWritten =0;

   if(m_b_ServerMode)
   {
     // initialize the mp3 encoder
      int i_Result;

      // clean up the global flags
      m_ps_LameGlobalFlags = NULL;

      m_ps_LameGlobalFlags  = lame_init();
      //fprintf(stderr,"init result = %d\n",i_Result);


      lame_set_num_channels(m_ps_LameGlobalFlags,2);
      lame_set_in_samplerate(m_ps_LameGlobalFlags,m_i_UsedSampleRate);
      if(m_i_MP3Bitrate <= 40)
      {
         // switch to mono
         lame_set_mode(m_ps_LameGlobalFlags, MONO);
         lame_set_brate(m_ps_LameGlobalFlags , m_i_MP3Bitrate );
      }
      else
      {
         // regular stereo
         lame_set_mode(m_ps_LameGlobalFlags, STEREO);
         lame_set_brate(m_ps_LameGlobalFlags , m_i_MP3Bitrate );
      }
      lame_set_quality(m_ps_LameGlobalFlags,2);


      i_Result = lame_init_params(m_ps_LameGlobalFlags);
      //fprintf(stderr,"init params result = %d\n",i_Result);
      // in server mode don't go to the real file routines
      return 0;
   }


   // open the file depending on the mode
   if(m_Destination == ModeMp3)
   {
      strcpy(s_Extension,".mp3");
   }
   else if(m_Destination == ModeWave)
   {
      strcpy(s_Extension,".wav");
   }
   else if(m_Destination == ModeOgg)
   {
      strcpy(s_Extension,".ogg");
   }
   else
   {
      strcpy(s_Extension,".dat");
   }



   if(  m_b_FilenameWithDate )
   {
      // get the current time
      time_t t_CurrentTime;
      struct tm *t_LocalTime;

      char sz_TimeString[100];


      t_CurrentTime = time(&t_CurrentTime);
      t_LocalTime   = localtime( &t_CurrentTime );

      strftime(sz_TimeString ,100,"%Y%m%d%H%M%S" ,t_LocalTime);

      m_qs_TotalFileName = m_qs_FileNamePrefix;
      m_qs_TotalFileName += sz_TimeString;
      m_qs_TotalFileName += s_Extension;


   }
   else // not analyze mode
   {
      if( m_i_FileUpCounter ==0 )
      {
         m_qs_TotalFileName = m_qs_FileNamePrefix;
         m_qs_TotalFileName += s_Extension;
      }
      else
      {
         char ca_SequenceString[20];
         sprintf(ca_SequenceString,"%d",m_i_FileUpCounter);

         m_qs_TotalFileName = m_qs_FileNamePrefix;
         m_qs_TotalFileName += ca_SequenceString;
         m_qs_TotalFileName += s_Extension;

      }
   }


   // try to open the file
   m_i_FileHandle =  open((const char *) m_qs_TotalFileName,
    	             O_CREAT | O_TRUNC | O_RDWR,
		     S_IRWXU);


   if(	m_i_FileHandle <0)
   {
      return -1;
   }

   if( m_Destination == ModeWave )
   {
      // write the wav-file prefix
      WaveHeaderStructure s_Header;

      char *riff = "RIFF";
      char *wave = "WAVE";
      char *fmt  = "fmt ";
      char *data = "data";
      int   i_WriteResult;

      memcpy(&(s_Header.main_chunk), riff, 4);
      s_Header.length = sizeof(WaveHeaderStructure) 
                        - 8 + 0; // gets up filled later
      memcpy(&(s_Header.chunk_type), wave, 4);

      memcpy(&(s_Header.sub_chunk), fmt, 4);
      s_Header.sc_len     = 16;
      s_Header.format     = 1;
      s_Header.modus      = 2;              // stereo, 2 channles
      s_Header.sample_fq  = m_i_UsedSampleRate;   // used rate
      s_Header.byte_p_sec = ( 2)*(2)*m_i_UsedSampleRate;
      s_Header.byte_p_spl = ( 2)*(2);
      s_Header.bit_p_spl  = 16;

      memcpy(&(s_Header.data_chunk), data, 4);
      s_Header.data_length = sizeof(s_Header);

      // write the header to file
      i_WriteResult = write(m_i_FileHandle,
                            &s_Header,
                            sizeof(s_Header));

   } // block for the wav header

   else if(m_Destination == ModeMp3)
   {
      // initialize the mp3 encoder
      int i_Result;


      m_ps_LameGlobalFlags = lame_init();
      //fprintf(stderr,"init result = %d\n",i_Result);


      lame_set_num_channels(m_ps_LameGlobalFlags,2);
      lame_set_in_samplerate(m_ps_LameGlobalFlags,m_i_UsedSampleRate);
      if(m_i_MP3Bitrate <= 40)
      {
         // switch to mono
         lame_set_mode(m_ps_LameGlobalFlags, MONO);
         lame_set_brate(m_ps_LameGlobalFlags , m_i_MP3Bitrate );
      }
      else
      {
         // regular stereo
         lame_set_mode(m_ps_LameGlobalFlags, STEREO);
         lame_set_brate(m_ps_LameGlobalFlags , m_i_MP3Bitrate );
      }
      lame_set_quality(m_ps_LameGlobalFlags,2);


      i_Result = lame_init_params(m_ps_LameGlobalFlags);
      //fprintf(stderr,"init params result = %d\n",i_Result);


   } // if mode = mp3

   else if(m_Destination == ModeOgg)
   {
      // initialize the ogg encoder

      int i_Status;
      int i_Return;
      int i_BytesWritten;


      // Have vorbisenc choose a mode for us
      vorbis_info_init(&s_VorbisInfo);


      // set channel number, samplerate and encoding bitrate
      i_Status = vorbis_encode_init(&s_VorbisInfo,
			2, // channels,
         		m_i_UsedSampleRate, // rate,
                        m_i_MP3Bitrate *1024,
                        m_i_MP3Bitrate *1024,
                        m_i_MP3Bitrate *1024);

      if(i_Status != 0 )
      {
         fprintf(stderr, "Mode initialisation failed: invalid parameters for bitrate\n");
         vorbis_info_clear(&s_VorbisInfo);
         return 1;
      }


      // Now, set up the analysis engine, stream encoder, and other
      // preparation before the encoding begins.
      //

      vorbis_analysis_init(&s_VorbisDspState,&s_VorbisInfo);
      vorbis_block_init(   &s_VorbisDspState,&s_VorbisBlock);


      ogg_stream_init(&s_OggStreamState, 4711);  // serial number ??

      vorbis_comment_init(&s_VorbisComment);
      vorbis_comment_add(&s_VorbisComment, "Testversion");



      // Now, build the three header packets and send through to the stream
      // output stage (but defer actual file output until the main encode loop)

      {
         ogg_packet header_main;
         ogg_packet header_comments;
         ogg_packet header_codebooks;
         int result;

         // Build the packets
         vorbis_analysis_headerout(&s_VorbisDspState,
                                          &s_VorbisComment,
				          &header_main,
                                          &header_comments,
                                          &header_codebooks);

         // And stream them out
         ogg_stream_packetin(&s_OggStreamState,&header_main);
         ogg_stream_packetin(&s_OggStreamState,&header_comments);
         ogg_stream_packetin(&s_OggStreamState,&header_codebooks);

         while((result = ogg_stream_flush(&s_OggStreamState, &s_OggPage)))
         {
            int i_PreambleLength;
            if(result==0)
            {
               break;
            }

	    i_PreambleLength = OggWritePage(&s_OggPage);

            if(i_PreambleLength != s_OggPage.header_len + s_OggPage.body_len)
            {
 	       fprintf(stderr,"Failed writing header to output stream\n");
               i_Return = 1;
               // goto cleanup; /* Bail and try to clean up stuff */
            }
            else
            {
               m_l_SampleDataWritten += i_PreambleLength;
            }
         } // while
      }  // block for the header



   } // if mode = ogg


   return m_i_FileHandle;


} // OpenFile()


//===========================================================================
//
//===========================================================================

int FileWriter::WriteBlock(SaveBlock * pc_Saveblock)
{

   if(m_b_ServerMode)
   {
      WriteBlockServer(pc_Saveblock);
   }
   else
   {
      if(m_Destination == ModeWave)
      {
         // count the bytes in the file
         m_l_SampleDataWritten += pc_Saveblock->BufferFilled;

         pc_Saveblock->WriteBlock(m_i_FileHandle);

      }
      else if(m_Destination == ModeMp3)
      {
         WriteBlockMP3(pc_Saveblock);
      }
      else if(m_Destination == ModeOgg)
      {
         WriteBlockOgg(pc_Saveblock);
      }
   }
} // WriteBlock()

//===========================================================================
//
//===========================================================================

int FileWriter::WriteBlockMP3(SaveBlock * pc_SaveBlock)
{
   unsigned char *pc_InBuffer;
   int            i;
   int            i_SampleSize;
   int            i_EncodeResult;

   if(m_i_FileHandle < 0)
   {
      return -1;
   }
   
   // get pointer to the buffer
   pc_InBuffer = (unsigned char *) pc_SaveBlock->Buffer;
   
   
   // samples = total /2 <left/right>  /2 <16Bit>
   i_SampleSize = pc_SaveBlock->BufferFilled / 4 ;   
      
      
   if(i_SampleSize > m_i_SingleBufferSize)
   {
       // should never occur
       i_SampleSize = m_i_SingleBufferSize;
   }      
      
   // split the buffer into left an right   
   for(i=0; i< i_SampleSize ;   i++)
   {
      unsigned short left;
      unsigned short right;

      left  =0;
      right =0;
      left  |= ((unsigned short)(*pc_InBuffer++)) ;
      left  |= ((unsigned short)(*pc_InBuffer++)) << 8;
      right |= ((unsigned short)(*pc_InBuffer++)) ;
      right |= ((unsigned short)(*pc_InBuffer++)) << 8;
         
      sip_LeftBuffer[i]  = left   ;
      sip_RightBuffer[i] = right  ;
        
   }

   // do the mp3 encoding

   i_EncodeResult = lame_encode_buffer(m_ps_LameGlobalFlags,
   		sip_LeftBuffer,
		sip_RightBuffer,
		i_SampleSize ,   
		(unsigned char *) m_cp_DestinationBuffer,
		m_i_DestinationBufferLength );


   // if successfull encoding
   if(	i_EncodeResult>0)
   {
      write(m_i_FileHandle,
             m_cp_DestinationBuffer,
             i_EncodeResult
             );
      // count the bytes in the file
      m_l_SampleDataWritten += i_EncodeResult;
   }			
   
   
   return 0;

} // WriteBlockMP3()

//===========================================================================
//
//===========================================================================

int FileWriter::WriteBlockOgg(SaveBlock * pc_SaveBlock)
{
   unsigned char *pc_InBuffer;
   int            i;
   int            i_SampleSize;
   int            i_EncodeResult;

   if(m_i_FileHandle < 0)
   {
      return -1;
   }
   
   // get pointer to the buffer
   pc_InBuffer = (unsigned char *) pc_SaveBlock->Buffer;
   


   // samples = total /2 <left/right>  /2 <16Bit>
   i_SampleSize = pc_SaveBlock->BufferFilled / 4 ;   
      
   // get a buffer for the vorbis analysation
   f_OggBuffer = vorbis_analysis_buffer(&s_VorbisDspState, i_SampleSize);

   // split the buffer into left an right   
   for(i=0; i< i_SampleSize ;   i++)
   {
      unsigned short left;
      unsigned short right;
      short s_left;
      short s_right;

      left  =0;
      right =0;
      left  |= ((unsigned short)(*pc_InBuffer++)) ;
      left  |= ((unsigned short)(*pc_InBuffer++)) << 8;
      right |= ((unsigned short)(*pc_InBuffer++)) ;
      right |= ((unsigned short)(*pc_InBuffer++)) << 8;

      // make a signed value
      s_left = left;  
      s_right = right;
      f_OggBuffer[0][i]= ((float) s_left)  / 32768.0;
      f_OggBuffer[1][i]= ((float) s_right) / 32768.0;

   } // for i to samplesize

   OggCode(i_SampleSize);

   return 0;

} // WriteBlockOgg()



//===========================================================================
//
//===========================================================================

void FileWriter::OggCode(int i_Size)
{
   int i_PacketsDone;
   int i_EndOfStream;
   int i_Return;
   int i_VorbisAnalyseResult;

   if(i_Size ==0)
   {
      // Tell the library that we wrote 0 bytes - signalling the end
      vorbis_analysis_wrote(&s_VorbisDspState,0);
   }
   else
   {

      // Tell the library how many samples (per channel) we wrote 
      // into the supplied buffer 
      vorbis_analysis_wrote(&s_VorbisDspState, i_Size);
   }

   i_EndOfStream =0;



   // While we can get enough data from the library to analyse, one
   // block at a time...
   i_VorbisAnalyseResult = vorbis_analysis_blockout(&s_VorbisDspState,&s_VorbisBlock);

   while(i_VorbisAnalyseResult ==1)
   {

      // Do the main analysis, creating a packet 
      vorbis_analysis(&s_VorbisBlock, NULL);
      vorbis_bitrate_addblock(&s_VorbisBlock);

      while(vorbis_bitrate_flushpacket(&s_VorbisDspState, &s_OggPacket)) 
      {
         // Add packet to bitstream 
         ogg_stream_packetin(&s_OggStreamState,&s_OggPacket);
         i_PacketsDone++;

         // If we've gone over a page boundary, we can do actual output,
         // so do so (for however many pages are available) 

         while(!i_EndOfStream)
         {
            int i_Result;
            i_Result = ogg_stream_pageout(&s_OggStreamState,&s_OggPage);
            if(! i_Result) 
            {
               break;
            }

            i_Return = OggWritePage(&s_OggPage);

            if(i_Return != s_OggPage.header_len + s_OggPage.body_len)
            {
               fprintf(stderr,"Failed writing data to output stream\n");
               return;
            }
            else
            {
               m_l_SampleDataWritten += i_Return; 
            }

            if(ogg_page_eos(&s_OggPage))
            {
               i_EndOfStream = 1;
            }
         } // while not i_Eos
      } // while flushpacket

      // analyse the block
      i_VorbisAnalyseResult = vorbis_analysis_blockout(&s_VorbisDspState,&s_VorbisBlock);
   } // while blockout


} // OggCode

//==========================================================================
//
//==========================================================================

int FileWriter::OggWritePage(ogg_page *pc_OggPage)
{
   int i_Written;
   int i_Return;
   i_Return = write(m_i_FileHandle,
                    pc_OggPage->header,
                    pc_OggPage->header_len);
 
   i_Written = i_Return;

   i_Return = write(m_i_FileHandle,
                    pc_OggPage->body,
                    pc_OggPage->body_len);
 

   i_Written += i_Return;

   return i_Written;

} // OggWritePage


//===========================================================================
//
//===========================================================================

int FileWriter::WriteBlockServer(SaveBlock * pc_SaveBlock)
{
   Mp3StreamBuffer *pc_ActualBuffer;
   unsigned char   *pc_InBuffer;
   char            *pc_ConvertBuffer;
   int              i;
   int              i_SampleSize;
   int              i_EncodeResult;


   
   if(!m_b_ClientConnected)
   {
      return 0;
   }

   // get pointer to the buffer
   pc_InBuffer = (unsigned char *) pc_SaveBlock->Buffer;
   
   
   // samples = total /2 <left/right>  /2 <16Bit>
   i_SampleSize = pc_SaveBlock->BufferFilled / 4 ;   
      
      
   if(i_SampleSize > m_i_SingleBufferSize)
   {
       // should never occur
       i_SampleSize = m_i_SingleBufferSize;
   }      
      
   // split the buffer into left an right   
   for(i=0; i< i_SampleSize ;   i++)
   {
      unsigned short left;
      unsigned short right;

      left  =0;
      right =0;
      left  |= ((unsigned short)(*pc_InBuffer++)) ;
      left  |= ((unsigned short)(*pc_InBuffer++)) << 8;
      right |= ((unsigned short)(*pc_InBuffer++)) ;
      right |= ((unsigned short)(*pc_InBuffer++)) << 8;
         
      sip_LeftBuffer[i]  = left   ;
      sip_RightBuffer[i] = right  ;
        
   }

   // do the mp3 encoding
   pc_ActualBuffer = & (m_c_StreamBuffer[m_i_ActualMp3StreamBuffer]);
//   fprintf(stderr, "pc_actualBuffer =%x\n",pc_ActualBuffer);

   // go to the next buffer
   m_i_ActualMp3StreamBuffer += 1;
   if(m_i_ActualMp3StreamBuffer >= m_i_StreamBufferCount)
   {
      m_i_ActualMp3StreamBuffer=0;
   }

   pc_ConvertBuffer = (char *) & (pc_ActualBuffer->uc_Mp3Buffer[0]) ;
//   fprintf(stderr, "pc_convertBuffer =%x\n",pc_ConvertBuffer);
   i_EncodeResult = lame_encode_buffer(m_ps_LameGlobalFlags,
   		sip_LeftBuffer,
		sip_RightBuffer,
		i_SampleSize ,   
		(unsigned char *) pc_ConvertBuffer  ,
		MP3_ENCODE_BUFFER_SIZE );

   // if successfull encoding
   if(	i_EncodeResult>0)
   {
      int i_SendSize;
      int i_SendResult;
      pc_ActualBuffer->i_Version      = 1;
      pc_ActualBuffer->i_Mp3Rate      = m_i_MP3Bitrate;
      pc_ActualBuffer->i_BufferFilled = i_EncodeResult;

      i_SendSize =    (unsigned char *) (& (pc_ActualBuffer->uc_Mp3Buffer[0])) 
                   -  (unsigned char *) ( pc_ActualBuffer);
      i_SendSize += pc_ActualBuffer->i_BufferFilled;


// fprintf(stderr,"sending %d bytes to %d\n",i_SendSize,m_i_ClientSocket);
      pc_ActualBuffer->i_TransferSize = i_SendSize;
      if(m_b_ClientConnected )
      {
         i_SendResult=send(m_i_ClientSocket,
                        pc_ActualBuffer,
                        i_SendSize,
                        0
                        );
      }
      else
      {
         i_SendResult = -1;
      }
//      fprintf(stderr,"i_SendResult = %d\n",i_SendResult);
      if(i_SendResult == -1)
      {
         m_b_ClientConnected = false;
         close(m_i_ClientSocket);
      }
      // count the bytes in the file
      m_l_SampleDataWritten += i_EncodeResult;
   }			
   
   return 0;

} // WriteBlockServer()

//===========================================================================
//
//===========================================================================

void FileWriter::SetFileNameWithDate(bool b_On)
{
   m_b_FilenameWithDate = b_On;
   
} // SetFileNameWithDate()

//===========================================================================
//
//===========================================================================
void FileWriter::SetFileNamePrefix(QString qs_FileName)
{
  m_qs_FileNamePrefix = qs_FileName;
   
} // SetFileNameWithDate()

//===========================================================================
//
//===========================================================================

QString FileWriter::GetTotalFileName()
{
   return  m_qs_TotalFileName;  
} // GetTotalFileName()

//===========================================================================
// test the status - is a file currently written
//===========================================================================

bool FileWriter::IsFileWriting()
{
   if( m_i_FileHandle >=0)
   {
      return true;
   }
   return false;
} // IsFileWriting()

//===========================================================================
//
//===========================================================================

void FileWriter::NextFile()
{
   m_i_FileUpCounter ++;
} // NextFile()

//===========================================================================
//
//===========================================================================

void FileWriter::ResetFileCounter()
{
   m_i_FileUpCounter = 0;

} // ResetFileCounter()

//===========================================================================
//
//===========================================================================

int FileWriter::GetFileCounter()
{
   return m_i_FileUpCounter ;

} // GetFileCounter()

//===========================================================================
//
//===========================================================================
void FileWriter::SetFileCounter(int i_Counter)
{
   m_i_FileUpCounter = i_Counter;

} // SetFileCounter()

//===========================================================================
//
//===========================================================================

void FileWriter::SetServerMode(bool b_ServerMode)
{

   if(    b_ServerMode== true
       && m_b_ServerMode != b_ServerMode )
   {
      m_i_ActualMp3StreamBuffer = 0;
      m_i_StreamBufferCount     = 10;
      m_c_StreamBuffer          = new Mp3StreamBuffer[m_i_StreamBufferCount];


      m_b_ClientConnected = false;

      // start the server wait for connect thread

      pthread_attr_init(& m_ServerAttribut);


      pthread_create(&m_p_ServerThread,
                     &m_ServerAttribut,
                     FileWriter::ServerFunctionCB,
                     (void *) this);
   }
   else if(  b_ServerMode== false
           && m_b_ServerMode != b_ServerMode )
   {
      pthread_cancel(m_p_ServerThread);
      m_i_StreamBufferCount     = 0;
      delete m_c_StreamBuffer;
      m_c_StreamBuffer = NULL;

      m_b_ClientConnected = false;

   }

   // remember the mode
   m_b_ServerMode = b_ServerMode;

} // SetFileCounter()

//===========================================================================
//
//===========================================================================


bool FileWriter::IsServerConnected()
{
   return m_b_ClientConnected;

} // SetFileCounter()

//===========================================================================
//
//===========================================================================

void * FileWriter::ServerFunctionCB(void * p_Pointer)
{
   FileWriter * pc_FileWriter;
   pc_FileWriter = (FileWriter *) p_Pointer;
   return pc_FileWriter->ServerFunction();

} // ServerFunctionCB()

//===========================================================================
//
//===========================================================================

void * FileWriter::ServerFunction()
{
   int i_BindResult;
   struct sockaddr_in ServerAddress;
   struct sockaddr_in ClientAddress;


   m_i_ServerSocket = socket(PF_INET,SOCK_STREAM,0);
   if(m_i_ServerSocket == -1)
   {
      fprintf(stderr,"Invalid Server Socket\n");
   }

   ServerAddress.sin_family      = PF_INET;
   ServerAddress.sin_addr.s_addr = INADDR_ANY;
   ServerAddress.sin_port        = m_i_ServerPort;


   // install a signal handler
   signal(SIGPIPE,::SignalHandler);


   do
   {
      i_BindResult = bind(m_i_ServerSocket,
                       (struct sockaddr *) &ServerAddress,
                       sizeof(ServerAddress));   


      if(i_BindResult !=0)
      {
//         fprintf(stderr,"Error: BindResult = %d\n",i_BindResult);
      }
   }
   while (i_BindResult == -1);

   // wait for a connection
   listen(m_i_ServerSocket,2);
   while(true)
   {
      socklen_t ClientLength;
      int i_NewClientSocket;

      i_NewClientSocket = accept(m_i_ServerSocket,
                                   (struct sockaddr *) &ClientAddress,
                                   & ClientLength
                                   );
      if(m_b_ClientConnected)
      {
         // close an earlier connection
         close(m_i_ClientSocket);
      }
      m_i_ClientSocket    = i_NewClientSocket;
      m_b_ClientConnected = true;

      OpenFile();

//      fprintf(stderr,"Client Socket = %d\n",m_i_ClientSocket);
   }
  
   return NULL;

} // ServerFunction()



//===========================================================================
//
//===========================================================================

void FileWriter::SignalHandler()
{
//   fprintf(stderr,"Class SignalHandler \n");
   // in case of a signal terminate the connection
   if(m_b_ClientConnected )
   {
      ::close(m_i_ClientSocket);
      m_b_ClientConnected = false;
   }

}


//===========================================================================
//
//===========================================================================


void FileWriter::SetServerPort(int i_PortNumber)
{
   m_i_ServerPort = i_PortNumber;
} // SetServerPort()

//===========================================================================
//
//===========================================================================






