#ifndef _FILEWRITER_H_DEFINED
#define _FILEWRITER_H_DEFINED 1

#include <qstring.h>
#include "basedef.h"
#include "saveblock.h"

#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <semaphore.h>



// include lame.h for the lame structures
extern "C"
{
#include "lame/lame.h"

}

#include "mp3stream.h"

#include "ogg/ogg.h"
#include "vorbis/vorbisenc.h"




class FileWriter
{

public:
   FileWriter();
   ~FileWriter();

   int OpenFile();
   int WriteBlock(SaveBlock * pc_SaveBlock);
   void AdjustFileAtEnd();

   void SetEncodeRate(int i_EncodeRate);
   void SetDestination(RecordMode e_Destination);
   void SetFileNameWithDate(bool b_On);
   void SetFileNamePrefix(QString qs_FileName);
   void NextFile();
   void ResetFileCounter();
   int  GetFileCounter();
   void SetFileCounter(int i_Counter);

   void SetServerMode(bool b_ServerMode);
   bool IsServerConnected();

   QString GetTotalFileName();

   bool IsFileWriting();


   int m_i_FileUpCounter;

   void SignalHandler();

   void SetServerPort(int i_PortNumber);
   void SetUsedSampleRate(int i_UsedSampleRate);


private:
   int WriteBlockMP3(SaveBlock * pc_Saveblock);
   int WriteBlockOgg(SaveBlock * pc_Saveblock);
   int WriteBlockServer(SaveBlock * pc_Saveblock);

   static void * ServerFunctionCB(void * p_Pointer);

   void * ServerFunction();

   void OggCode(int i_Size);
   int  OggWritePage(ogg_page *page);


   RecordMode m_Destination;

   int m_i_MP3Bitrate;
   int m_i_FileHandle;
   
   int m_i_UsedSampleRate;

   char *m_cp_DestinationBuffer;
   int   m_i_DestinationBufferLength;


   int m_i_SingleBufferSize;
   short int *sip_LeftBuffer;
   short int *sip_RightBuffer;


   // samples for the actual file
   long m_l_SampleDataWritten;

   lame_global_flags *m_ps_LameGlobalFlags;

   bool m_b_FilenameWithDate;

   QString m_qs_FileNamePrefix;
   QString m_qs_TotalFileName;

   bool m_b_ServerMode;
   bool m_b_ClientConnected;
   

   pthread_attr_t m_ServerAttribut;
   pthread_t      m_p_ServerThread;
 
   int            m_i_ClientSocket;
   int            m_i_ServerSocket;
   int            m_i_ActualMp3StreamBuffer;
   int            m_i_StreamBufferCount;
   Mp3StreamBuffer *m_c_StreamBuffer;

   int            m_i_ServerPort;


   // structures needed for ogg vorbis
   ogg_stream_state s_OggStreamState; 	// os
   ogg_page 	    s_OggPage;         	// og;
   ogg_packet       s_OggPacket;	// op


   vorbis_block     s_VorbisBlock;      // vb;
   vorbis_dsp_state s_VorbisDspState;   // vd;
   vorbis_info      s_VorbisInfo;       // vi;
   vorbis_comment   s_VorbisComment;    // comments
   float            **f_OggBuffer;      // input buffer for encoding

}; // class FileWriter



#endif // _FILEWRITER_H_DEFINED




