// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// main hdrecord application window class

// #define _DEBUG

#include <qapplication.h>

#include <qvbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qgrid.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qmessagebox.h>


#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>


#include "hdrecord.h"
#include "saveblock.h"
#include "aboutbox.h"
#include "settingsbox.h"
#include "hdrecord_basis.h"
#include "recordlistbox.h"
#include "mixer.h"
#include "timedsplitbox.h"



//===========================================================================
// constructor for the recording program
//===========================================================================

HdRecord::HdRecord(QWidget *parent,
               QApplication *mainApplication)
{
   QBoxLayout  *TopLayout;
   QWidget     *TopWidget;

   // save the main app pointer
   this->mainApplication = mainApplication;

   // if no main application, then we are in the command line mode
   if(mainApplication == NULL)
   {
      m_ProgramState.b_CommandLineMode = true;
   }
   else
   {
      m_ProgramState.b_CommandLineMode = false;
   }


   // check, whether we can access the hardware
   m_qs_SoundDevice = "/dev/dsp";
   m_qs_MixerDevice = "/dev/mixer";

   m_qs_Basename = "test";


   m_i_SampleLevel = 60;	  // sample level init
   				  // it is the best value for my soundcard
   m_i_SampleLevelMaster = 40;	  // master sample level init
   		
   m_i_MP3Bitsize        = 128;	  // initial bitsize
   m_i_TimedSplitSeconds = 60;    // default: split all 60 seconds 

#ifdef LOWFI
   m_i_UsedSampleRate    = 22050; // LOW_FI used reduced SAMPLE-RATE
#else
   m_i_UsedSampleRate    = 44100; // ogg currently only likes 44100 !!!
#endif

   m_i_ServerPort        = DEFAULT_SERVER_PORT;
   
   
   m_i_MixerInputDevice = SOUND_MIXER_LINE;	
   m_p_Mixer = new MixerDevice(m_qs_MixerDevice);

   m_p_FileWriter = new FileWriter();
   

   // initital not the analyze mode   
   m_ProgramState.b_AnalyzeMode = false;
   // if analyze, then start with wait
   m_i_AnalyzeStatus = ModeAnalyzeWait;


   // initital not in timer mode
   m_ProgramState.b_TimerMode = false;

   // saveblock is empty
   m_p_PreviousSaveBlock = NULL;
   	

   if( ! m_ProgramState.b_CommandLineMode)
   {
      // create the basic data
      m_BasicData = new BasisDatenType();

      // read basisdata file
      BasicDataRead(this);
   }

   // main dialog is initial empty
   m_p_MainDialog = NULL;


   // set the volume of the line channel
   SetInputMixer();

   // no audiorecorder until now
   theAudioRecorder = NULL;
   
   // recording is not paused
   m_ProgramState.b_Paused = false;

   // currently everything is stopped
   m_ProgramState.b_GlobalStop = true;

   m_ProgramState.b_TotalFileNameChanged = false;

   // now read by basic data
   //e_RecordMode = ModeMp3;


   // check, wether we can access the devices
   CheckDevices();
   
   l_TotalBytesWritten=0;
   l_SingleBytesWritten =0;
} // HdRecord()

//===========================================================================
// update the time display
//===========================================================================

HdRecord::~HdRecord()
{
   if(m_p_Mixer != NULL)
   {
      delete m_p_Mixer;
      m_p_Mixer = NULL;
   }
   
   if(m_p_FileWriter != NULL)
   {
      delete m_p_FileWriter;
      m_p_FileWriter = NULL;
   }
   
} // destructor


//===========================================================================
// update the time display
//===========================================================================
void HdRecord::UpdateRecordingDisplay()
{
   
   double d_Delay;
   char   c_DelayString[80];
   int    i_Delay;
   int    i_LastDelay;
   int    i_Seconds;
   int    i_Minutes;
   int    i_Hours;
  
   if(m_p_MainDialog == NULL)
   {
      // no display -> nothing to update
      return; 
   }

   if(m_ProgramState.b_TotalFileNameChanged )
   {
      QString qs_TotalFileName;

      // reset trigger
      m_ProgramState.b_TotalFileNameChanged = false;
      
      // get the file name
      qs_TotalFileName = m_p_FileWriter->GetTotalFileName();
      
      // set it into the display
      m_p_MainDialog->SetRecordingFilename(qs_TotalFileName);
   }

   // calculate the delay in seconds   
   i_Delay = (int)( l_TotalBytesWritten / (m_i_UsedSampleRate*4));
   m_p_MainDialog->SetTotalTime(i_Delay);



   // ------ now the same for the last track
   // calculate the delay in seconds
   i_Delay = (int)( l_SingleBytesWritten / (m_i_UsedSampleRate*4));
   m_p_MainDialog->SetSingleTime(i_Delay);
 
   
   // force a display update
   FlushDisplay();
   
 

} // UpdateRecordingDisplay()


//=============================================================================
//
//=============================================================================

void HdRecord::DoRecordListBox()
{
   RecordListBox * recordListBox;
 
   if(m_p_MainDialog != NULL)
   {
      recordListBox = new RecordListBox(m_p_MainDialog,this);
   
      recordListBox->exec();
   
      delete recordListBox;
   }
} // DoRecordListBox()

//=============================================================================
//
//=============================================================================

void HdRecord::DoRecordListStart()
{

   // reset the stop flag
   m_ProgramState.b_GlobalStop = false;
   m_ProgramState.b_TimerStop = false;
   
   m_ProgramState.b_TimerMode = true;
   StartThreads();
   
} // DoRecordListBox()

//=============================================================================
//
//=============================================================================

void HdRecord::DoTimedSplitBox()
{
   TimedSplitBox * timeSplitBox;

   if(m_p_MainDialog != NULL)
   { 
      timeSplitBox = new TimedSplitBox(m_p_MainDialog,this);
   
      timeSplitBox->exec();
   
      delete timeSplitBox;
   }
   
} // DoTimedSplitBox()

//=============================================================================
//
//=============================================================================

void HdRecord::DoTimedSplitStart()
{

   // reset the stop flag
   m_ProgramState.b_GlobalStop     = false;
   m_ProgramState.b_TimerStop    = false;   
   m_ProgramState.b_TimedSplitMode = true;
   StartThreads();
   
} // DoTimedSplitBox()

//=============================================================================
//
//=============================================================================

void HdRecord::DoRecordListTestData()
{
   m_p_RecordList .CreateTestDataset();

} // DoRecordListBox()



//===========================================================================
// eventfunction for the start button
//===========================================================================


void HdRecord::DoStartButton()
{

   // initalize some variables
   m_ProgramState.b_TimerStop          = false;
   m_ProgramState.b_GlobalStop         = false;
   m_ProgramState.b_TriggerSplitting   = false;
   l_TotalBytesWritten  = 0;
   l_SingleBytesWritten = 0;

   // refresh the volume of the line channel
   SetInputMixer();


   // set input field sensistivity
   SetSensitivity();

   // initialze the file writer with the current settings
   PrepareFileWriterForRecording();

   // start all recording threads
   StartThreads();
} // DoStartButton()

//===========================================================================
// eventfunction for the stop button
//===========================================================================


void HdRecord::DoStopButton()
{

   if(m_ProgramState.b_TimerMode)
   {
      
      m_ProgramState.b_TimerMode = false;

      // set global stop
      m_ProgramState.b_GlobalStop = true;

      // tell the audio recorder the stop condition
      if(theAudioRecorder != NULL)
      {
         theAudioRecorder->StopRecording = true;
      }

      // stop the timer thread
      m_ProgramState.b_TimerStop = true;
      
      // stop all running threads -- but only if recording
      if( m_ProgramState.b_ControllerIsRecording )
      {
         // if just recording
         StopThreads();
         m_ProgramState.b_ControllerIsRecording = false;
      }

      pthread_cancel(p_ControllerThread);
   
      // clean up the status line
      ShowTimerStatus(0,0);

      // set input field sensistivity
      SetSensitivity();   
   }

   else if(m_ProgramState.b_TimedSplitMode)
   {
      
      m_ProgramState.b_TimedSplitMode = false;

      // set global stop
      m_ProgramState.b_GlobalStop = true;

      // tell the audio recorder the stop condition
      if(theAudioRecorder != NULL)
      {
         theAudioRecorder->StopRecording = true;
      }
      
      // stop the timer thread
      m_ProgramState.b_TimerStop = true;   
      
      // stop all running threads -- but only if recording
      if( m_ProgramState.b_ControllerIsRecording )
      {
         // if just recording
         StopThreads();
         m_ProgramState.b_ControllerIsRecording = false;
      }

      pthread_cancel(p_ControllerThread);

      // clean up the status line
      ShowTimerStatus(0,0);

      // set input field sensistivity
      SetSensitivity();   
   }
   else // regular mode
   {
      // set global stop
      m_ProgramState.b_GlobalStop = true;

      // tell the audio recorder the stop condition
      if(theAudioRecorder != NULL)
      {
         theAudioRecorder->StopRecording = true;
      }
   
      // stop all running threads();
      StopThreads();
   
      // set input field sensistivity
      SetSensitivity();
   }
   
   
} // DoStopButton()

//===========================================================================
// eventfunction for the pause button
//===========================================================================


void HdRecord::DoPauseButton()
{
   // are we already in a pause
   if(m_ProgramState.b_Paused)
   {
      // finish the pause
      m_ProgramState.b_Paused=false;
      if(m_p_MainDialog != NULL)
      {
         m_p_MainDialog->SetPauseButtonText("Pause");
      }
   }
   else
   {
      // begin the pause
      m_ProgramState.b_Paused=true;
      if(m_p_MainDialog != NULL)
      {
         m_p_MainDialog->SetPauseButtonText("Pause active");
      }
   }
   
   
} // DoPauseButton()

//===========================================================================
// event for the split button, trigger the split in a variable
//===========================================================================


void HdRecord::DoSplitButton()
{

   // only enable the trigger flag
   m_ProgramState.b_TriggerSplitting = true;

}   // DoSplitpButton()

//===========================================================================
// terminate
//===========================================================================


void HdRecord::Beenden()
{
   // dont write basedata in command line mode
   if( ! m_ProgramState.b_CommandLineMode)
   {
      // before exiting write the basis data
      BasicDataWrite(this);
   }

   exit(0);
}  // Beenden()


//===========================================================================
// start the recording, the writing and the display-update thread
//===========================================================================

void HdRecord::StartThreads()
{


   if(m_ProgramState.b_TimerMode)
   {
      // in timer mode only start the controller.
      // it will start the other threads, if the time has come.
      pthread_attr_init(&ControllerAttribut);
      pthread_cond_init(&p_ControllerCondition,NULL);
      
      pthread_create(&p_ControllerThread,
                     &ControllerAttribut,
                     HdRecord::ControllerThreadCB,
                     this);
   }
   else if(m_ProgramState.b_TimedSplitMode)
   {
      // in timer mode only start the controller.
      // it will start the other threads, if the time has come.
      pthread_attr_init(&ControllerAttribut);
      pthread_cond_init(&p_ControllerCondition,NULL);
      
      pthread_create(&p_ControllerThread,
                     &ControllerAttribut,
                     HdRecord::ControllerThreadCB,
                     this);
   }
   else // regular mode -- not the timer
   {
      // set up the semaphore
      sem_init(&TimerSemaphore,
               0,                  // process share
               0                   // initital value
               );

      // init thread attributes
      pthread_attr_init(&RecorderAttribut);
      pthread_attr_init(&WriterAttribut);
      pthread_attr_init(&TimerAttribut);


      // create save block structure
      theSaveBlocks = new SaveBlockArray();
   
      // start the 3 threads

      pthread_create(&p_RecorderThread,
                     &RecorderAttribut,
                     HdRecord::RecorderThreadCB,
                     this);

      pthread_create(&p_WriterThread,
                     &WriterAttribut,
                     HdRecord::WriterThreadCB,
                     this);

      pthread_create(&p_TimerThread,
                     &TimerAttribut,
                     HdRecord::TimerThreadCB,
                     this);
   
   } // else regular mode

   

} // StartThreads()
   

//===========================================================================
// wait for all thread to finish.
//===========================================================================

void HdRecord::StopThreads()
{
   m_ProgramState.b_TimerStop = true;
   m_ProgramState.b_GlobalStop =true;
   // wait for finish of all threads
   pthread_join(p_RecorderThread,NULL);
   pthread_join(p_WriterThread,NULL);
   pthread_join(p_TimerThread,NULL);

   // free the saveblock structure
   if( theSaveBlocks != NULL)
   {
      delete theSaveBlocks;      
      theSaveBlocks = NULL;
   }

   // destroy the timer semaphore
   sem_destroy(&TimerSemaphore)   ;


} // StopThreads()

//===========================================================================
// set the line input in the input mixed to maximum
//===========================================================================



void HdRecord::SetInputMixer()
{
   // mixer available ?
   if(m_p_Mixer != NULL)
   {
      // call the mixerdevice object
      m_p_Mixer-> SetInputMixer(m_i_MixerInputDevice,
                                m_i_SampleLevel,
                                m_i_SampleLevelMaster);    
   }
   
} // SetInputMixer()

//===========================================================================
// display the about box
//===========================================================================

void HdRecord::DoAboutBox()
{
   AboutBox * aboutBox;
 
   if(m_p_MainDialog != NULL)
   {
      aboutBox = new AboutBox(m_p_MainDialog);
   
      aboutBox->exec();

      delete aboutBox;
   }
} // DoAboutBox()


//===========================================================================
// eventfunction if the mode is recording mode is changed 
//===========================================================================


void HdRecord::DoModeChanged(int mode)
{

   if(mode ==0)
   {
      e_RecordMode = ModeMp3;
   }

   if(mode ==1)
   {
      e_RecordMode = ModeWave;
   }
   if(mode ==2)
   {
      e_RecordMode = ModeOgg;
   }

} // DoModeChanged()


//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void HdRecord::DoSliderValueChanged(int i_SliderValue)
{
   // remember the level
   m_i_SampleLevel = i_SliderValue;

   // set the volume of the line channel
   SetInputMixer();
   
} // DoSliderValueChanged()

//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void HdRecord::DoMasterSliderValueChanged(int i_SliderValue)
{
   // remember the level
   m_i_SampleLevelMaster = i_SliderValue;

   // set the volume of the line channel
   SetInputMixer();
   
} // DoSliderValueChanged()

//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void HdRecord::DoTimeoutSliderValueChanged(int i_SliderValue)
{
  
   // remember the level
   // * 3 -- do a simple conversion to seconds
   m_i_AnalyzeTimeout = i_SliderValue ;

   
} // DoTimeoutSliderValueChanged()

//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void HdRecord::DoDetectLevelSliderValueChanged(int i_SliderValue)
{
   // remember the level
   m_i_AnalyzeLevel = i_SliderValue;

   
} // DoDetectLevelSliderValueChanged()

//===========================================================================
//
//===========================================================================

void HdRecord::DoAnalyzeButton(bool enabled)
{

   if(enabled)
   {
      m_ProgramState.b_AnalyzeMode = true;
      m_i_AnalyzeStatus = ModeAnalyzeWait;
   }
   else
   {
      m_ProgramState.b_AnalyzeMode = false;
   }


} // DoAnalyzeButton()

//===========================================================================
// show the current analye state in the output field
//===========================================================================

void HdRecord::SetAnalyzeState()
{
   if(m_p_MainDialog == NULL)
   {
      // no dialog -> no output
      return;
   }

   // if analyze mode and recording
   if(     m_ProgramState.b_AnalyzeMode
      && ! m_ProgramState.b_GlobalStop )
   {
      
      if(m_i_AnalyzeStatus == ModeAnalyzeWait)
      {
          m_p_MainDialog->SetAnalyzeStateLabel("Wait for Signal");
          m_p_MainDialog->SetAnalyzeColor(AnalyzeColor_Wait);
      }
      else if(m_i_AnalyzeStatus == ModeAnalyzeRecord)
      {
          m_p_MainDialog->SetAnalyzeStateLabel("Recording");
          
          m_p_MainDialog->SetAnalyzeColor(AnalyzeColor_Record);
      }
      else if(m_i_AnalyzeStatus == ModeAnalyzeEmptyBlock)
      {
          m_p_MainDialog->SetAnalyzeStateLabel("Timing out");
          m_p_MainDialog->SetAnalyzeColor(AnalyzeColor_Timeout);
      }
   }
   else // not analyze mode
   {
        m_p_MainDialog->SetAnalyzeStateLabel("");
        m_p_MainDialog->SetAnalyzeColor(AnalyzeColor_Default);
   }

} // SetAnalyzeState()


//===========================================================================
//
//===========================================================================

void HdRecord::ShowTimerStatus(int i_Mode,time_t t_NextTime)
{
   struct tm  t_LocalTime;
   char       ca_TimeOutput[100];

   if(m_p_MainDialog == NULL)
   {
      // no dialog -> no output
      return;
   }
   
   if(i_Mode == 0)
   {
      // clean up status line
      m_p_MainDialog->SetStatusLabel("");
   }
   else if(i_Mode == 1)
   {
      // show waiting for record text
       memcpy(&t_LocalTime,
              localtime(&t_NextTime),
              sizeof(struct tm));

      strftime(ca_TimeOutput,sizeof(ca_TimeOutput)-1,
                "Waiting for record at %Y-%m-%d %H:%M:%S",
                &t_LocalTime);
      m_p_MainDialog->SetStatusLabel(ca_TimeOutput);

   }
   else if(i_Mode == 2)
   {
      // show waiting for record text
       memcpy(&t_LocalTime,
              localtime(&t_NextTime),
              sizeof(struct tm));

      strftime(ca_TimeOutput,sizeof(ca_TimeOutput)-1,
                "Recording until %Y-%m-%d %H:%M:%S",
                &t_LocalTime);
      m_p_MainDialog->SetStatusLabel(ca_TimeOutput);

   }
   else if(i_Mode == 3)
   {
      // show waiting for record text
       memcpy(&t_LocalTime,
              localtime(&t_NextTime),
              sizeof(struct tm));

      strftime(ca_TimeOutput,sizeof(ca_TimeOutput)-1,
                "Making split at %Y-%m-%d %H:%M:%S",
                &t_LocalTime);
      m_p_MainDialog->SetStatusLabel(ca_TimeOutput);

   }

   // redraw the output
   FlushDisplay();

   // redraw the output
   FlushDisplay();
   
} // ShowTimerStatus()

//===========================================================================
// Adjust the display values to the internal ones
//===========================================================================

void HdRecord::SetDisplayFromValues()
{
   if(m_p_MainDialog == NULL)
   {
      return;
   }
   m_p_MainDialog->SetRecordMode(e_RecordMode);

   m_p_MainDialog->SetSampleLevel(m_i_SampleLevel);

   m_p_MainDialog->SetSampleLevelMaster(m_i_SampleLevelMaster);

} // SetDisplayFromValues()

//===========================================================================
//
//===========================================================================

void HdRecord::SetSensitivity()
{
   if(m_p_MainDialog == NULL)
   {
      return;
   }
   m_p_MainDialog->SetSensitivity(m_ProgramState);


} // SetSensitivity()

//===========================================================================
//
//===========================================================================

bool HdRecord::TestDevice(const char *cp_Device,bool b_TestSpeed)
{
   bool b_Result;
   int  fd;


   b_Result = true;
   // open the mixer device
   fd=open(cp_Device, O_RDWR | O_NONBLOCK );

   if(fd<0)
   {

      if(m_ProgramState.b_CommandLineMode)
      {
         // command line mode
         fprintf(stderr,"Could not access the Device %s\n",cp_Device);
         fprintf(stderr,"HdRecord can not record!\n");

      }
      else // GUI mode
      {
         QMessageBox::critical( 0, "KHdRecord",
                           QString("Could not access the Device ") +
                           cp_Device +
                           "\n\nKHdRecord can not record!" );
      }

      b_Result = false;
   }
   else
   {
      ::close(fd);
      
      // if testspeed then detext the possible samplespeed
      if( b_TestSpeed )
      {
         AudioRecorder *pc_TestRecorder;
         pc_TestRecorder = new AudioRecorder(cp_Device);


         if(pc_TestRecorder != NULL)
         {

            pc_TestRecorder -> SetSpeed(m_i_UsedSampleRate);

            delete pc_TestRecorder;
         }

      } // if testspeed
   }

   return b_Result;

} // TestDevices

//===========================================================================
//
//===========================================================================

void HdRecord::PrepareFileWriterForRecording()
{
   // set the file format
   m_p_FileWriter ->SetDestination(e_RecordMode);
   // set the bitrate
   m_p_FileWriter ->SetEncodeRate(m_i_MP3Bitsize);

   // set the filename prefix
   m_p_FileWriter ->SetFileNamePrefix(m_qs_Basename);

   // switch on/off the filename date feature
   if(    m_ProgramState.b_AnalyzeMode
       || m_ProgramState.b_TimerMode
     )
   {
      m_p_FileWriter ->SetFileNameWithDate(true);
   }
   else // normal mode
   {
      if(e_FilenameChangeMode == FilenameChange_StartWithBasename)
      {
         m_p_FileWriter ->SetFileNameWithDate(false);
         m_p_FileWriter ->ResetFileCounter();
      }
      if(e_FilenameChangeMode == FilenameChange_UseDate)
      {
          m_p_FileWriter ->SetFileNameWithDate(true);
      }
      if(e_FilenameChangeMode == FilenameChange_AlwaysIncrement)
      {
         m_p_FileWriter ->SetFileNameWithDate(false);
         m_p_FileWriter ->NextFile();
      }

   } // else normal mode



} // PrepareFileWriterForRecording()

//===========================================================================
//
//===========================================================================

int HdRecord::OpenFile()
{
   int i_OpenSuccess;

   m_p_FileWriter->SetUsedSampleRate(m_i_UsedSampleRate);

   // open the output file
   i_OpenSuccess = m_p_FileWriter->OpenFile();
   if( i_OpenSuccess <0)
   {
      // open failure

      // indicate a stop
      m_ProgramState.b_GlobalStop         = true;

      return -1;
   }

   // trigger, that there is a new filename
   m_ProgramState.b_TotalFileNameChanged = true;

   // new file has zero bytes
   l_SingleBytesWritten =0;

   // everything is ok
   return 0;

} // OpenFile

//===========================================================================
//
//===========================================================================

int HdRecord::WriteBlock(SaveBlock * pc_Saveblock)
{
   // paused ??
   if( m_ProgramState.b_Paused )
   {
      // then do nothing
      return 0;
   }


   // increase the byte counter
   l_TotalBytesWritten   += pc_Saveblock->BufferFilled;
   l_SingleBytesWritten  += pc_Saveblock->BufferFilled;

   // write the block to file
   m_p_FileWriter->WriteBlock(pc_Saveblock);


   return 0;
    
} // WriteBlock()

//===========================================================================
//
//===========================================================================

void HdRecord::SetMainDialog(MainDialog *pc_MainDialog)
{
   m_p_MainDialog = pc_MainDialog;
   if(m_p_MainDialog != NULL)
   {
      InitializeDisplay();
   }
} // SetMainDialog()

//===========================================================================
// transfer all settings initital to the display
//===========================================================================

void HdRecord::InitializeDisplay()
{
   if( m_p_MainDialog == NULL)
   {
      return;
   }

   m_p_MainDialog->SetRecordMode(e_RecordMode);
   m_p_MainDialog->SetAnalyzeLevel(m_i_AnalyzeLevel);
   m_p_MainDialog->SetAnalyzeTimeout(m_i_AnalyzeTimeout);
   m_p_MainDialog->SetAnalyzeMode(false);
   m_p_MainDialog->SetAnalyzeStateLabel("");
   m_p_MainDialog->SetAnalyzeColor(AnalyzeColor_Default);
   m_p_MainDialog->SetSampleLevel(m_i_SampleLevel);
   m_p_MainDialog->SetSampleLevelMaster(m_i_SampleLevelMaster);
   m_p_MainDialog->SetTotalTime(0);
   m_p_MainDialog->SetSingleTime(0);
   m_p_MainDialog->SetRecordingFilename("");
   m_p_MainDialog->SetFilename(m_qs_Basename);
   m_p_MainDialog->SetPauseButtonText("Pause");
   m_p_MainDialog->SetStatusLabel("");

   m_p_MainDialog->SetSensitivity(m_ProgramState);

} // InitializeDisplay()

//===========================================================================
//
//===========================================================================

void HdRecord::FlushDisplay()
{
   if(m_p_MainDialog != NULL)
   {
      m_p_MainDialog->FlushDisplay();
   }
} // FlushDisplay()

//===========================================================================
//
//===========================================================================

bool HdRecord::CheckDevices()
{
   // check, wether we can access the devices
   if( strcmp( m_qs_SoundDevice, "artsd" ) == 0 )
   {
	m_ProgramState.b_DeviceAvailable = true;
        m_i_UsedSampleRate = 44100;
   }
   else
   {
      m_ProgramState.b_DeviceAvailable  = TestDevice(m_qs_SoundDevice,true);
   }
   m_ProgramState.b_DeviceAvailable &= TestDevice(m_qs_MixerDevice,false);

   if(m_p_MainDialog != NULL)
   {
      SetSensitivity();
   }
   return m_ProgramState.b_DeviceAvailable;
} // CheckDevices()

//===========================================================================
//
//===========================================================================

int HdRecord::CreateRecordListFromCommand(CommandLineParameter *pc_CommandLineParameter)
{
   int i_Result;

   i_Result =0;


   // first disable all
   m_ProgramState.b_ServerMode = false;
   m_ProgramState.b_RemoteMode = false;


   if(pc_CommandLineParameter == NULL)
   {
      return -1;
   }

   if( pc_CommandLineParameter->m_b_ServerMode)
   {
      m_ProgramState.b_ServerMode = true;
      m_p_FileWriter->SetServerMode(true);
      m_i_MP3Bitsize = pc_CommandLineParameter-> m_i_Bitrate;
      m_i_SampleLevel = pc_CommandLineParameter-> m_i_InputLevel;
      m_i_SampleLevelMaster = pc_CommandLineParameter-> m_i_InputLevelMaster;
      m_i_MixerInputDevice = pc_CommandLineParameter->m_i_InputDevice;
      m_p_Mixer->SetInputMixer(m_i_MixerInputDevice,
                               m_i_SampleLevel,
                               m_i_SampleLevelMaster);
      m_p_FileWriter->SetEncodeRate(pc_CommandLineParameter-> m_i_Bitrate);

      m_i_ServerPort = pc_CommandLineParameter->m_i_ServerPort;
      if( m_p_FileWriter != NULL)
      {
         m_p_FileWriter->SetServerPort(m_i_ServerPort);
      }
   }

   if( pc_CommandLineParameter->m_b_RemoteMode)
   {
      m_ProgramState.b_RemoteMode = true;
      m_i_MP3Bitsize = pc_CommandLineParameter-> m_i_Bitrate;
      m_i_SampleLevel = pc_CommandLineParameter-> m_i_InputLevel;
      m_i_SampleLevelMaster = pc_CommandLineParameter-> m_i_InputLevelMaster;
      m_i_MixerInputDevice = pc_CommandLineParameter->m_i_InputDevice;
      m_p_Mixer->SetInputMixer(m_i_MixerInputDevice,
                               m_i_SampleLevel,
                               m_i_SampleLevelMaster);
      m_p_FileWriter->SetEncodeRate(pc_CommandLineParameter-> m_i_Bitrate);

      m_i_ServerPort = pc_CommandLineParameter->m_i_ServerPort;
   }

   if( pc_CommandLineParameter->m_b_AnalyzeMode)
   {
      m_ProgramState.b_AnalyzeMode = true;
      m_ProgramState.b_ServerMode  = false;
      m_p_FileWriter->SetServerMode(false);
      m_i_MP3Bitsize        = pc_CommandLineParameter-> m_i_Bitrate;
      m_i_SampleLevel       = pc_CommandLineParameter-> m_i_InputLevel;
      m_i_SampleLevelMaster = pc_CommandLineParameter-> m_i_InputLevelMaster;
      m_i_MixerInputDevice  = pc_CommandLineParameter-> m_i_InputDevice;
      m_p_Mixer->SetInputMixer(m_i_MixerInputDevice,
                               m_i_SampleLevel,
                               m_i_SampleLevelMaster);
      m_p_FileWriter->SetEncodeRate(pc_CommandLineParameter-> m_i_Bitrate);

      m_i_AnalyzeLevel    = pc_CommandLineParameter->m_i_AnalyzeLevel;
      m_i_AnalyzeTimeout  = pc_CommandLineParameter->m_i_AnalyzeTimeout;
      m_p_FileWriter->SetFileNameWithDate(true);
      e_RecordMode        = pc_CommandLineParameter-> m_e_RecordMode;

   }


   if( pc_CommandLineParameter->m_b_TestMode)
   {
      fprintf(stderr,"Creating test dataset\n");
      m_p_RecordList.CreateTestDataset();
   }
   else if( ! pc_CommandLineParameter->m_qs_RecorderFile.isEmpty() )
   {
      // read in the record description file
      i_Result = m_p_RecordList.ReadFromFile(pc_CommandLineParameter->m_qs_RecorderFile);
   }
   else // no record file
   {
      // create a record list from the other parameters
      RecordDescription *pc_NewRecord;
      pc_NewRecord = new RecordDescription;

      if(pc_NewRecord != NULL)
      {
         pc_NewRecord->m_t_StartTime      = pc_CommandLineParameter-> m_t_StartTime;
         pc_NewRecord->m_t_EndTime        = pc_CommandLineParameter-> m_t_StopTime;
         pc_NewRecord->m_i_InputDevice    = pc_CommandLineParameter-> m_i_InputDevice;
         pc_NewRecord->m_i_RecordingLevel = pc_CommandLineParameter-> m_i_InputLevel;
         pc_NewRecord->m_i_RecordingLevelMaster = pc_CommandLineParameter-> m_i_InputLevelMaster;
         pc_NewRecord->m_i_MP3Rate        = pc_CommandLineParameter-> m_i_Bitrate;
	 pc_NewRecord->m_e_RecordingFormat= pc_CommandLineParameter-> m_e_RecordMode;
         // add the record into the global list
         m_p_RecordList.AddRecord(pc_NewRecord);
     

      } // if record there
      
   } // else no record file
   
   // set the other global parameters
   
   // set the output file
   if( ! pc_CommandLineParameter->m_qs_Outputname.isEmpty() )
   {
      m_qs_Basename = pc_CommandLineParameter->m_qs_Outputname;
   }

   // change the sound device
   if( ! pc_CommandLineParameter->m_qs_SoundDevice.isEmpty() )
   {
      m_qs_SoundDevice = pc_CommandLineParameter->m_qs_SoundDevice;
      if(theAudioRecorder != NULL)
      {
         theAudioRecorder->SetDevice(m_qs_SoundDevice);
      }
   }

   // change the mixer device
   if( ! pc_CommandLineParameter->m_qs_MixerDevice.isEmpty() )
   {
      m_qs_MixerDevice = pc_CommandLineParameter->m_qs_MixerDevice;
      if(m_p_Mixer != NULL)
      {
         m_p_Mixer->SetMixerDevice(m_qs_MixerDevice);
      }
   }

   // everything fine
   return i_Result;  

} // CreateRecordListFromCommand()

//===========================================================================
//
//===========================================================================


void HdRecord::WaitForRecordingEnd()
{
   pthread_join(p_ControllerThread,NULL);
} // WaitForRecordingEnd()

//===========================================================================
//
//===========================================================================




void HdRecord::WaitForever()
{
   pthread_join(p_TimerThread,NULL);
} // WaitForever()

//===========================================================================
//
//===========================================================================













