//
#ifndef _HDRECORD_H_LOADED
#define _HDRECORD_H_LOADED


#include <stdio.h>
#include <unistd.h>
#include <pthread.h>
#include <semaphore.h>

#include <qwidget.h>

#include <qapplication.h>
#include <qmainwindow.h>

#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qscrollbar.h>
#include <qlabel.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qradiobutton.h>

extern "C"
{
#include "lame/lame.h"
}

#include "basedef.h"

#include "saveblock.h"
#include "audiorecorder.h"
#include "basisdaten.h"
#include "recdescr.h"

#include "mixer.h"
#include "filewriter.h"
#include "maindialog.h"
#include "commandline.h"


enum AnalyzeMode
{
   ModeAnalyzeWait =1,
   ModeAnalyzeRecord,
   ModeAnalyzeEmptyBlock
};

enum RemoteModeState
{
   RemoteWait=0,
   RemoteStart,
   RemoteStop,
   RemoteSplit
};


class HdRecord 
{
public:
   HdRecord(QWidget *parent,
                  QApplication *mainApplication);

   ~HdRecord();



   public:  		//-----------------public functions 
   void DoAboutBox();

   void DoRecordListBox();
   void DoRecordListStart();
   void DoRecordListTestData();

   void DoTimedSplitBox();
   void DoTimedSplitStart();

   void Beenden();

   void DoStartButton();
   void DoStopButton();
   void DoPauseButton();
   void DoSplitButton();

   void DoAnalyzeButton(bool);

   void DoModeChanged(int);

   void DoSliderValueChanged(int);
   void DoMasterSliderValueChanged(int);
   void DoTimeoutSliderValueChanged(int);
   void DoDetectLevelSliderValueChanged(int);
   void SetInputMixer();
   
   void SetMainDialog(MainDialog *pc_MainDialog);
   bool CheckDevices();

   int CreateRecordListFromCommand(CommandLineParameter   *pc_CommandLineParameter);
   void WaitForRecordingEnd();
   void WaitForever();


public:			// --------------- public variables
   RecordDescriptionList m_p_RecordList;
   FilenameChangeMode    e_FilenameChangeMode;
   RemoteModeState       m_i_RemoteState;   
   SaveBlockArray       *theSaveBlocks;
   BasisDatenType       *m_BasicData;
   QApplication         *mainApplication;
   ProgramState          m_ProgramState;
   AudioRecorder        *theAudioRecorder;
   MixerDevice          *m_p_Mixer;
   RecordMode            e_RecordMode;
   FileWriter           *m_p_FileWriter;
   MainDialog           *m_p_MainDialog;
   QString               m_qs_Basename;
   QString               m_qs_SoundDevice;
   QString               m_qs_MixerDevice;
   
   
   int                   m_i_MP3Bitsize;
   int                   m_i_MixerInputDevice;
   int                   m_i_SampleLevel;
   int                   m_i_SampleLevelMaster;
   int                   m_i_AnalyzeLevel;
   int                   m_i_AnalyzeTimeout;
   int                   m_i_TimedSplitSeconds;
   int                   m_i_ServerPort;
   
   int                   m_i_UsedSampleRate;

   float                 m_f_AnalyzeNoiseLevel;



private:

   // ------------------- regular functions
   void FlushDisplay();
   void InitializeDisplay();

   void ShowTimerStatus(int i_Mode,time_t t_NextTime);
   void StartThreads();
   void StopThreads();

   void UpdateRecordingDisplay();
   void SetDisplayFromValues();
   void SetSensitivity();

   void SetAnalyzeState();
   bool TestDevice(const char *cp_Device,bool TestSpeed=false);

   void PrepareFileWriterForRecording();
   int  OpenFile();
   int  WriteBlock(SaveBlock * pc_Saveblock);


   // ---------------------------stuff needed for threads
   pthread_attr_t WriterAttribut;
   pthread_t      p_WriterThread;
   pthread_attr_t RecorderAttribut;
   pthread_t      p_RecorderThread;

   pthread_attr_t TimerAttribut;
   pthread_t      p_TimerThread;

   pthread_attr_t ControllerAttribut;
   pthread_t      p_ControllerThread;

   pthread_attr_t RemoteListenerAttribut;
   pthread_t      p_RemoteListenerThread;

   pthread_cond_t p_ControllerCondition;
   pthread_mutex_t p_ControllerMutex;
   struct timespec t_ControllerTimespec;

   sem_t          TimerSemaphore;
  
   static void * RecorderThreadCB(void *pc_HdRecorder);
   static void * WriterThreadCB(void *pc_HdRecorder);
   static void * TimerThreadCB(void *pc_HdRecorder);
   static void * ControllerThreadCB(void *pc_HdRecorder);
   static void * RemoteListenerThreadCB(void *pc_HdRecorder);
   
   
   void * RecorderThread();
   void * RecorderThreadTimer();
   void * TimerThread();
   void * TimerThreadTimer();
   void * WriterThread();
   void * WriterThreadAnalyze();
   void * WriterThreadTimer();
   void * ControllerThreadTimer();
   void * ControllerThreadTimedSplit();
   void * ControllerThreadServer();
   void * ControllerThreadRemote();
   void * RemoteListenerThread();
   

   // ------------------ private regular variables
   unsigned short us_SampleLevel[SAMPLE_AREA_VALUES];
  
   
   unsigned long l_TotalBytesWritten;
   unsigned long l_SingleBytesWritten;

   // variables needed for the mp3 encoding

   AnalyzeMode m_i_AnalyzeStatus;
   int m_i_AnalyzeNumberOfEmptyBlocks;
   SaveBlock *m_p_PreviousSaveBlock;
   
}; // class hdrecord

#endif // _HDRECORD_H_LOADED













