// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// connection camera to basisdata


#include <stdlib.h>

#include <qapplication.h>
#include <qcursor.h>


#include "basedef.h"
#include "hdrecord_basis.h"
#include "hdrecord.h"
#include "basisdaten.h"

/*--------------------------------------------------------------------------*/
//
// constant names for the base settings variables
//
#define BASISDATENNAME              ".KHdRecordrc"
#define BASISDATAINPUTDEVICE        "InputDevice"
#define BASISDATAMP3RATE            "MP3Bitrate"
#define BASISDATASAMPLELEVEL        "SampleLevel"
#define BASISDATASAMPLELEVELMASTER  "SampleLevelMaster"
#define BASISDATAANALYZELEVEL       "AnalyzeLevel"
#define BASISDATAANALYZETIMEOUT     "AnalyzeTimeout"
#define BASISDATAFILENAMECHANGEMODE "FilenameChangeMode"
#define BASISDATAFILENAMEPREFIX     "FilenamePrefix"
#define BASISDATAFILENAMECOUNTER    "FilenameCounter"
#define BASISDATATIMEDSPLITSECONDS  "TimedSplitSeconds"
#define BASISDATASOUNDDEVICE        "SoundDevice"
#define BASISDATAMIXERDEVICE        "MixerDevice"
#define BASISDATARECORDMODE         "RecordMode"


/*--------------------------------------------------------------------------*/

static char *BasicDataName()
{
   static char exportName[500];

   char        *envReturn;
   
   
   envReturn =getenv("HOME");

   if(envReturn ==NULL)
   {
      strcpy(exportName,BASISDATENNAME);
   }
   else
   {
      strcpy(exportName,envReturn);
      strcat(exportName,"/");
      strcat(exportName,BASISDATENNAME);
   }

   return exportName;

} // BasicDataName()

//-------------------------------------------------------------------

void BasicDataRead(HdRecord *p_Application)
{
   BasisDatenType *p_Basic;
   char            sz_InputDevice [500];
   char            sz_Filename    [1500];
   bool            gelungen;
   int             i_Value;
   
   p_Basic =  p_Application->m_BasicData;
   if(p_Basic == NULL)
   {
      return;
   }

   gelungen = p_Basic->BasisDatenLesen(BasicDataName());
   if(!gelungen)
      return;
      
   gelungen = p_Basic->BasisDatenReadString(
   		BASISDATAINPUTDEVICE,
   		sz_InputDevice);
   if(gelungen)
   {
      if(strcmp(sz_InputDevice,"Line") ==0)
      {
         p_Application->m_i_MixerInputDevice = SOUND_MIXER_LINE;
      }
      else if(strcmp(sz_InputDevice,"Microphone") ==0)
      {
         p_Application->m_i_MixerInputDevice = SOUND_MIXER_MIC;
      }
      else if(strcmp(sz_InputDevice,"CD") ==0)
      {
         p_Application->m_i_MixerInputDevice = SOUND_MIXER_CD;
      }
      else
      {
         p_Application->m_i_MixerInputDevice = SOUND_MIXER_LINE;
      }
   }
   else
   {
      p_Application->m_i_MixerInputDevice = SOUND_MIXER_LINE;
   }

   p_Application->m_i_MP3Bitsize = 128;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATAMP3RATE,
   		&p_Application->m_i_MP3Bitsize);
   if(   p_Application->m_i_MP3Bitsize != 40
      && p_Application->m_i_MP3Bitsize != 128
      && p_Application->m_i_MP3Bitsize != 160
      && p_Application->m_i_MP3Bitsize != 192
      && p_Application->m_i_MP3Bitsize != 256
      && p_Application->m_i_MP3Bitsize != 320
      && p_Application->m_i_MP3Bitsize != 16
      && p_Application->m_i_MP3Bitsize != 8
     )
   {
      p_Application->m_i_MP3Bitsize = 128;
   }

   p_Application->m_i_SampleLevel = 70;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATASAMPLELEVEL,
   		&p_Application->m_i_SampleLevel);
   if(   p_Application->m_i_SampleLevel < 0
      || p_Application->m_i_SampleLevel > 100 
     )
   {
      p_Application->m_i_SampleLevel = 70;
   }

   p_Application->m_i_SampleLevelMaster = 50;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATASAMPLELEVELMASTER,
   		&p_Application->m_i_SampleLevelMaster);
   if(   p_Application->m_i_SampleLevelMaster < 0
      || p_Application->m_i_SampleLevelMaster > 100 
     )
   {
      p_Application->m_i_SampleLevelMaster = 70;
   }

   p_Application->m_i_AnalyzeLevel = 5;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATAANALYZELEVEL,
   		&p_Application->m_i_AnalyzeLevel);
   if(   p_Application->m_i_AnalyzeLevel < 0
      || p_Application->m_i_AnalyzeLevel > 10 
     )
   {
      p_Application->m_i_AnalyzeLevel = 5;
   }
   
   p_Application->m_i_AnalyzeTimeout = 5;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATAANALYZETIMEOUT,
   		&p_Application->m_i_AnalyzeTimeout);
   if(   p_Application->m_i_AnalyzeTimeout < 0
      || p_Application->m_i_AnalyzeTimeout > 10 
     )
   {
      p_Application->m_i_AnalyzeTimeout = 5;
   }

   p_Application->e_FilenameChangeMode = FilenameChange_StartWithBasename;

   i_Value =0;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATAFILENAMECHANGEMODE,
   		& i_Value );
   // convert back from integer to enum
   if(gelungen)
   {
      if(i_Value == 0 )
      {
         p_Application-> e_FilenameChangeMode = FilenameChange_StartWithBasename;
      }
      else if (i_Value == 1 )
      {
         p_Application-> e_FilenameChangeMode = FilenameChange_AlwaysIncrement;
      }
      else if (i_Value == 2 )
      {
         p_Application-> e_FilenameChangeMode = FilenameChange_UseDate;
      }
   }


   p_Application->m_qs_Basename = "test";
   gelungen = p_Basic->BasisDatenReadString(
   		BASISDATAFILENAMEPREFIX,
   		sz_Filename );
   if(gelungen)
   {
      p_Application->m_qs_Basename = sz_Filename;
   }

   i_Value=0;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATAFILENAMECOUNTER,
   		& i_Value );
   if(gelungen)
   {
      p_Application->m_p_FileWriter->SetFileCounter(i_Value);
   }
   else
   {
      p_Application->m_p_FileWriter->ResetFileCounter();
   }

   p_Application->m_i_TimedSplitSeconds = 60;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATATIMEDSPLITSECONDS,
   		&p_Application->m_i_TimedSplitSeconds);
   if(   p_Application->m_i_TimedSplitSeconds < 0
     )
   {
      p_Application->m_i_AnalyzeLevel = 60;
   }

   gelungen = p_Basic->BasisDatenReadString(
   		BASISDATASOUNDDEVICE,
   		sz_Filename );
   if(gelungen)
   {
      p_Application->m_qs_SoundDevice = sz_Filename;
   }

   gelungen = p_Basic->BasisDatenReadString(
   		BASISDATAMIXERDEVICE,
   		sz_Filename );
   if(gelungen)
   {
      p_Application->m_qs_MixerDevice = sz_Filename;
   }

   p_Application->e_RecordMode = ModeMp3;
   gelungen = p_Basic->BasisDatenReadInt(
   		BASISDATARECORDMODE,
   		&i_Value);
   if(gelungen)
   {
      p_Application->e_RecordMode = (RecordMode) i_Value;
   }

} // BasicDataRead

//-------------------------------------------------------------------

void BasicDataWrite(HdRecord  *p_Application)
{
   BasisDatenType *p_Basic;
   bool            gelungen;
   char            sz_InputDevice [500];
   char            sz_Filename[1500];

   p_Basic =  p_Application->m_BasicData;
   switch(p_Application->m_i_MixerInputDevice)
   {
      case SOUND_MIXER_LINE:
         strcpy(sz_InputDevice,"Line");
         break;
      case SOUND_MIXER_MIC:
         strcpy(sz_InputDevice,"Microphone");
         break;
      case SOUND_MIXER_CD:
         strcpy(sz_InputDevice,"CD");
         break;
      default:
         strcpy(sz_InputDevice,"Line");
         break;
   }

   gelungen = p_Basic->BasisDatenWriteString(
   		BASISDATAINPUTDEVICE,
   		sz_InputDevice);

   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATAMP3RATE,
   		p_Application-> m_i_MP3Bitsize);

   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATASAMPLELEVEL,
   		p_Application-> m_i_SampleLevel);

   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATASAMPLELEVELMASTER,
   		p_Application-> m_i_SampleLevelMaster);

   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATAANALYZELEVEL,
   		p_Application-> m_i_AnalyzeLevel);

   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATAANALYZETIMEOUT,
   		p_Application-> m_i_AnalyzeTimeout);

   {
      int i_Mode=0;
      if( p_Application-> e_FilenameChangeMode == FilenameChange_StartWithBasename)
      {
         i_Mode =0;
      }
      if( p_Application-> e_FilenameChangeMode == FilenameChange_AlwaysIncrement)
      {
         i_Mode =1;
      }
      if( p_Application-> e_FilenameChangeMode == FilenameChange_UseDate)
      {
         i_Mode =2;
      }
      gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATAFILENAMECHANGEMODE,
   		i_Mode);
   }

   strcpy(sz_Filename,p_Application-> m_qs_Basename);
   gelungen = p_Basic->BasisDatenWriteString(
   		BASISDATAFILENAMEPREFIX,
   		sz_Filename);

   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATAFILENAMECOUNTER,
   		p_Application-> m_p_FileWriter->GetFileCounter());

   gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATATIMEDSPLITSECONDS,
   		p_Application->m_i_TimedSplitSeconds);

   strcpy(sz_Filename,p_Application->m_qs_SoundDevice);
   gelungen = p_Basic->BasisDatenWriteString(
   		BASISDATASOUNDDEVICE,
   		sz_Filename);

   strcpy(sz_Filename,p_Application->m_qs_MixerDevice);
   gelungen = p_Basic->BasisDatenWriteString(
   		BASISDATAMIXERDEVICE,
   		sz_Filename);

  gelungen = p_Basic->BasisDatenWriteInt(
   		BASISDATARECORDMODE,
   		p_Application->e_RecordMode);


   gelungen = p_Basic->BasisDatenSchreiben(BasicDataName());

} // BasicDataWrite()



