// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// thread working modules



#include <qvbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qgrid.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qmessagebox.h>


#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>
#include <time.h>
#include <sys/socket.h>
#include <linux/in.h>


#include "hdrecord.h"
#include "saveblock.h"
#include "aboutbox.h"
#include "samplearea.h"
#include "settingsbox.h"
#include "hdrecord_basis.h"
#include "recordlistbox.h"
#include "mixer.h"
#include "timedsplitbox.h"


//===========================================================================
// recorder thread. get samples from the audio device
//===========================================================================


void * HdRecord::RemoteListenerThreadCB(void *v_pc_HdRecorder)
{
   HdRecord * pc_HdRecorder;


   pc_HdRecorder = (HdRecord *)  v_pc_HdRecorder;

   return pc_HdRecorder->RemoteListenerThread();

} // RecorderThreadCB()

//===========================================================================
//
//===========================================================================

void *  HdRecord::RemoteListenerThread()
{
   int i_ServerSocket;
   int i_BindResult;
   unsigned char  uc_Buffer[100];


   struct sockaddr_in ServerAddress;

      // create a socket 
      i_ServerSocket = socket(PF_INET,SOCK_DGRAM,0);

      if(i_ServerSocket == -1)
      {
         fprintf(stderr,"Invalid Server Socket\n");
      }

      ServerAddress.sin_family      = PF_INET;
      ServerAddress.sin_addr.s_addr = INADDR_ANY;
      ServerAddress.sin_port        = m_i_ServerPort;

   do
   {
      i_BindResult = bind(i_ServerSocket,
                       (struct sockaddr *) &ServerAddress,
                       sizeof(ServerAddress));   


      if(i_BindResult !=0)
      {
         fprintf(stderr,"Error: BindResult = %d\n",i_BindResult);
      }
   }
   while (i_BindResult == -1);

   while(true)
   {
      int i_ReceiveMessage;
      socklen_t xx;
      struct sockaddr Sender;

      memset(uc_Buffer,0,sizeof(uc_Buffer));
      

      i_ReceiveMessage = recvfrom(i_ServerSocket,
                           uc_Buffer,
                           4,
                           0,
			   (struct sockaddr *) &Sender,
                           &xx
			);
     

      switch(uc_Buffer[0])
      {
      case 1:
         m_i_RemoteState = RemoteStart;
         break;
      case 2:
         m_i_RemoteState = RemoteStop;
         break;
      case 3:
         m_i_RemoteState = RemoteSplit;
         break;
      default:
         m_i_RemoteState = RemoteWait;
         break;
      } 
      
   }

   close(i_ServerSocket);
   

   return NULL;
} // RemoteListenerThread()

//===========================================================================
// recorder thread. get samples from the audio device
//===========================================================================


void * HdRecord::RecorderThreadCB(void *v_pc_HdRecorder)
{
   HdRecord * pc_HdRecorder;


   pc_HdRecorder = (HdRecord *)  v_pc_HdRecorder;

   if(    pc_HdRecorder->m_ProgramState.b_TimerMode
       || pc_HdRecorder->m_ProgramState.b_TimedSplitMode
     )
   {
      return pc_HdRecorder->RecorderThreadTimer();
   }
   return pc_HdRecorder->RecorderThread();

} // RecorderThreadCB()

//===========================================================================
//
//===========================================================================

void * HdRecord::RecorderThread()
{

   theAudioRecorder = new AudioRecorder(m_qs_SoundDevice);


   if(theAudioRecorder == NULL)
   {
      m_ProgramState.b_GlobalStop=true;
      return NULL;
   }


   theAudioRecorder -> SetSpeed(m_i_UsedSampleRate);

   theAudioRecorder->SetSaveBlockArray(  theSaveBlocks);

   if(theAudioRecorder->GetError() != 0)
   {
      m_ProgramState.b_GlobalStop=true;
      theAudioRecorder->Abort();

      return NULL;
   }

   theAudioRecorder -> ReadAudio();

   delete theAudioRecorder;
   theAudioRecorder=NULL;

   return NULL;

} // RecorderThread()

//===========================================================================
// recorder thread, special variant for the timer mode
//===========================================================================

void * HdRecord::RecorderThreadTimer()
{

   // initialize the audio recorder
   theAudioRecorder = new AudioRecorder(m_qs_SoundDevice);
   if(theAudioRecorder == NULL)
   {
      m_ProgramState.b_GlobalStop=true;
      return NULL;
   }


   theAudioRecorder -> SetSpeed(m_i_UsedSampleRate);

   theAudioRecorder->SetSaveBlockArray(  theSaveBlocks);

   if(theAudioRecorder->GetError() != 0)
   {
      m_ProgramState.b_GlobalStop=true;
      theAudioRecorder->Abort();

      return NULL;
   }

   // lesen
   theAudioRecorder -> ReadAudio();

   delete theAudioRecorder;
   theAudioRecorder=NULL;

   return NULL;

} // RecorderThread()

//===========================================================================
// writer thread, write data into a file
//===========================================================================

void * HdRecord::WriterThread()
{
   SaveBlock *actualSaveBlock;
   bool       b_Finish;
   int        i_OpenSuccess;


   // termination flag to false
   b_Finish = false;

   // open the output file
   i_OpenSuccess = OpenFile();
   if( i_OpenSuccess <0)
   {
      // post semaphore, that block has been written
      sem_post(&TimerSemaphore);
      return NULL;
   }  
   
   while(!b_Finish)
   {
      // get the next block
      actualSaveBlock= theSaveBlocks -> GetNextWriteBlock();

      // no more left, then terminate recording
      if(actualSaveBlock == NULL)
      {
         b_Finish = true;
         break;
      }

      // prepare the sample level
      {
         unsigned char *pc_Buffer;
         int            i;
         int            i_between;
         
         i_between      = (SAVEBLOCKBUFFERSIZE /4 ) / SAMPLE_AREA_VALUES;
         
         pc_Buffer = (unsigned char *) actualSaveBlock->Buffer;
         for(i=0; i<SAMPLE_AREA_VALUES ; i++)
         {
            short left;
            short right;
            int   together;
            int   index;
            
            index = i * 4 * i_between;
            // get pointer to the buffer
            left =0;
            right=0;
            left  |= ((int)(pc_Buffer[index++])) ;
            left  |= ((int)(pc_Buffer[index++])) << 8;
            right |= ((int)(pc_Buffer[index++])) ;
            right |= ((int)(pc_Buffer[index++])) << 8;
            
            together =(left + right) /2;
            
            together += 0x8000;


            us_SampleLevel[i] = (unsigned short) (( together ) );
	 }
      }


      // is 'split the file' triggered?
      if( m_ProgramState.b_TriggerSplitting )
      {

         // reset the trigger
         m_ProgramState.b_TriggerSplitting = false;

         // finalize the header
         m_p_FileWriter->AdjustFileAtEnd();

         // increment the file number
         m_p_FileWriter->NextFile();

         // open the next file
         i_OpenSuccess = OpenFile();


      } // if trigger splitting

      actualSaveBlock->Analyze(m_i_AnalyzeLevel,
                               m_f_AnalyzeNoiseLevel);

      // write the block to hard disk
      WriteBlock(actualSaveBlock);

      // final block, then also finish
      if(actualSaveBlock->IsFinalBlock())
      {
         b_Finish = true;
      }

      // post semaphore, that block has been written
      sem_post(&TimerSemaphore);

   } // while not finish

   // finalize the header
   m_p_FileWriter->AdjustFileAtEnd();
   
   return NULL;
} // WriterThread()

//===========================================================================
// writer thread, write data into a file
//===========================================================================

void * HdRecord::WriterThreadTimer()
{
   SaveBlock *actualSaveBlock;
   bool       b_Finish;
   int        i_OpenSucess;
   
   
   b_Finish = false;
   
   // open the output file   
   i_OpenSucess = OpenFile();
   if( i_OpenSucess <0)
   {
      return NULL;
   }  

   while(!b_Finish)
   {
     
      
      actualSaveBlock= theSaveBlocks -> GetNextWriteBlock();
      
      if(actualSaveBlock == NULL)
      {
         b_Finish = true;
         break;
      }
      
      // prepare the sample level
      {
         unsigned char *pc_Buffer;
         int            i;
         int            i_between;
         
         i_between      = ( SAVEBLOCKBUFFERSIZE /4 ) / SAMPLE_AREA_VALUES;
         
         pc_Buffer = (unsigned char *) actualSaveBlock->Buffer;
         for(i=0; i<SAMPLE_AREA_VALUES ; i++)
         {
            short left;
            short right;
            int   together;
            int   index; 
            
            index = i * 4 * i_between;
            // get pointer to the buffer
            left =0;
            right=0;
            left  |= ((int)(pc_Buffer[index++])) ;
            left  |= ((int)(pc_Buffer[index++])) << 8;
            right |= ((int)(pc_Buffer[index++])) ;
            right |= ((int)(pc_Buffer[index++])) << 8;
            
            together =(left + right) /2;
            
            together += 0x8000;


            us_SampleLevel[i] = (unsigned short) (( together ) );
	 }
      }


      if( m_ProgramState.b_TriggerSplitting )
      {

         // reset the trigger
         m_ProgramState.b_TriggerSplitting = false;

         // finalize the header
         m_p_FileWriter->AdjustFileAtEnd();
   
         // increment the file number
         m_p_FileWriter->NextFile();
         
         // open the next file
         i_OpenSucess = OpenFile();

         
      } // if trigger splitting 
      
      actualSaveBlock->Analyze(m_i_AnalyzeLevel,
                               m_f_AnalyzeNoiseLevel);
      WriteBlock(actualSaveBlock);
      
      if(actualSaveBlock->IsFinalBlock())
      {
         b_Finish = true;
      }

      
      sem_post(&TimerSemaphore);
   } // while not finish

   // finalize the header
   m_p_FileWriter->AdjustFileAtEnd();
   
   return NULL;
} // WriterThreadTimer()

//===========================================================================
// writer thread, write data into a file
//===========================================================================

void * HdRecord::WriterThreadAnalyze()
{
   SaveBlock *actualSaveBlock;
   bool       b_Finish;
   bool       b_AnalyzeResult;
   int        i_OpenSuccess;

   m_ProgramState.b_AnalyzeQuiet        = false;

   b_Finish = false;
   m_p_PreviousSaveBlock = NULL;

   // initital is no file attached
   i_OpenSuccess = -1;

   while(!b_Finish)
   {


      actualSaveBlock= theSaveBlocks -> GetNextWriteBlock();

      if(actualSaveBlock == NULL)
      {
         b_Finish = true;
         break;
      }

      // prepare the sample level
      {
         unsigned char *pc_Buffer;
         int            i;
         int            i_between;

         i_between      = (SAVEBLOCKBUFFERSIZE /4 ) / SAMPLE_AREA_VALUES;

         pc_Buffer = (unsigned char *) actualSaveBlock->Buffer;
         for(i=0; i<SAMPLE_AREA_VALUES ; i++)
         {
            short left;
            short right;
            int   together;
            int   index;

            index = i * 4 * i_between;
            // get pointer to the buffer
            left =0;
            right=0;
            left  |= ((int)(pc_Buffer[index++])) ;
            left  |= ((int)(pc_Buffer[index++])) << 8;
            right |= ((int)(pc_Buffer[index++])) ;
            right |= ((int)(pc_Buffer[index++])) << 8;

            together =(left + right) /2;

            together += 0x8000;


            us_SampleLevel[i] = (unsigned short) (( together ) );
	 }
      }

      //-----------------------------------------------

      // analyze the block, whether it has content
      // result true -> noise found
      b_AnalyzeResult = actualSaveBlock->Analyze(m_i_AnalyzeLevel,
                                                 m_f_AnalyzeNoiseLevel);

      if(m_i_AnalyzeStatus == ModeAnalyzeWait)
      {
         if(b_AnalyzeResult)
         {
            // found data -- start the recording

            // open the output file
            i_OpenSuccess = OpenFile();
            if( i_OpenSuccess <0)
            {
               return NULL;
            }  

            if( m_p_PreviousSaveBlock !=NULL)
            {
               WriteBlock(m_p_PreviousSaveBlock);
            }

               
            m_i_AnalyzeStatus = ModeAnalyzeRecord;
            m_ProgramState.b_AnalyzeQuiet    = false;
            m_p_PreviousSaveBlock =NULL;
            
         }
         else // quit block
         {
            m_ProgramState.b_AnalyzeQuiet = true;
            m_p_PreviousSaveBlock = actualSaveBlock;
         }
            
      }
      else if(m_i_AnalyzeStatus == ModeAnalyzeRecord)
      {
         if(b_AnalyzeResult)
         {
            // keep recording
         }
         else
         {
            // no data -- wait for timeout
            m_i_AnalyzeStatus = ModeAnalyzeEmptyBlock;
            m_i_AnalyzeNumberOfEmptyBlocks =1 ;
         }
         m_p_PreviousSaveBlock =NULL;
      }
      else if(m_i_AnalyzeStatus == ModeAnalyzeEmptyBlock)
      {
         if(b_AnalyzeResult)
         {
            // data found
            // switch back to recording
            m_i_AnalyzeStatus = ModeAnalyzeRecord;
         }
         else
         {
            // more empty blocks
            m_i_AnalyzeNumberOfEmptyBlocks +=1 ;
            // calculate in seconds
            if(m_i_AnalyzeNumberOfEmptyBlocks >= ( m_i_AnalyzeTimeout *3 ) )
            {
               // close the current file
               // finalize the header
               m_p_FileWriter->AdjustFileAtEnd();
   
        
               // increment the file number
               m_p_FileWriter->NextFile();
                  
               m_i_AnalyzeStatus  = ModeAnalyzeWait;
               m_ProgramState.b_AnalyzeQuiet     = true;
            }
         }
         m_p_PreviousSaveBlock =NULL;
      }

         
      // split pressed, and also just recording into a file
      if(    m_ProgramState.b_TriggerSplitting 
          && m_p_FileWriter->IsFileWriting()
          && (   m_i_AnalyzeStatus == ModeAnalyzeEmptyBlock
              || m_i_AnalyzeStatus == ModeAnalyzeRecord
             )
         )
      {


         // finalize the header
         m_p_FileWriter->AdjustFileAtEnd();
   
         // increment the file number
         m_p_FileWriter->NextFile();
         
         // open the next file
         i_OpenSuccess = OpenFile();

         
      } // if trigger splitting 

      // reset the trigger
      m_ProgramState.b_TriggerSplitting = false;



      if( ! m_ProgramState.b_AnalyzeQuiet)         
      {
      
         WriteBlock(actualSaveBlock);
      }
      
      if(actualSaveBlock->IsFinalBlock())
      {
         b_Finish = true;
      }

      
      sem_post(&TimerSemaphore);
   } // while not finish

   if(m_p_FileWriter->IsFileWriting() )
   {
      // finalize the header
      m_p_FileWriter->AdjustFileAtEnd();
   
   }

   // reset into the waiting state
   m_i_AnalyzeStatus  = ModeAnalyzeWait;
   return NULL;

} // WriterThreadAnalyze()

//===========================================================================
// writer thread, write data into a file
//===========================================================================

void *HdRecord::WriterThreadCB(void *v_pc_HdRecorder)
{
   HdRecord * pc_HdRecorder;
   
   
   pc_HdRecorder = (HdRecord *)  v_pc_HdRecorder;
   if( pc_HdRecorder->m_ProgramState.b_AnalyzeMode)
   {
      return pc_HdRecorder->WriterThreadAnalyze();
   }
   else
   {
      if(    pc_HdRecorder->m_ProgramState.b_TimerMode
          || pc_HdRecorder->m_ProgramState.b_TimedSplitMode
        )
      {
         return pc_HdRecorder->WriterThreadTimer();
      }
      else
      {
         return pc_HdRecorder->WriterThread();
      }
   }

} // WriterThreadCB()


//===========================================================================
// timer-tread. do the display update
//===========================================================================

void *HdRecord::TimerThread()
{
   while(true)
   {
      sem_wait(&TimerSemaphore);
      if( m_ProgramState.b_GlobalStop )
      {

         return NULL;
      }

      UpdateRecordingDisplay();

      if(m_p_MainDialog != NULL)
      {
         m_p_MainDialog->SetSampleData(us_SampleLevel);
         m_p_MainDialog->SetLevelMeter(m_f_AnalyzeNoiseLevel);
      }

      SetAnalyzeState();
   }  // forever

} // TimerThread();

//===========================================================================
// timer-tread. do the display update
//===========================================================================

void *HdRecord::TimerThreadTimer()
{
   // gets terminated by m_b_TimerStop
   while(true)
   {
      sem_wait(&TimerSemaphore);

      if( m_ProgramState.b_TimerStop )
      {
         return NULL;
      }
      UpdateRecordingDisplay();

      if(m_p_MainDialog != NULL)
      {
         m_p_MainDialog->SetSampleData(us_SampleLevel);
         m_p_MainDialog->SetLevelMeter(m_f_AnalyzeNoiseLevel);
      }

      SetAnalyzeState();
      if( m_ProgramState.b_TimerStop )
      {
         return NULL;
      }
   }  // forever
   
} // TimerThreadTimer();

//===========================================================================
// timer-tread. do the display update
//===========================================================================

void *HdRecord::TimerThreadCB(void *v_pc_HdRecorder)
{
   HdRecord * pc_HdRecorder;

   pc_HdRecorder = (HdRecord *)  v_pc_HdRecorder;

   if(    pc_HdRecorder->m_ProgramState.b_TimerMode
       || pc_HdRecorder->m_ProgramState.b_TimedSplitMode
     )
   {
      return pc_HdRecorder->TimerThreadTimer();
   }
   else
   {
      return pc_HdRecorder->TimerThread();
   }

   
} // TimerThreadCB();

//===========================================================================
// controller thread for regular recordings
//===========================================================================

void *HdRecord::ControllerThreadRemote()
{
   time_t t_ActualTime;
   
   m_i_RemoteState = RemoteWait;

   // disable most of the input elements
   SetSensitivity();

   pthread_attr_init(&RemoteListenerAttribut);
   // start the remote listener thread

   pthread_create(&p_RemoteListenerThread,
                     &RemoteListenerAttribut,
                     HdRecord::RemoteListenerThreadCB,
                     this);

   
   // start not recording
   m_ProgramState.b_ControllerIsRecording = false;

   // clean up the total  and single byte counts.
   l_TotalBytesWritten = 0;
   l_SingleBytesWritten = 0;         
   UpdateRecordingDisplay();
   
   // terminated, when global stop pressed    
   while(true)
   {
      // wait for start or split   
      while(   m_i_RemoteState != RemoteStart 
            && m_i_RemoteState != RemoteSplit
           )
      {
         // wait for the time to record
         t_ControllerTimespec.tv_sec  =  time(NULL)+1;
         t_ControllerTimespec.tv_nsec =  1;

         // inititalize the timed waiting condition
         pthread_cond_init(&p_ControllerCondition,NULL);

         pthread_mutex_init(& p_ControllerMutex,NULL);


         pthread_cond_timedwait( &p_ControllerCondition,
                              &p_ControllerMutex,
                              &t_ControllerTimespec);
         if(m_ProgramState.b_GlobalStop) 
         {
            break;
         }                              
      }
      // clean up the state
      m_i_RemoteState = RemoteWait;
      

      // woke up -- now start the recording

      m_ProgramState.b_TimerStop = false;
      
      // change mixer device to the new values
      m_p_Mixer->SetInputMixer(m_i_MixerInputDevice,
                               m_i_SampleLevel, 
                               m_i_SampleLevelMaster); 

      // update display
      SetDisplayFromValues();
     

      // initialze the file writer with the current settings
      PrepareFileWriterForRecording();

      // new file has 0 bytes
      l_SingleBytesWritten = 0;         
      
      // set the recording state
      m_ProgramState.b_ControllerIsRecording = true;

      // now start the recoder, writer and timer
      {
         // set up the semaphore
         sem_init(&TimerSemaphore,
               0,                  // process share
               0                   // initital value
               );

         // init thread attributes
         pthread_attr_init(&RecorderAttribut);
         pthread_attr_init(&WriterAttribut);
         pthread_attr_init(&TimerAttribut);


         // create save block structure
         theSaveBlocks = new SaveBlockArray();

         // start the 3 threads

         pthread_create(&p_RecorderThread,
                     &RecorderAttribut,
                     HdRecord::RecorderThreadCB,
                     this);

         pthread_create(&p_WriterThread,
                     &WriterAttribut,
                     HdRecord::WriterThreadCB,
                     this);

         pthread_create(&p_TimerThread,
                     &TimerAttribut,
                     HdRecord::TimerThreadCB,
                     this);
   
      } 


      // now wait for split or stop
      while(   m_i_RemoteState != RemoteStop 
            && m_i_RemoteState != RemoteSplit
           )
      {
         // wait for the time to record
         t_ControllerTimespec.tv_sec  =  time(NULL)+1;
         t_ControllerTimespec.tv_nsec =  1;

         // inititalize the timed waiting condition
         pthread_cond_init(&p_ControllerCondition,NULL);

         pthread_mutex_init(& p_ControllerMutex,NULL);


         pthread_cond_timedwait( &p_ControllerCondition,
                              &p_ControllerMutex,
                              &t_ControllerTimespec);
         if(m_ProgramState.b_GlobalStop) 
         {
            break;
         }                              
      }
      if(m_i_RemoteState == RemoteStop)
      {
         m_i_RemoteState = RemoteWait;
      }
     
      m_ProgramState.b_TimerStop = true;

      // tell the audio recorder to stop
      if(theAudioRecorder != NULL)
      {
         theAudioRecorder->StopRecording = true;
      }

   
      {
         // wait for finish of all threads
         pthread_join(p_RecorderThread,NULL);
         pthread_join(p_WriterThread,NULL);
         pthread_join(p_TimerThread,NULL);


         // free the saveblock structure
         if( theSaveBlocks != NULL)
         {
            delete theSaveBlocks;      
            theSaveBlocks = NULL;
         }
   
         // destroy the timer semaphore
         sem_destroy(&TimerSemaphore)   ;

      }   
      // we are not recording any more
      m_ProgramState.b_ControllerIsRecording = false;
      

   }  // forever

   // should never get here !! 
   // end of timer recording loop - go to normal mode

   // clean up the status line
   ShowTimerStatus(0,0);

   // disable the timer mode
   m_ProgramState.b_TimerMode = false;

   // enable normal buttons
   SetSensitivity();

   // redraw the output
   FlushDisplay();

   return NULL;   
} // ControllerThreadRemote();

//===========================================================================
//
//===========================================================================

void *HdRecord::ControllerThreadTimer()
{
   time_t t_ActualTime;
   RecordDescription *pc_ActualRecord;
   
   // disable most of the input elements
   SetSensitivity();

   // delete all records before this time
   m_p_RecordList.DeleteOldTimes();
   
   // start not recording
   m_ProgramState.b_ControllerIsRecording = false;

   // clean up the total  and single byte counts.
   l_TotalBytesWritten = 0;
   l_SingleBytesWritten = 0;         
   UpdateRecordingDisplay();
   
   // terminated, when global stop pressed or no more record available   
   while(true)
   {

      // get the next recorder description
      pc_ActualRecord = m_p_RecordList.GetRecord(0);
      
      // no more left -> end
      if(pc_ActualRecord == NULL)
      {
         break;
      }

      

      // show waiting message
      ShowTimerStatus( 1, pc_ActualRecord->m_t_StartTime);
   
   
      // wait for the time to record
      t_ControllerTimespec.tv_sec  =  pc_ActualRecord->m_t_StartTime;
      t_ControllerTimespec.tv_nsec =  1;
      
     
      // inititalize the timed waiting condition
      pthread_cond_init(&p_ControllerCondition,NULL);

      pthread_mutex_init(& p_ControllerMutex,NULL);

      pthread_cond_timedwait( &p_ControllerCondition,
                              &p_ControllerMutex,
                              &t_ControllerTimespec);
      if(m_ProgramState.b_GlobalStop) 
      {
         break;
      }                              

      // woke up -- now start the recording

      m_ProgramState.b_TimerStop = false;
      
      // copy the values of the record-description into the state variables
      m_ProgramState.b_AnalyzeMode = pc_ActualRecord -> m_b_UseAnalyzeMode;
      // volume
      m_i_SampleLevel = pc_ActualRecord -> m_i_RecordingLevel;
      // device
      m_i_MixerInputDevice = pc_ActualRecord ->m_i_InputDevice;
     
      // change mixer device to the new values
      m_p_Mixer->SetInputMixer(m_i_MixerInputDevice,
                               m_i_SampleLevel, 
                               m_i_SampleLevelMaster); 

      // type
      e_RecordMode = pc_ActualRecord ->m_e_RecordingFormat;
      // bitrate
      m_i_MP3Bitsize = pc_ActualRecord -> m_i_MP3Rate;

      // update display
      SetDisplayFromValues();
     

      // initialze the file writer with the current settings
      PrepareFileWriterForRecording();

      // new file has 0 bytes
      l_SingleBytesWritten = 0;         
      
      // set the recording state
      m_ProgramState.b_ControllerIsRecording = true;

      // now start the recoder, writer and timer
      {
         // set up the semaphore
         sem_init(&TimerSemaphore,
               0,                  // process share
               0                   // initital value
               );

         // init thread attributes
         pthread_attr_init(&RecorderAttribut);
         pthread_attr_init(&WriterAttribut);
         pthread_attr_init(&TimerAttribut);


         // create save block structure
         theSaveBlocks = new SaveBlockArray();
   
         // start the 3 threads

         pthread_create(&p_RecorderThread,
                     &RecorderAttribut,
                     HdRecord::RecorderThreadCB,
                     this);

         pthread_create(&p_WriterThread,
                     &WriterAttribut,
                     HdRecord::WriterThreadCB,
                     this);

         pthread_create(&p_TimerThread,
                     &TimerAttribut,
                     HdRecord::TimerThreadCB,
                     this);
   
      } 

      				

      // show recording message
      ShowTimerStatus( 2, pc_ActualRecord->m_t_EndTime);

      // wait for the time to finish the recording
      t_ControllerTimespec.tv_sec  =  pc_ActualRecord->m_t_EndTime;
      t_ControllerTimespec.tv_nsec =  1;
      
      // inititalize the timed waiting condition
      pthread_cond_init(&p_ControllerCondition,NULL);

      pthread_mutex_init(& p_ControllerMutex,NULL);
     
      pthread_cond_timedwait( &p_ControllerCondition,
                              &p_ControllerMutex,
                              &t_ControllerTimespec);
      if(m_ProgramState.b_GlobalStop) 
      {
          break;
      }                              
      
      // woke up -- finish recording


      m_ProgramState.b_TimerStop = true;

      // tell the audio recorder to stop
      if(theAudioRecorder != NULL)
      {
         theAudioRecorder->StopRecording = true;
      }

   
      {
         // wait for finish of all threads
         pthread_join(p_RecorderThread,NULL);
         pthread_join(p_WriterThread,NULL);
         pthread_join(p_TimerThread,NULL);


         // free the saveblock structure
         if( theSaveBlocks != NULL)
         {
            delete theSaveBlocks;      
            theSaveBlocks = NULL;
         }
   
         // destroy the timer semaphore
         sem_destroy(&TimerSemaphore)   ;

      }   
      // we are not recording any more
      m_ProgramState.b_ControllerIsRecording = false;
      
      // delete all records before this time
      m_p_RecordList.DeleteOldTimes();


   }  // forever

   // end of timer recording loop - go to normal mode

   // clean up the status line
   ShowTimerStatus(0,0);

   // disable the timer mode
   m_ProgramState.b_TimerMode = false;

   // enable normal buttons
   SetSensitivity();

   // redraw the output
   FlushDisplay();

   return NULL;   
} // ControllerThreadTimer();

//===========================================================================
// controller thread for the timed split
//===========================================================================

void *HdRecord::ControllerThreadTimedSplit()
{
   time_t t_ActualTime;
   
   // disable most of the input elements
   SetSensitivity();

   // clean up the total  and single byte counts.
   l_TotalBytesWritten = 0;
   l_SingleBytesWritten = 0;         
   UpdateRecordingDisplay();
   
   // initialze the file writer with the current settings
   PrepareFileWriterForRecording();


   // set the recording state
   m_ProgramState.b_ControllerIsRecording = true;

   // now start the recoder, writer and timer
   {
      // set up the semaphore
      sem_init(&TimerSemaphore,
               0,                  // process share
               0                   // initital value
               );

      // init thread attributes
      pthread_attr_init(&RecorderAttribut);
      pthread_attr_init(&WriterAttribut);
      pthread_attr_init(&TimerAttribut);


      // create save block structure
      theSaveBlocks = new SaveBlockArray();
   
      // start the 3 threads

      pthread_create(&p_RecorderThread,
                     &RecorderAttribut,
                     HdRecord::RecorderThreadCB,
                     this);

      pthread_create(&p_WriterThread,
                     &WriterAttribut,
                     HdRecord::WriterThreadCB,
                     this);

      pthread_create(&p_TimerThread,
                     &TimerAttribut,
                     HdRecord::TimerThreadCB,
                     this);
   
   } 


   // terminated, when global stop pressed    
   while(true)
   {
      int i_WaitResult;

      time(&t_ActualTime);

      t_ActualTime += m_i_TimedSplitSeconds;


      // show recording message
      ShowTimerStatus( 3, t_ActualTime);

      // wait for the time to finish the recording
      t_ControllerTimespec.tv_sec  =   t_ActualTime;
      t_ControllerTimespec.tv_nsec =  1;
      

      // inititalize the timed waiting condition
      pthread_cond_init(&p_ControllerCondition,NULL);

      pthread_mutex_init(& p_ControllerMutex,NULL);

      i_WaitResult = pthread_cond_timedwait( &p_ControllerCondition,
                              &p_ControllerMutex,
                              &t_ControllerTimespec);


      m_ProgramState.b_TriggerSplitting = true;

      if(m_ProgramState.b_GlobalStop) 
      {
         break;
      }                              
      

   }  // forever


   // end of timer recording loop - go to normal mode

   // clean up the status line
   ShowTimerStatus(0,0);

   // disable the timer mode
   m_ProgramState.b_TimerMode = false;

   // enable normal buttons
   SetSensitivity();

   // redraw the output
   FlushDisplay();

   return NULL;   
} // ControllerThread();


//===========================================================================
//
//===========================================================================

void *HdRecord::ControllerThreadServer()
{
   time_t t_ActualTime;
   RecordDescription *pc_ActualRecord;
   
   // disable most of the input elements
   SetSensitivity();

   // start not recording
   m_ProgramState.b_ControllerIsRecording = false;

   // clean up the total  and single byte counts.
   l_TotalBytesWritten = 0;
   l_SingleBytesWritten = 0;
   UpdateRecordingDisplay();
   
   // terminated, when global stop pressed or no more record available
   while(true)
   {

      // wait for a server connection
      while( ! m_p_FileWriter->IsServerConnected())
      {
         time( & t_ActualTime);


         // show waiting message
	 if(pc_ActualRecord != NULL)
	 {
            ShowTimerStatus( 1, pc_ActualRecord->m_t_StartTime);
	 }


         // wait for the time to record
         t_ControllerTimespec.tv_sec  =  t_ActualTime+2;
         t_ControllerTimespec.tv_nsec =  1;


         // inititalize the timed waiting condition
         pthread_cond_init(&p_ControllerCondition,NULL);

         pthread_mutex_init(& p_ControllerMutex,NULL);


         pthread_cond_timedwait( &p_ControllerCondition,
                              &p_ControllerMutex,
                              &t_ControllerTimespec);
         if(m_ProgramState.b_GlobalStop)
         {
            break;
         }
      } // wait for server connection

      if(m_ProgramState.b_GlobalStop)
      {
         break;
      }


      // woke up -- now start the recording

      m_ProgramState.b_TimerStop = false;

      // update display
      SetDisplayFromValues();


      // initialze the file writer with the current settings
      PrepareFileWriterForRecording();

      // new file has 0 bytes
      l_SingleBytesWritten = 0;

      // set the recording state
      m_ProgramState.b_ControllerIsRecording = true;

      // now start the recoder, writer and timer
      {
         // set up the semaphore
         sem_init(&TimerSemaphore,
               0,                  // process share
               0                   // initital value
               );

         // init thread attributes
         pthread_attr_init(&RecorderAttribut);
         pthread_attr_init(&WriterAttribut);
         pthread_attr_init(&TimerAttribut);


         // create save block structure
         theSaveBlocks = new SaveBlockArray();

         // start the 3 threads

         pthread_create(&p_RecorderThread,
                     &RecorderAttribut,
                     HdRecord::RecorderThreadCB,
                     this);

         pthread_create(&p_WriterThread,
                     &WriterAttribut,
                     HdRecord::WriterThreadCB,
                     this);

         pthread_create(&p_TimerThread,
                     &TimerAttribut,
                     HdRecord::TimerThreadCB,
                     this);

      }



      // wait for a server connection
      while(  m_p_FileWriter->IsServerConnected())
      {

         time( & t_ActualTime);

         // show recording message
	 if(pc_ActualRecord != NULL)
	 {
            ShowTimerStatus( 2, pc_ActualRecord->m_t_EndTime);
	 }


         // wait for the time to record
         t_ControllerTimespec.tv_sec  =  t_ActualTime+2;
         t_ControllerTimespec.tv_nsec =  1;


         // inititalize the timed waiting condition
         pthread_cond_init(&p_ControllerCondition,NULL);

         pthread_mutex_init(& p_ControllerMutex,NULL);


         pthread_cond_timedwait( &p_ControllerCondition,
                              &p_ControllerMutex,
                              &t_ControllerTimespec);
         if(m_ProgramState.b_GlobalStop)
         {
            break;
         }
      } // wait for server connection

      if(m_ProgramState.b_GlobalStop)
      {
         break;
      }

      // woke up -- finish recording


      m_ProgramState.b_TimerStop = true;

      // tell the audio recorder to stop
      if(theAudioRecorder != NULL)
      {
         theAudioRecorder->StopRecording = true;
      }


      {
         // wait for finish of all threads
         pthread_join(p_RecorderThread,NULL);
         pthread_join(p_WriterThread,NULL);
         pthread_join(p_TimerThread,NULL);


         // free the saveblock structure
         if( theSaveBlocks != NULL)
         {
            delete theSaveBlocks;
            theSaveBlocks = NULL;
         }

         // destroy the timer semaphore
         sem_destroy(&TimerSemaphore)   ;

      }
      // we are not recording any more
      m_ProgramState.b_ControllerIsRecording = false;


   }  // forever

   // end of timer recording loop - go to normal mode

   // clean up the status line
   ShowTimerStatus(0,0);

   // disable the timer mode
   m_ProgramState.b_TimerMode = false;

   // enable normal buttons
   SetSensitivity();

   // redraw the output
   FlushDisplay();

   return NULL;

} // ControllerThreadServer();

//===========================================================================
// timer-tread. do the display update
//===========================================================================

void *HdRecord::ControllerThreadCB(void *v_pc_HdRecorder)
{
   HdRecord * pc_HdRecorder;

   pc_HdRecorder = (HdRecord *)  v_pc_HdRecorder;




   // start the controller thread
   if(pc_HdRecorder->m_ProgramState.b_ServerMode)
   {
      // call member funtion
      return pc_HdRecorder->ControllerThreadServer();
   }

   else if(pc_HdRecorder->m_ProgramState.b_RemoteMode)
   {
      // call member funtion
      return pc_HdRecorder->ControllerThreadRemote();
   }
   else if(pc_HdRecorder->m_ProgramState.b_TimerMode)
   {
      // call member funtion
      return pc_HdRecorder->ControllerThreadTimer();
   }

   else if(pc_HdRecorder->m_ProgramState.b_TimedSplitMode)
   {
      // call member funtion
      return pc_HdRecorder->ControllerThreadTimedSplit();
   }


   return NULL;

   
} // ControllerThreadCB();











