// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// level meter widget

#include <stdio.h>


#include <qpainter.h>

#include "levelmeter.h"

#define SIZE_FACTOR 3


//=========================================================================
// constructor
//=========================================================================

LevelMeter::LevelMeter(QWidget *parent)
		: QWidget(parent,"LevelMeter")
{
   int i;

   // set the dimension of the area
   // make quadratic area
   setMinimumSize(SIZE_FACTOR *10,SIZE_FACTOR *10);
   setMaximumSize(SIZE_FACTOR *10,SIZE_FACTOR *10);
   i_Width = width();
   i_Height = height();
   
   // allocate colors
   pc_BlackColor  = new QColor(0x00,0x00,0x00);
   pc_GreenColor  = new QColor(0x00,0xFF,0x00);
   pc_YellowColor = new QColor(0xFF,0xFF,0x00);
   pc_RedColor    = new QColor(0xFF,0x00,0x00);
   
   // allocate a brush
   pc_Brush      = new QBrush(*pc_BlackColor);
   
  
   b_Stopped = false;

   m_f_Level = 0.0;
}


//=========================================================================
//
//=========================================================================

LevelMeter::~LevelMeter()
{
   // delete the brush
   delete pc_Brush  ;

   // delete the colors
   delete pc_RedColor ;
   delete pc_YellowColor ;
   delete pc_GreenColor ;
   delete pc_BlackColor ;
   
} // ~SampleArea()

//=========================================================================
// repaint the area.
// if theEvent is NULL, the function was called from the programm
// if theEvent is valid, it is called by the X-System
//=========================================================================
		
void  LevelMeter::paintEvent(QPaintEvent *theEvent)
{
 
   QPainter painter(this);
   int      i;

   int i_Value;
   int i_MaxSize;
   int i_ColorHeight;

   // if user redraw, but stopped, do nothing
   if(   theEvent ==NULL 
      && b_Stopped)
   {
      return;
   }
   
   // draw a black background
   pc_Brush->setColor(*pc_BlackColor);
   painter.fillRect(0,0,i_Width,i_Height,*pc_Brush);
   i_Value = (int ) (m_f_Level *11.0);

   // draw the green part
   i_MaxSize = i_Value;
   if(i_MaxSize>4)
   {
      i_MaxSize =4;
   }
   pc_Brush->setColor(*pc_GreenColor);

   painter.fillRect(SIZE_FACTOR ,
                    i_Height-(i_MaxSize*SIZE_FACTOR),
                    i_Width -2 *SIZE_FACTOR,
                    (i_MaxSize*SIZE_FACTOR),
                    *pc_Brush);


   // draw the yellow part
   if(i_Value >4)
   {
      i_MaxSize = i_Value;
      if(i_MaxSize>7)
      {
         i_MaxSize =7;
      }
      i_ColorHeight = (i_MaxSize-4) * SIZE_FACTOR;
      pc_Brush->setColor(*pc_YellowColor);
      
      painter.fillRect(SIZE_FACTOR,
                       i_Height - (4*SIZE_FACTOR  ) - i_ColorHeight ,
                       i_Width - 2 *SIZE_FACTOR,
                       i_ColorHeight,
                       *pc_Brush);
   }      
   
   // draw the red part
   if(i_Value >7)
   {
      i_MaxSize = i_Value;
      if(i_MaxSize>10)
      {
         i_MaxSize =10;
      }
      i_ColorHeight = (i_MaxSize-7) * SIZE_FACTOR;
      pc_Brush->setColor(*pc_RedColor);
      painter.fillRect(SIZE_FACTOR,
                       i_Height - (7*SIZE_FACTOR  ) - i_ColorHeight ,
                       i_Width - 2 *SIZE_FACTOR,
                       i_ColorHeight,
                       *pc_Brush);
   }      
   
} // paintEvent()

//=========================================================================
//
//=========================================================================


void  LevelMeter::resizeEvent(QResizeEvent *theEvent)
{
   i_Width  = width();
   i_Height = height();
}


//=========================================================================
//
//=========================================================================

void  LevelMeter::mousePressEvent(QMouseEvent *theEvent)
{

   // on a mouse event toggle the stopped state
   b_Stopped = ! b_Stopped;
   
}


//=========================================================================
// set a new array of sample values
//=========================================================================

void  LevelMeter::SetValue(float f_Level)
{
   m_f_Level = f_Level;
} // SetValue()

//=========================================================================
//
//=========================================================================
