// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// main-modules for khdrecord



#include <qapplication.h>
#include <qcursor.h>

#include <qvbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qgrid.h>
#include <qpopupmenu.h>
#include <qmenubar.h>

#include <qmotifstyle.h>
#include <qwindowsstyle.h>
#include <qplatinumstyle.h>


#include <stdio.h>
#include <stdlib.h>

#include "maindialog.h"
#include "commandline.h"
#include "hdrecord.h"
#include "aboutbox.h"

/*---------------------------------------------------------------*/

//===========================================================================
//
//===========================================================================

void PrintHelp()
{
   printf("KHdRecord Version %s  - Description of Parameters \n",
           KHDRECORD_VERSION);
   printf("-command\n");
   printf("\tSwitch to command line mode\n");
   printf("\tMust be the first parameter\n");
   printf("-outputfile=<filename>\n");
   printf("\tSet name of the output file\n");
   printf("-input=<inputdevice>\n");
   printf("\tInput channel (line,cd,mic)\n");
   printf("-sounddevice=<device>\n");
   printf("\tSound device like /dev/dsp\n");
   printf("-mixerdevice=<device>\n");
   printf("\tMixer device like /dev/mixer\n");
   printf("-recorderfile=<filename>\n");
   printf("\tFile containig record lists\n");
   printf("-level=<number>\n");
   printf("\tLevel of the input channel\n");
   printf("-master=<number>\n");
   printf("\tMaster level of the soundcard\n");
   printf("-bitrate=<number>\n");
   printf("\tMP3 bitrate (40,128,160,192,256,320)\n");
   printf("-start=<date>\n");
   printf("\tStart date of the recording.\n");
   printf("\tDate is of the form like 2002-05-23/15:42:47\n");
   printf("-stop=<date>\n");
   printf("\tStop date of the recording\n");
   printf("-server\n");
   printf("\tSwitch into the server mode\n");
   printf("-port=<portnumber>\n");
   printf("\tPort on which the server listens\n");
   printf("-analyze\n");
   printf("\tSwitch into the analyze mode\n");
   printf("-analyzelevel=<number>\n");
   printf("\tLevel for analyze mode (0..10)\n");
   printf("-analyzetimeout=<number>\n");
   printf("\tTimeout for analyze mode (0..10)\n");
} // PrintHelp

//===========================================================================
//
//===========================================================================


int main(int argc, char *argv[])
{
   CommandLineParameter   *pc_CommandLineParameter;
   QApplication           *pc_Application;
   MainDialog             *pc_MainDialog;
   HdRecord               *pc_Recorder;
   bool                   b_CommandLineVersion;

   b_CommandLineVersion =false;
   if(argc >1)
   {
      if(strcmp(argv[1],"-command") ==0)
      {
	 fprintf(stderr,"KHdRecord command line version\n");
         b_CommandLineVersion    = true;
         pc_CommandLineParameter = new CommandLineParameter(argc,argv);
         if(! pc_CommandLineParameter->m_b_ParseOK)
         {
            exit(3);
         }
         pc_Application = NULL;
      }
      else if(strcmp(argv[1],"-help") ==0)
      {
         PrintHelp();
         exit(4);
      }
   }

   // not command line version
   if( ! b_CommandLineVersion )
   {
      // then create a GUI application
      pc_Application = new QApplication(argc,argv);
   }
   else
   {
      // no GUI
      pc_Application = NULL;
   }


   // create recording main class
   pc_Recorder = new HdRecord(0,pc_Application);

   if( ! b_CommandLineVersion )
   {
      pc_MainDialog = new MainDialog(0,pc_Application,pc_Recorder);
      pc_Recorder->SetMainDialog(pc_MainDialog);
   
      // show the window
      pc_MainDialog->show();

      // execte main window   
      return pc_Application->exec();
   }
   else
   {
      // execute recording from the command line parameters
      int i_Result;
      i_Result = pc_Recorder->CreateRecordListFromCommand(pc_CommandLineParameter);
      
      if(pc_Recorder->m_ProgramState.b_AnalyzeMode)
      {
            pc_Recorder->DoStartButton();

            // wait for termination -- will not happen in analyze mode
            pc_Recorder->WaitForever();
      }
      else
      {
         if(i_Result == 0)
         {
            // start the recording list
            pc_Recorder->DoRecordListStart();

            // wait for termination
            pc_Recorder->WaitForRecordingEnd();
         }
      } // else not analyze mode
   } // else command line version



   
} // main()









