// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// main hdrecord application window class

// #define _DEBUG

#include <qapplication.h>

#include <qvbox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qgrid.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qmessagebox.h>


#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>


#include "maindialog.h"
#include "aboutbox.h"
#include "samplearea.h"
#include "settingsbox.h"
#include "recordlistbox.h"
#include "timedsplitbox.h"
#include "hdrecord.h"



//===========================================================================
// constructor for the recording program
//===========================================================================

MainDialog::MainDialog(QWidget *parent,
               QApplication *mainApplication,
               HdRecord *pc_Recorder)
   		: QMainWindow(parent,
   		              "KHdRecord",
   		              WType_TopLevel )
{
   QBoxLayout  *TopLayout;
   QWidget     *TopWidget;

   m_pc_Recorder = pc_Recorder;

   m_pc_MainApplication = mainApplication;


   // set the window title
   {
      char cs_TitleString[100];
#ifdef LOWFI
      sprintf(cs_TitleString,"KHdRecord LOWFI-Version %s",KHDRECORD_VERSION);
#else
      sprintf(cs_TitleString,"KHdRecord Version %s",KHDRECORD_VERSION);
#endif
      setCaption(cs_TitleString);
   }

   // create the menu   
   m_FileMenu = new QPopupMenu( this );
   m_FileMenu->insertItem("About KHdRecord", this, SLOT( DoAboutBox() ) );
   m_i_MenuItemSettings = m_FileMenu->insertItem("Settings", this, SLOT( DoSettingsBox() ) );
   m_FileMenu->insertItem("Exit" , this, SLOT( Beenden() ) );

   m_RecordListMenu = new QPopupMenu( this );
   m_i_MenuItemRecordlistEdit  = m_RecordListMenu->insertItem("Edit", this, SLOT( DoRecordListBox() ) );
   m_i_MenuItemRecordlistStart = m_RecordListMenu->insertItem("Start", this, SLOT( DoRecordListStart() ) );
#ifdef _DEBUG
   m_RecordListMenu->insertItem("Generate Test Data", this, SLOT( DoRecordListTestData() ) );
#endif   

   m_TimedSplitMenu = new QPopupMenu( this );
   m_i_MenuItemTimedSplitEdit  = m_TimedSplitMenu->insertItem("Edit", this, SLOT( DoTimedSplitBox() ) );
   m_i_MenuItemTimedSplitStart = m_TimedSplitMenu->insertItem("Start", this, SLOT( DoTimedSplitStart() ) );


   // add menus to menubars
   menuBar()->insertItem("File"                 , m_FileMenu );
   menuBar()->insertItem("Record List"          , m_RecordListMenu );
   menuBar()->insertItem("Timed Splitting Mode ", m_TimedSplitMenu );



   
   // topwidgets erzeugen
   TopWidget = new QWidget(this);
   setCentralWidget(TopWidget);
   
  

   TopLayout     = new QVBoxLayout( TopWidget,1);
   

   // block for the filename
   {
      QBoxLayout * FilenameLayout;
      QWidget    * FilenameWidget;
      QLabel     * FilenameLabel;
      
      
      
      FilenameWidget = new QWidget(TopWidget);
      TopLayout->addWidget(FilenameWidget);
     
      FilenameLayout = new  QHBoxLayout( FilenameWidget,2);
      
      FilenameLabel        = new QLabel("Filename: ",FilenameWidget);  
      FilenameStringLabel  = new QLabel("",FilenameWidget);  
      FilenameSelectButton = new QPushButton("Select",FilenameWidget);
      
      FilenameLayout->addWidget(FilenameLabel);
      FilenameLayout->addWidget(FilenameStringLabel);
      FilenameLayout->addWidget(FilenameSelectButton);
   }


   // block for the mode
   {
      QButtonGroup     * ModeWidget;
      QBoxLayout       * ModeLayout;
      

      ModeWidget = new QButtonGroup(3, 
                                    QGroupBox::Horizontal,
                                    "Recording Type",
                                    TopWidget);
      TopLayout->addWidget(ModeWidget);

      ModeMp3Button  = new QRadioButton("mp3",ModeWidget);
#ifndef LOWFI
      ModeWaveButton = new QRadioButton("wav",ModeWidget);
      ModeOggButton  = new QRadioButton("ogg",ModeWidget);
#else
      ModeWaveButton = NULL;
      ModeOggButton  = NULL;

#endif
      connect( ModeWidget, SIGNAL(clicked(int)), SLOT(DoModeChanged(int)) );
   }


   // block for the analyze mode
   {
      QGridLayout  *grid;
      QBoxLayout   *AnalyzeLayout;
      QWidget      *AnalyzeSettingsWidget;

      
      AnalyzeSettingsWidget = new QWidget(TopWidget);
      TopLayout->addWidget(AnalyzeSettingsWidget);
      AnalyzeLayout         = new  QHBoxLayout( AnalyzeSettingsWidget,2);
      
      AnalyzeEnableButton   = new QCheckBox("Analyze Mode",AnalyzeSettingsWidget);
      AnalyzeLayout->addWidget(AnalyzeEnableButton);

      AnalyzeOptionsWidget  = new QGroupBox(AnalyzeSettingsWidget);
      AnalyzeLayout->addWidget(AnalyzeOptionsWidget);
      grid = new QGridLayout( AnalyzeOptionsWidget, 3, 3, 10 );
      
      grid->addWidget(new QLabel("Record Start Level",AnalyzeOptionsWidget),0,0);
      grid->addWidget(new QLabel("Timeout Time",AnalyzeOptionsWidget),1,0);
      grid->addWidget(new QLabel("sec",AnalyzeOptionsWidget),1,2);
      grid->addWidget(new QLabel("State",AnalyzeOptionsWidget),2,0);


//      AnalyzeDetectLevel  = new QSlider(1,10,1, m_i_AnalyzeLevel,Horizontal,AnalyzeOptionsWidget);
      AnalyzeDetectLevel  = new QSlider(1,10,1, 1,Horizontal,AnalyzeOptionsWidget);
      AnalyzeDetectLevel->setTickmarks(QSlider::Below);
//      AnalyzeTimeoutLevel = new QSlider(1,10,1, m_i_AnalyzeTimeout,Horizontal,AnalyzeOptionsWidget);
      AnalyzeTimeoutLevel = new QSlider(1,10,1, 1,Horizontal,AnalyzeOptionsWidget);
      AnalyzeTimeoutLevel->setTickmarks(QSlider::Below);

      AnalyzeStateLabel = new QLabel(" ",AnalyzeOptionsWidget);
      grid->addWidget(AnalyzeDetectLevel,0,1);
      grid->addWidget(AnalyzeTimeoutLevel,1,1);
      grid->addWidget(AnalyzeStateLabel,2,1);

      AnalyzeColorWidget = new QWidget(AnalyzeOptionsWidget);
      AnalyzeColorWidget->setMinimumSize(20,20);
      AnalyzeColorWidget->setMaximumSize(20,20);
      
      AnalyzeDefaultColor = AnalyzeColorWidget->backgroundColor();
      
      grid->addWidget(AnalyzeColorWidget,2,2);
      
     
      connect( AnalyzeEnableButton, 
               SIGNAL(toggled(bool)), 
               SLOT(DoAnalyzeButton(bool)) );
      connect( AnalyzeDetectLevel,  
               SIGNAL(valueChanged(int)), 
               SLOT(DoDetectLevelSliderValueChanged(int)) );
      connect( AnalyzeTimeoutLevel, 
               SIGNAL(valueChanged(int)), 
               SLOT(DoTimeoutSliderValueChanged(int)) );

      // set the enabled mode depending on the analyzemode
//      AnalyzeOptionsWidget->setEnabled(m_b_AnalyzeMode);
//      AnalyzeOptionsWidget->setEnabled(false);
   }

   
   
   // block for the sample level
   {
      QBoxLayout      * SampleLevelLayout;
      QGridLayout     * SliderLayout;
      QWidget         * SampleLevelWidget;
      QWidget         * SliderWidget;
      QLabel          * DescriptionLabel;
      QLabel          * DescriptionMasterLabel;
      char             LevelString[30];
      

      SampleLevelWidget = new QWidget(TopWidget);
      TopLayout->addWidget(SampleLevelWidget);
     
      SampleLevelLayout = new  QHBoxLayout( SampleLevelWidget,2);
      
      SliderWidget = new QWidget(SampleLevelWidget);

      SliderLayout = new QGridLayout( SliderWidget,2,3);
	
      DescriptionLabel       = new QLabel("Input Level: ",SliderWidget); 
      DescriptionMasterLabel = new QLabel("Master Level: ",SliderWidget); 
      
      SampleLevelSlider = new QSlider(0,100, 			// min/max
                                      1,     			// pageStep,
                                      50, 		//int value, 
                                      QSlider::Horizontal, 	// Orientation, 
                                      SliderWidget);
                                      
      SampleLevelMasterSlider = new QSlider(0,100, 			// min/max
                                      1,     			// pageStep, 
                                      50, 		//int value, 
                                      QSlider::Horizontal, 	// Orientation, 
                                      SliderWidget);

       
      sprintf(LevelString,"%3d",100);
      SampleLevelLabel       = new QLabel(LevelString,SliderWidget);  
      SampleLevelMasterLabel = new QLabel(LevelString,SliderWidget);  


      SliderLayout->addWidget(DescriptionLabel,0,0);
      SliderLayout->addWidget(DescriptionMasterLabel,1,0);
      SliderLayout->addWidget(SampleLevelSlider,0,1);
      SliderLayout->addWidget(SampleLevelMasterSlider,1,1);
      SliderLayout->addWidget(SampleLevelLabel ,0,2);
      SliderLayout->addWidget(SampleLevelMasterLabel ,1,2);

      // output of level graphics
      SampleOutputWidget = new SampleArea(SampleLevelWidget);
      
      SampleLevelLayout->addWidget(SliderWidget);
      SampleLevelLayout->addWidget(SampleOutputWidget);

      // output of level-meter graphics
      LevelMeterWidget = new LevelMeter(SampleLevelWidget);
      SampleLevelLayout->addWidget(LevelMeterWidget);

      connect( SampleLevelSlider, SIGNAL(valueChanged(int)), SLOT(DoSliderValueChanged(int)) );
      connect( SampleLevelMasterSlider, SIGNAL(valueChanged(int)), SLOT(DoMasterSliderValueChanged(int)) );

   } // block for sample level


   // block for the time
   {
      QGridLayout * TimeLayout;
      QWidget     * TimeWidget;
      QLabel      * TimeLabel;
      QLabel      * SingleTimeLabel;
      QLabel      * TrackLabel;
      
      
      
      TimeWidget = new QWidget(TopWidget);
      TopLayout->addWidget(TimeWidget);
     
      TimeLayout = new  QGridLayout( TimeWidget,3,2,10);

      TimeLabel       = new QLabel("Total Time: ",TimeWidget);  
      TimeStringLabel = new QLabel("00:00:00",TimeWidget);  

      SingleTimeLabel       = new QLabel("Last Track Time: ",TimeWidget);  
      SingleTimeStringLabel = new QLabel("00:00:00",TimeWidget);  
      
      TrackLabel                   = new QLabel("Last Track Name: ",TimeWidget);  
      RecordingFilenameStringLabel = new QLabel("-",TimeWidget);  

      TimeLayout->addWidget(TimeLabel,0,0);
      TimeLayout->addWidget(TimeStringLabel,0,1);
      TimeLayout->addWidget(SingleTimeLabel,1,0);
      TimeLayout->addWidget(SingleTimeStringLabel,1,1);
      TimeLayout->addWidget(TrackLabel,2,0);
      TimeLayout->addWidget(RecordingFilenameStringLabel,2,1);

   } // block for time display


   // block fuer start/stop button   
   {
      QGridLayout *ButtonsLayout;
      QWidget     *ButtonWidget;
      
      ButtonWidget = new QWidget(TopWidget);
      TopLayout->addWidget(ButtonWidget);
      
      ButtonsLayout = new QGridLayout( ButtonWidget, 2, 2,10);


      StartButton  = new QPushButton("Start",ButtonWidget);
      StopButton   = new QPushButton("Stop",ButtonWidget);
      PauseButton  = new QPushButton("Pause",ButtonWidget);
      SplitButton  = new QPushButton("Split",ButtonWidget);
      ButtonsLayout->addWidget (StartButton,0,0);
      ButtonsLayout->addWidget (StopButton,0,1);
      ButtonsLayout->addWidget (PauseButton,1,0);
      ButtonsLayout->addWidget (SplitButton,1,1);

      ButtonAccel = new QAccel(this);

   } // block for start stop



   StatusLabel = new QLabel("",TopWidget);
   TopLayout->addWidget(StatusLabel);


   // connect the buttons
   connect(StartButton,
           SIGNAL(clicked()),
           this,
           SLOT(DoStartButton()) );

   // connect the stop button
   connect(StopButton,
           SIGNAL(clicked()),
           this,
           SLOT(DoStopButton()) );
   // connect the select button
   connect(FilenameSelectButton,
           SIGNAL(clicked()),
           this,
           SLOT(DoSelectButton()) );
   // connect the pause button
   connect(PauseButton,
           SIGNAL(clicked()),
           this,
           SLOT(DoPauseButton()) );
   connect(SplitButton,
           SIGNAL(clicked()),
           this,
           SLOT(DoSplitButton()) );


   ButtonAccel->connectItem(ButtonAccel->insertItem(Key_F1),
                                 this,
                                 SLOT(DoStartButton())
                                 );

   ButtonAccel->connectItem(ButtonAccel->insertItem(Key_F12),
                                 this,
                                 SLOT(DoStopButton())
                                 );
   ButtonAccel->connectItem(ButtonAccel->insertItem(Key_P),
                                 this,
                                 SLOT(DoPauseButton())
                                 );
   ButtonAccel->connectItem(ButtonAccel->insertItem(Key_Space),
                                 this,
                                 SLOT(DoSplitButton())
                                 );

   // the stop button ist disabled
   StopButton->setEnabled(false);


} // MainDialog()

//===========================================================================
// update the time display
//===========================================================================

MainDialog::~MainDialog()
{
   
} // destructor


//=============================================================================
//
//=============================================================================

void MainDialog::DoSettingsBox()
{
   SettingsBox * settingsBox;
 
   settingsBox = new SettingsBox(this,m_pc_Recorder);
   
   settingsBox->exec();
  
   // check again, whether the devices are available
   m_pc_Recorder->CheckDevices();   

   delete settingsBox;
} // DoSettingsBox()

//=============================================================================
//
//=============================================================================

void MainDialog::DoRecordListBox()
{
   RecordListBox * recordListBox;
 
   recordListBox = new RecordListBox(this,m_pc_Recorder);
   
   recordListBox->exec();
   
   delete recordListBox;
} // DoRecordListBox()

//=============================================================================
//
//=============================================================================

void MainDialog::DoRecordListStart()
{

   m_pc_Recorder ->DoRecordListStart();
   
} // DoRecordListBox()

//=============================================================================
//
//=============================================================================

void MainDialog::DoTimedSplitBox()
{
   TimedSplitBox * timeSplitBox;
 
   timeSplitBox = new TimedSplitBox(this,m_pc_Recorder);
   
   timeSplitBox->exec();
   
   delete timeSplitBox;
   
} // DoTimedSplitBox()

//=============================================================================
//
//=============================================================================

void MainDialog::DoTimedSplitStart()
{
   m_pc_Recorder -> DoTimedSplitStart();
   
} // DoTimedSplitStart()

//=============================================================================
//
//=============================================================================

void MainDialog::DoRecordListTestData()
{
   m_pc_Recorder->m_p_RecordList .CreateTestDataset();

} // DoRecordTestData()

//===========================================================================
// eventfunction for the start button
//===========================================================================


void MainDialog::DoStartButton()
{
   if( ! StartButton->isEnabled())
   {
      return;
   }

   m_pc_Recorder->DoStartButton();

} // DoStartButton()

//===========================================================================
// eventfunction for the stop button
//===========================================================================


void MainDialog::DoStopButton()
{
   if( ! StopButton->isEnabled())
   {
      return;
   }

   m_pc_Recorder->DoStopButton();

} // DoStopButton()

//===========================================================================
// eventfunction for the pause button
//===========================================================================


void MainDialog::DoPauseButton()
{
   if( ! PauseButton->isEnabled())
   {
      return;
   }

   m_pc_Recorder->DoPauseButton();

} // DoPauseButton()

//===========================================================================
// event for the split button, trigger the split in a variable
//===========================================================================


void MainDialog::DoSplitButton()
{
   if( ! SplitButton->isEnabled())
   {
      return;
   }

   m_pc_Recorder->DoSplitButton();

}   // DoSplitpButton()

//===========================================================================
// select a filename for writing destination
//===========================================================================



void MainDialog::DoSelectButton()
{

   QFileDialog *fileDialog;
   QString     returnPath;
   QString     qs_BaseName;

   int         i_FileDialogReturn;

   // open file dialog
   fileDialog = new QFileDialog(this,"fileDialog",TRUE);
   fileDialog->setCaption("Enter Filename");
   fileDialog->setSelection(FilenameStringLabel->text() ) ;
   fileDialog->setMode ( QFileDialog::AnyFile);
   
   // set the file modes depnding on the current record mode
   if(ModeMp3Button->isChecked())
   {
      fileDialog->setFilter ( "MP3 Files (*.mp3)");
   }
   if(ModeWaveButton != NULL)
   {
      if(ModeWaveButton->isChecked())
      {
         fileDialog->setFilter ( "Wave Files (*.wav)");
      }
   }

   if(ModeOggButton != NULL )
   {
      if(ModeOggButton->isChecked())
      {
         fileDialog->setFilter ( "Ogg Files (*.ogg)");
      }
   }

   // execute it
   i_FileDialogReturn = fileDialog->exec();
   
  
   // not canceled
   if(i_FileDialogReturn >0)
   {
      // get the selected value 
      returnPath = fileDialog->selectedFile();
      FilenameStringLabel->setText(returnPath);
      
      {
         // calculate the basename
         if(returnPath.isEmpty())
         {
            qs_BaseName ="test";

         }
         else
         {
            qs_BaseName = returnPath;
         }
         if(qs_BaseName.length() >4)
         {
            QString qs_Right;

            qs_Right = qs_BaseName.right(4);
            if(qs_Right.compare(".wav") ==0)
            {
               qs_BaseName = qs_BaseName.left(qs_BaseName.length() -4) ;
            }
            else if(qs_Right.compare(".mp3") ==0)
            {
               qs_BaseName = qs_BaseName.left(qs_BaseName.length() -4) ;
            }
            else if(qs_Right.compare(".ogg") ==0)
            {
               qs_BaseName = qs_BaseName.left(qs_BaseName.length() -4) ;
            }

         }
         else
         {
         }

         m_pc_Recorder->m_qs_Basename = qs_BaseName;
         
      }
      
   
   } // if not canceled
   
   
   // delete the file dialog
   delete fileDialog;

} // DoSelectButton()


//===========================================================================
// display the about box
//===========================================================================

void MainDialog::DoAboutBox()
{
   AboutBox * aboutBox;
 
   aboutBox = new AboutBox(this);
   
   aboutBox->exec();
   
   delete aboutBox;
} // DoAboutBox()


//===========================================================================
//
//===========================================================================


void MainDialog::FlushDisplay()
{
   //- not needed anymore
   // flush all X events
   //- m_pc_MainApplication->flushX();
   // for QT2 I have also to use this call
   //-m_pc_MainApplication->sendPostedEvents();
   //-m_pc_MainApplication->processEvents();

}  //  FlushDisplay()


//===========================================================================
//
//===========================================================================

void MainDialog::SetSensitivity(ProgramState c_ProgramState)
{
   if(c_ProgramState.b_TimerMode)
   {
      // disable all elements, except the stop button
      StartButton            -> setEnabled(false);
      StopButton             -> setEnabled(true);
      PauseButton            -> setEnabled(false);
      SplitButton            -> setEnabled(false);
      SampleLevelSlider      -> setEnabled(false);
      SampleLevelMasterSlider-> setEnabled(false);
      AnalyzeEnableButton    -> setEnabled(false);
      AnalyzeOptionsWidget   -> setEnabled(false);
      if(ModeWaveButton != NULL)
      {
         ModeWaveButton         -> setEnabled(false);
      }
      ModeMp3Button          -> setEnabled(false);
      if(ModeOggButton)
      {
         ModeOggButton          -> setEnabled(false);
      }
      FilenameSelectButton   -> setEnabled(false);

      // disable some menu items
      EnableMenus(false);

   }


   if(c_ProgramState.b_TimedSplitMode)
   {
      // disable all elements, except the stop button
      StartButton            -> setEnabled(false);
      StopButton             -> setEnabled(true);
      PauseButton            -> setEnabled(true);
      SplitButton            -> setEnabled(true);
      SampleLevelSlider      -> setEnabled(true);
      SampleLevelMasterSlider-> setEnabled(true);
      AnalyzeEnableButton    -> setEnabled(false);
      AnalyzeOptionsWidget   -> setEnabled(false);
      if(ModeWaveButton != NULL)
      {
         ModeWaveButton         -> setEnabled(false);
      }
      ModeMp3Button          -> setEnabled(false);
      if(ModeOggButton)
      {
         ModeOggButton          -> setEnabled(false);
      }
      FilenameSelectButton   -> setEnabled(false);

      // disable some menu items
      EnableMenus(false);

   }
   else // not timer mode
   {
      if(c_ProgramState.b_AnalyzeMode )
      {
         AnalyzeOptionsWidget->setEnabled(true);
      }
      else
      {
         AnalyzeOptionsWidget->setEnabled(false);
      }


      if(c_ProgramState.b_GlobalStop)
      {
          // not in recording
         StartButton            -> setEnabled(true);
         StopButton             -> setEnabled(false);
         PauseButton            -> setEnabled(true);
         SplitButton            -> setEnabled(false);
         SampleLevelSlider      -> setEnabled(true);
         SampleLevelMasterSlider-> setEnabled(true);
         AnalyzeEnableButton    -> setEnabled(true);
         if(ModeWaveButton!= NULL)
         {
            ModeWaveButton         -> setEnabled(true);
         }
         ModeMp3Button          -> setEnabled(true);
         if(ModeOggButton!= NULL)
         {
            ModeOggButton          -> setEnabled(true);
         }
         FilenameSelectButton   -> setEnabled(true);

         // enable some menu items
         EnableMenus(true);

      }
      else
      {
         // just recording
         StartButton            -> setEnabled(false);
         StopButton             -> setEnabled(true);
         PauseButton            -> setEnabled(true);
         SplitButton            -> setEnabled(true);
         SampleLevelSlider      -> setEnabled(true);
         SampleLevelMasterSlider-> setEnabled(true);
         AnalyzeEnableButton    -> setEnabled(false);
         if(ModeWaveButton != NULL)
         {
            ModeWaveButton         -> setEnabled(false);
         }
         ModeMp3Button          -> setEnabled(false);
         if(ModeOggButton != NULL)
         {
            ModeOggButton          -> setEnabled(false);
         }
         FilenameSelectButton   -> setEnabled(false);

         // disable some menu items
         EnableMenus(false);
      }

   } // else not timer mode

   // if the devices are not available, then disable recording
   if(!c_ProgramState.b_DeviceAvailable)
   {
      StartButton      -> setEnabled(false);
      m_RecordListMenu ->setItemEnabled(m_i_MenuItemRecordlistStart,false);
      m_TimedSplitMenu ->setItemEnabled(m_i_MenuItemTimedSplitStart,false);

   }


} // SetSensitivity()


//===========================================================================
//
//===========================================================================

void MainDialog::EnableMenus(bool b_Enable)
{
   // set state of menu items
   m_FileMenu ->setItemEnabled(m_i_MenuItemSettings,b_Enable);

   m_RecordListMenu ->setItemEnabled(m_i_MenuItemRecordlistStart,b_Enable);
   m_RecordListMenu ->setItemEnabled(m_i_MenuItemRecordlistEdit ,b_Enable);

   m_TimedSplitMenu ->setItemEnabled(m_i_MenuItemTimedSplitStart,b_Enable);
   m_TimedSplitMenu ->setItemEnabled(m_i_MenuItemTimedSplitEdit ,b_Enable);

} // EnableMenus()

//===========================================================================
//
//===========================================================================

void MainDialog::SetRecordMode(RecordMode e_RecordMode)
{
#ifdef LOWFI
   ModeMp3Button->setChecked(1);
#else
   if(e_RecordMode == ModeMp3)
   {
   }
   if(e_RecordMode == ModeWave)
   {
      ModeWaveButton->setChecked(1);
   }
   if(e_RecordMode == ModeOgg)
   {
      ModeOggButton->setChecked(1);
   }
#endif
} // SetRecordMode()

//===========================================================================
//
//===========================================================================

void MainDialog::SetAnalyzeLevel(int i_AnalyzeLevel)
{
   AnalyzeDetectLevel->setValue(i_AnalyzeLevel);
} //SetAnalyzeLevel()

//===========================================================================
//
//===========================================================================

void MainDialog::SetAnalyzeTimeout(int i_AnalyzeTimeout)
{
   AnalyzeTimeoutLevel->setValue(i_AnalyzeTimeout);
}  // SetAnalyzeTimeout()
 
//===========================================================================
//
//===========================================================================

void MainDialog::SetAnalyzeMode(bool b_AnalyeEnabled)
{
   AnalyzeEnableButton->setChecked(b_AnalyeEnabled);
} // SetAnalyzeMode(

//===========================================================================
//
//===========================================================================

void MainDialog::SetSampleLevel(int i_SampleLevel)
{
   SampleLevelSlider -> setValue(i_SampleLevel);
} //SetSampleLevel()

//===========================================================================
//
//===========================================================================

void MainDialog::SetSampleLevelMaster(int i_SampleLevelMaster)
{
   SampleLevelMasterSlider -> setValue(i_SampleLevelMaster);
} //SetSampleLevel()

//===========================================================================
//
//===========================================================================

void MainDialog::SetTotalTime(int i_TotalSeconds)
{
   static int i_LastSeconds = -1;

   char       c_DelayString[80];
   int        i_Seconds;
   int        i_Minutes;
   int        i_Hours;
   

   // same delay as the last time?
   if(i_TotalSeconds == i_LastSeconds)
   {
      return;
   }
   i_LastSeconds = i_TotalSeconds;
   
   // calculate h m s
   i_Seconds        = i_TotalSeconds %60;
   i_TotalSeconds  /= 60;
   
   i_Minutes = i_TotalSeconds %60;
   
   i_Hours   = i_TotalSeconds /60;

   // format a string   
   sprintf(c_DelayString,"%2d:%02d:%02d",i_Hours,i_Minutes,i_Seconds);

   // output the new string
   m_pc_MainApplication->lock();
   TimeStringLabel->setText(c_DelayString);
   m_pc_MainApplication->unlock();


} // SetTotalTime()

//===========================================================================
//
//===========================================================================


void MainDialog::SetSingleTime(int i_TotalSeconds)
{
   static int i_LastSeconds = -1;

   char       c_DelayString[80];
   int        i_Seconds;
   int        i_Minutes;
   int        i_Hours;


   // same delay as the last time?
   if(i_TotalSeconds == i_LastSeconds)
   {
      return;
   }
   i_LastSeconds = i_TotalSeconds;
   
   // calculate h m s
   i_Seconds        = i_TotalSeconds %60;
   i_TotalSeconds  /= 60;
   
   i_Minutes = i_TotalSeconds %60;
   
   i_Hours   = i_TotalSeconds /60;

   // format a string   
   sprintf(c_DelayString,"%2d:%02d:%02d",i_Hours,i_Minutes,i_Seconds);

   // output the new string   
   m_pc_MainApplication->lock();
   SingleTimeStringLabel->setText(c_DelayString);
   m_pc_MainApplication->unlock();


} // SetSingleTime()

//===========================================================================
//
//===========================================================================

void MainDialog::SetRecordingFilename(QString qs_Filename)
{
   m_pc_MainApplication->lock();
   RecordingFilenameStringLabel->setText(qs_Filename);
   m_pc_MainApplication->unlock();
} //SetRecordingFilename()

//===========================================================================
//
//===========================================================================

void MainDialog::SetFilename(QString qs_Filename)
{

   m_pc_MainApplication->lock();
   FilenameStringLabel->setText(qs_Filename);
   m_pc_MainApplication->unlock();
} //SetRecordingFilename()


//===========================================================================
//
//===========================================================================

void MainDialog::SetAnalyzeStateLabel(QString qs_StateString)
{
   m_pc_MainApplication->lock();
   AnalyzeStateLabel->setText(qs_StateString);
   m_pc_MainApplication->unlock();

} //SetRecordingFilename()

//===========================================================================
//
//===========================================================================

void MainDialog::SetAnalyzeColor(AnalyzeColorMode c_ColorMode)
{
   QColor q_Color;
   switch(c_ColorMode)
   {
      case AnalyzeColor_Wait:
         q_Color = QColor(0,0,255);
         break;
      case AnalyzeColor_Record:
         q_Color = QColor(255,0,0);
         break;
      case AnalyzeColor_Timeout:
         q_Color = QColor(255,255,0);
         break;

      default:
         q_Color = AnalyzeDefaultColor;
         break;
   }
   m_pc_MainApplication->lock();
   AnalyzeColorWidget->setBackgroundColor(q_Color);
   m_pc_MainApplication->unlock();
} // SetAnalyzeColor

//===========================================================================
//
//===========================================================================

void MainDialog::DoAnalyzeButton(bool enabled)
{
   AnalyzeOptionsWidget->setEnabled(enabled);

   m_pc_Recorder->DoAnalyzeButton(enabled);


} // DoAnalyzeButton()

//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void MainDialog::DoSliderValueChanged(int i_SliderValue)
{
   char LevelString[30];
   
   // write it out
   sprintf(LevelString,"%3d",i_SliderValue);
   SampleLevelLabel->setText(LevelString);  

   m_pc_Recorder->DoSliderValueChanged(i_SliderValue);
   
} // DoSliderValueChanged()

//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void MainDialog::DoMasterSliderValueChanged(int i_SliderValue)
{
   char LevelString[30];
   
   // write it out
   sprintf(LevelString,"%3d",i_SliderValue);
   SampleLevelMasterLabel->setText(LevelString);  

   m_pc_Recorder->DoMasterSliderValueChanged(i_SliderValue);
   
} // DoMasterSliderValueChanged()

//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void MainDialog::DoTimeoutSliderValueChanged(int i_SliderValue)
{
   m_pc_Recorder->DoTimeoutSliderValueChanged(i_SliderValue);
   
} // DoTimeoutSliderValueChanged()

//===========================================================================
// eventfunction if the input level slider has changed
//===========================================================================

void MainDialog::DoDetectLevelSliderValueChanged(int i_SliderValue)
{
   m_pc_Recorder->DoDetectLevelSliderValueChanged(i_SliderValue);
   
} // DoDetectLevelSliderValueChanged()

//===========================================================================
// terminate
//===========================================================================


void MainDialog::Beenden()
{
   m_pc_Recorder->Beenden();
} // Beenden()


//===========================================================================
// catch the close event from the main window x-corner
//===========================================================================

void MainDialog::closeEvent(QCloseEvent *)
{
   Beenden();
} // closeEvent


//===========================================================================
// eventfunction if the mode is recording mode is changed 
//===========================================================================


void MainDialog::DoModeChanged(int i_Mode)
{
   m_pc_Recorder->DoModeChanged(i_Mode);

} // DoModeChanged()
//===========================================================================
//
//===========================================================================

void MainDialog::SetPauseButtonText(QString qs_ButtonText)
{
   m_pc_MainApplication->lock();
   PauseButton->setText(qs_ButtonText);
   m_pc_MainApplication->unlock();
} // SetPauseButtonText()

//===========================================================================
//
//===========================================================================

void MainDialog::SetStatusLabel(QString qs_StatusString)
{
   m_pc_MainApplication->lock();
   StatusLabel->setText(qs_StatusString);
   m_pc_MainApplication->unlock();
} // SetStatusLabel()

//===========================================================================
//
//===========================================================================
void MainDialog::SetSampleData(unsigned short *pus_Data)
{
   m_pc_MainApplication->lock();
   SampleOutputWidget->SetValues(pus_Data);
   SampleOutputWidget->paintEvent(NULL);
   m_pc_MainApplication->unlock();

} // SetSampleData()

//===========================================================================
//
//===========================================================================

void MainDialog::SetLevelMeter(float f_Level)
{
   m_pc_MainApplication->lock();
   LevelMeterWidget->SetValue(f_Level);
   LevelMeterWidget->paintEvent(NULL);
   m_pc_MainApplication->unlock();

} // SetLevelMeter()

//===========================================================================
//
//===========================================================================






