//
#ifndef _MAINDIALOG_H_LOADED
#define _MAINDIALOG_H_LOADED


#include <stdio.h>
#include <unistd.h>

#include <qwidget.h>

#include <qapplication.h>
#include <qmainwindow.h>

#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qscrollbar.h>
#include <qlabel.h>
#include <qslider.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qaccel.h>

#include "basedef.h"

#include "samplearea.h"
#include "levelmeter.h"


class HdRecord;

extern "C"
{
#include "lame/lame.h"
}

enum AnalyzeColorMode
{
   AnalyzeColor_Default,
   AnalyzeColor_Wait,
   AnalyzeColor_Record,
   AnalyzeColor_Timeout,

}; // AnalyzeColorMode


class MainDialog : public QMainWindow
{
  Q_OBJECT
public:
   MainDialog(QWidget *parent,
                  QApplication *mainApplication,
                  HdRecord *pc_Recorder);
   
   ~MainDialog();
  
public slots:  
   void DoSettingsBox();

   void DoRecordListBox();
   void DoRecordListStart();

   void DoTimedSplitBox();
   void DoTimedSplitStart();
   void DoRecordListTestData();

   void DoStartButton();
   void DoStopButton();
   void DoPauseButton();
   void DoSplitButton();

   void DoSelectButton();

   void DoAboutBox();
   void DoAnalyzeButton(bool);
   void DoSliderValueChanged(int);
   void DoMasterSliderValueChanged(int);
   void DoTimeoutSliderValueChanged(int);
   void DoDetectLevelSliderValueChanged(int);

   void Beenden();

   void DoModeChanged(int);


public:
   QApplication   *m_pc_MainApplication;
   HdRecord       *m_pc_Recorder;

   //------ menubar section
   QPopupMenu  *m_FileMenu;
   QPopupMenu  *m_RecordListMenu;
   QPopupMenu  *m_TimedSplitMenu;

   int m_i_MenuItemSettings;
   int m_i_MenuItemRecordlistStart;
   int m_i_MenuItemRecordlistEdit;

   int m_i_MenuItemTimedSplitStart;   
   int m_i_MenuItemTimedSplitEdit;

   //------- filename 
   QLabel *FilenameStringLabel;
   QPushButton *FilenameSelectButton;

   //-------- recording mode
   QRadioButton     * ModeWaveButton;
   QRadioButton     * ModeMp3Button;
   QRadioButton     * ModeOggButton;

   //-------- analyze mode section
   QCheckBox    *AnalyzeEnableButton;
   QGroupBox    *AnalyzeOptionsWidget;
   QWidget      *AnalyzeColorWidget;
   QSlider      *AnalyzeDetectLevel;
   QSlider      *AnalyzeTimeoutLevel;
   QLabel       *AnalyzeStateLabel;
   QColor        AnalyzeDefaultColor;

   //-------- sample level section
   QSlider     *SampleLevelSlider;
   QLabel      *SampleLevelLabel;
   SampleArea  *SampleOutputWidget;
   LevelMeter  *LevelMeterWidget;

   //-------- sample level section
   QSlider     *SampleLevelMasterSlider;
   QLabel      *SampleLevelMasterLabel;

   //-------- time display section
   QLabel *TimeStringLabel;
   QLabel *SingleTimeStringLabel;
   // changeable output resources
   QLabel *RecordingFilenameStringLabel;

   //------- main buttons
   QPushButton *StartButton;
   QPushButton *StopButton;
   QPushButton *PauseButton;
   QPushButton *SplitButton;
   QAccel      *ButtonAccel;

   //-------- status line at the lower end
   QLabel      *StatusLabel;

public:
   void FlushDisplay();


public:

   void SetSensitivity(ProgramState c_ProgramState);

   void SetRecordMode(RecordMode e_RecordMode);
   void SetAnalyzeLevel(int i_AnalyzeLevel);
   void SetAnalyzeTimeout(int i_AnalyzeTimeout);
   void SetAnalyzeMode(bool b_AnalyeEnabled);
   void SetAnalyzeStateLabel(QString qs_StateString);
   void SetAnalyzeColor(AnalyzeColorMode c_ColorMode);
   void SetSampleLevel(int i_SampleLevel);
   void SetSampleLevelMaster(int i_SampleLevelMaster);
   void SetTotalTime(int i_TotalSeconds);
   void SetSingleTime(int i_TotalSeconds);
   void SetRecordingFilename(QString qs_Filename);
   void SetFilename(QString qs_Filename);
   void SetPauseButtonText(QString qs_ButtonText);
   void SetStatusLabel(QString qs_StatusString);
   void SetSampleData(unsigned short *pus_Data);
   void SetLevelMeter(float f_Value);


protected:
   void closeEvent(QCloseEvent *);

private:
   void EnableMenus(bool b_Enable);

} ; // class MainDialog

#endif



