// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// mixer
// modul to access the input mixer
//
//===========================================================================


#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/ioctl.h>
#include <linux/soundcard.h>


#include "mixer.h"

typedef struct stereovolume
{
   unsigned char left;
   unsigned char right;
   unsigned char dummy[2];
} MixerStereoVolume;


//===========================================================================
//
//===========================================================================

MixerDevice::MixerDevice(QString qs_DeviceName)
{
   m_qs_DeviceName = qs_DeviceName;

} // constructor

//===========================================================================
//
//===========================================================================

MixerDevice::~MixerDevice()
{

}  // destructor

//===========================================================================
//
//===========================================================================

void MixerDevice::SetMixerDevice(QString qs_DeviceName)
{
   m_qs_DeviceName = qs_DeviceName;

} // SetMixerDevice();

//===========================================================================
//
//===========================================================================

void MixerDevice::SetInputMixer(int i_InputDevice, 
                                int i_SampleLevel,
                                int i_SampleLevelMaster)
{
   MixerStereoVolume  volume;
   MixerStereoVolume  ZeroVolume;
   unsigned int       ui_RecordingDevice[4];
   int                ergebnis;
   int                mixer_fd;

   // clean the memory
   memset(ui_RecordingDevice,0,sizeof(ui_RecordingDevice));

   // open the mixer device
   mixer_fd = open(m_qs_DeviceName, O_RDWR | O_NONBLOCK, 0);
   
   if(mixer_fd < 0)
   {
      // could not open
      return;
   }
  
   // set input level to i_SampleLevel percent
   volume.left  = i_SampleLevel;
   volume.right = i_SampleLevel;
   
   // zero volume is 0
   ZeroVolume.left  = 0;
   ZeroVolume.right = 0;

   
   if(i_InputDevice == SOUND_MIXER_LINE)
   {
      // set the line in to that level
      ergebnis = ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_LINE) , &volume);
      ergebnis = ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_MIC)  , &ZeroVolume);
      ergebnis = ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_CD)   , &ZeroVolume);
      ui_RecordingDevice[0] = SOUND_MASK_LINE;
      ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_RECSRC) ,ui_RecordingDevice);
   }
   else if(i_InputDevice == SOUND_MIXER_MIC)
   {
      // set the line in to that level
      ergebnis = ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_LINE) , &ZeroVolume);
      ergebnis = ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_MIC)  , &volume);
      ergebnis = ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_CD)   , &ZeroVolume);
      ui_RecordingDevice[0] = SOUND_MASK_MIC;
      ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_RECSRC) , ui_RecordingDevice);
   }

   else if(i_InputDevice == SOUND_MIXER_CD)
   {
      // set the line in to that level
      ergebnis = ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_LINE) , &ZeroVolume);
      ergebnis = ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_MIC)  , &ZeroVolume);
      ergebnis = ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_CD)   , &volume);
      ui_RecordingDevice[0] = SOUND_MASK_CD;
      ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_RECSRC) , ui_RecordingDevice);
   }
   

   // set master input level to i_SampleLevelMaster percent
   volume.left  = i_SampleLevelMaster;
   volume.right = i_SampleLevelMaster;
   // also the total input 
   ergebnis = ioctl(mixer_fd, 
                    MIXER_WRITE(SOUND_MIXER_IGAIN)   , 
                    &volume);

   // close the mixer device
   ::close(mixer_fd); 

} // SetInputMixer()

//===========================================================================
//
//===========================================================================





