#include "recdescr.h"

#include <stdio.h>

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

// simple constructor
RecordDescription::RecordDescription()
{
   // fill in some empty data
   m_t_StartTime             = 0;
   m_t_EndTime               = 0;
   
   m_e_RecordingFormat       = ModeMp3;
   m_i_RecordingLevel        = 50;
   m_i_RecordingLevelMaster  = 50;
   m_i_MP3Rate               = 128;
   m_i_InputDevice           = SOUND_MIXER_LINE;
   
   m_b_UseAnalyzeMode        = false;  // currently not used !!
   
} // contructor

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

// copy constructor
RecordDescription::RecordDescription(RecordDescription &ExistingOne)
{
   // copy all elements
   *this = ExistingOne;

} // contructor

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

// simple destructor
RecordDescription::~RecordDescription()
{

} // nothing to destruct


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

bool RecordDescription::OverlapsWith(RecordDescription & c_OtherRecord)  
{

   if(m_t_EndTime < c_OtherRecord.m_t_StartTime)
   {
      return false;
   }
   if(c_OtherRecord.m_t_EndTime < m_t_StartTime)
   {
      return false;
   }
   
   // no, we have overlapping time !
   return true;


} // OverlapsWith( )

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordDescription::MergeRecords(RecordDescription & c_OtherRecord)  
{
           
   // take the lowest StartTime
   if(m_t_StartTime > c_OtherRecord.m_t_StartTime)
   {
      m_t_StartTime = c_OtherRecord.m_t_StartTime;
   }
   
   // take the highest endTime
   if(m_t_EndTime < c_OtherRecord.m_t_EndTime)
   {
      m_t_EndTime = c_OtherRecord.m_t_EndTime;
   }


} // MergeRecords( )

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

int RecordDescription::SaveToFile(FILE *fp_Outfile)
{
   if(fp_Outfile == NULL)
   {
      return 1;
   }

   fprintf(fp_Outfile,"%ld,",m_t_StartTime);
   fprintf(fp_Outfile,"%ld,",m_t_EndTime);
   fprintf(fp_Outfile,"%d," ,m_e_RecordingFormat);
   fprintf(fp_Outfile,"%d," ,m_i_RecordingLevel);
   fprintf(fp_Outfile,"%d," ,m_i_RecordingLevelMaster);
   fprintf(fp_Outfile,"%d," ,m_i_MP3Rate);
   fprintf(fp_Outfile,"%d" ,m_i_InputDevice);
   fprintf(fp_Outfile,"\n");
   return 0;
   
} // SaveToFile()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

int RecordDescription::LoadFromFile(FILE *fp_Infile)
{
   int i_ScanResult;
   if(fp_Infile == NULL)
   {
      return 1;
   }

   if(feof(fp_Infile))
   {
      return 2;
   }

   i_ScanResult=fscanf(fp_Infile,"%ld,",&m_t_StartTime);
   if(i_ScanResult<1)
   {
      return 3;
   }
   i_ScanResult=fscanf(fp_Infile,"%ld,",&m_t_EndTime);
   if(i_ScanResult<1)
   {
      return 3;
   }
   i_ScanResult=fscanf(fp_Infile,"%d," ,&m_e_RecordingFormat);
   if(i_ScanResult<1)
   {
      return 3;
   }
   i_ScanResult=fscanf(fp_Infile,"%d," ,&m_i_RecordingLevel);
   if(i_ScanResult<1)
   {
      return 3;
   }
   i_ScanResult=fscanf(fp_Infile,"%d," ,&m_i_RecordingLevelMaster);
   if(i_ScanResult<1)
   {
      return 3;
   }
   i_ScanResult=fscanf(fp_Infile,"%d," ,&m_i_MP3Rate);
   if(i_ScanResult<1)
   {
      return 3;
   }
   i_ScanResult=fscanf(fp_Infile,"%d"  ,&m_i_InputDevice);
   if(i_ScanResult<1)
   {
      return 3;
   }

   // everything is alright
   return 0;
   
} // LoadFromFile()



//===========================================================================
//===========================================================================
//===========================================================================
//===========================================================================

RecordDescriptionList::RecordDescriptionList()
{
   // automatic delete of unused items
   m_List.setAutoDelete(TRUE);
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

RecordDescriptionList::~RecordDescriptionList()
{
}


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

int RecordDescriptionList::ReadFromFile(FILE *fp_Infile)
{
   RecordDescription *pc_NewRecord;

   pc_NewRecord = new RecordDescription;
   while(pc_NewRecord != NULL)
   {
      int i_ReadResult;
      i_ReadResult = pc_NewRecord->LoadFromFile(fp_Infile);
      if(i_ReadResult != 0)
      {
         // read error, finish the loop
         
         // delete the empty description
         delete pc_NewRecord;
         
         break;
      }
      AddRecord(pc_NewRecord);     
      pc_NewRecord = new RecordDescription;
      
   } // while records in file

   return 0;
   
} // ReadFromFile()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

int RecordDescriptionList::ReadFromFile(QString qs_Filename)
{
   FILE *fp_Infile;
   char  c_Buffer[200];
   int   i_Result;

   i_Result =0;

   if(qs_Filename.isEmpty() )
   {
      return 1;
   }

   // now read the data
   fp_Infile = fopen(qs_Filename,"r");
   if(fp_Infile == NULL)
   {
      // perhaps a warning box ??
      
      return 2;
   }
   
   // read the first version line
   fgets(c_Buffer,sizeof(c_Buffer),fp_Infile);
   // TODO: check version 
   

   // delete the old data
   DeleteAll();

   // read all records
   i_Result = ReadFromFile(fp_Infile);     

   // close the file
   fclose(fp_Infile);


   return i_Result;
   
} // ReadFromFile()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

int RecordDescriptionList::WriteToFile(FILE *fp_Outfile)
{
   int i;
   int i_RecordCount;
   
   
   i_RecordCount = GetCount();
   for(i=0; i < i_RecordCount; i++)
   {
      RecordDescription *pc_Record;
      pc_Record = GetRecord(i);
      pc_Record->SaveToFile(fp_Outfile);
   }

   return 0;
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordDescriptionList::AddRecord(RecordDescription *pc_Description)
{
   // do not simply add it, but sort it
   // after the start time
   RecordDescription *pc_OldRecord ;
   int                i;

   for(i=0; i<m_List.count(); i++)
   {
      pc_OldRecord =  m_List.at(i);
   
      if(pc_OldRecord->m_t_StartTime > pc_Description->m_t_StartTime)
      {
         // insert new record before this position
         m_List.insert(i,pc_Description);         
         return;
      }
      
   } // for i
   
   // put new record at the end
   m_List.append(pc_Description);

} // AddRecord()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

int RecordDescriptionList::GetCount()
{
   return m_List.count();  

} // GetCount()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

RecordDescription *RecordDescriptionList::GetRecord(int i_Number)
{
   // is the number valid ?
   if(   i_Number <  0
      || i_Number >= m_List.count() )
   {
       // no, return empty record
       return NULL;
   }

   // return the element 
   return m_List.at(i_Number);
   
} // GetRecord()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void RecordDescriptionList::DeleteRecord(int i_Delete)
{
   RecordDescription *pc_Record;
   // is the number valid ?
   if(   i_Delete <  0
      || i_Delete >= m_List.count() )
   {
       // no, return 
       return ;
   }
   
   // get contents of the ptr
   pc_Record = m_List.at(i_Delete);
   
   // remove from list
   m_List.remove((unsigned int) i_Delete);
   
   
   if(! m_List.autoDelete() ) 
   {
      // remove element data
      delete pc_Record;
   }

} // DeleteRecord()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void RecordDescriptionList::DeleteAll()
{
   RecordDescription *pc_Record;
   int                i_Count;
   int                i;
   
   // get the count
   i_Count =  m_List.count();
   
   // delete from the back
   for(i=i_Count -1; i>=0 ; i--)
   {
      // get contents of the ptr
      pc_Record = m_List.at(i);
   
      // remove from list
      m_List.remove((unsigned int) i);
   
      // free element stiorage
      if(! m_List.autoDelete() ) 
      {
         // remove element data
         delete pc_Record;
      }
      
   } // for i
   

} // DeleteAll()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------



bool RecordDescriptionList::HasOverlappingTime(RecordDescription *pc_NewRecord,
   			int i_ExistingRecordIndex)
{
   RecordDescription *pc_OldRecord ;

   if(   i_ExistingRecordIndex <  0
      || i_ExistingRecordIndex >= m_List.count() )
   {
       // no, return false
       return false;
   }
   
   
   pc_OldRecord =  m_List.at(i_ExistingRecordIndex);
   if(pc_OldRecord == NULL)
   {
      return false;
   }
   
   return pc_OldRecord->OverlapsWith(*pc_NewRecord);   
   
   

} // HasOverlappingTime()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

bool RecordDescriptionList::TestForOverlappingTime(RecordDescription *pc_NewRecord)
{
   RecordDescription *pc_OldRecord ;
   int i;
   int i_Number;
   
   
   i_Number = m_List.count();
   
   for(i=0 ; i< i_Number; i ++)
   {
       if( HasOverlappingTime( pc_NewRecord,i) )
       {
          return true;
       }
   }
   
   return false;


} // TestForOverlappingTime()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordDescriptionList::MergeRecord(RecordDescription *pc_Description)
{

   RecordDescription *pc_OldRecord ;
   RecordDescription *pc_NextRecord ;
   int                i;

   for(i=0; i<m_List.count(); i++)
   {
      pc_OldRecord =  m_List.at(i);

      if( pc_OldRecord->OverlapsWith(*pc_Description))
      {
         // found first overlapping
         
         // merge old with new
         pc_OldRecord->MergeRecords(*pc_Description);
         
         
         // now look, whether the new record overlaps with following
         // records
         
         
         while(i+1<m_List.count() )
         {
            pc_NextRecord =  m_List.at(i+1);
            if( pc_OldRecord->OverlapsWith(*pc_NextRecord))
            {
               // merge with next
               pc_OldRecord->MergeRecords(*pc_NextRecord);
               
               // remove the next record
               m_List.remove(i+1);
               
               // and delete it
               if(! m_List.autoDelete())
               {
                  delete pc_NextRecord;
               }
            }
            else
            {
               // break the loop
               break;
            }

         } // if following records available

      } // if overlaps

      
   } // for i
   

} // MergeRecord()

//-----------------------------------------------------------------------------
// create dummy data for testing the timer function
//-----------------------------------------------------------------------------

void RecordDescriptionList::CreateTestDataset()
{
   time_t t_ActualTime;
   RecordDescription *pc_NewRecord ;
   
   time(&t_ActualTime);
   
   pc_NewRecord = new RecordDescription();
   pc_NewRecord-> m_t_StartTime = t_ActualTime+10;
   pc_NewRecord-> m_t_EndTime   = t_ActualTime+20;
   pc_NewRecord-> m_i_RecordingLevel = 66;
   AddRecord(pc_NewRecord);
  
   pc_NewRecord = new RecordDescription();
   pc_NewRecord-> m_t_StartTime = t_ActualTime+30;
   pc_NewRecord-> m_t_EndTime   = t_ActualTime+40;
   pc_NewRecord-> m_i_MP3Rate   = 256;
   AddRecord(pc_NewRecord);
   
   pc_NewRecord = new RecordDescription();
   pc_NewRecord-> m_t_StartTime = t_ActualTime+50;
   pc_NewRecord-> m_t_EndTime   = t_ActualTime+60;
   pc_NewRecord-> m_i_InputDevice = SOUND_MIXER_MIC;
   AddRecord(pc_NewRecord);
   
} // CreateTestDataset()

//-----------------------------------------------------------------------------
// delete times, which have already expired
//-----------------------------------------------------------------------------

void RecordDescriptionList::DeleteOldTimes()
{
   RecordDescription *pc_Record ;
   time_t             t_ActualTime;
   int                i_ActualIndex;

   time(&t_ActualTime);

   i_ActualIndex =0;
   while(i_ActualIndex < m_List.count() )
   {
   
      pc_Record =  m_List.at(i_ActualIndex);
      
      if(pc_Record->m_t_StartTime < t_ActualTime)
      {
         DeleteRecord(i_ActualIndex);
      }
      else
      {
         i_ActualIndex += 1;
      }
   } // while

} //  DeleteOldTimes()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------




