// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <time.h>

#include "recordlistbox.h"
#include <qdialog.h>
#include <qpushbutton.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qdir.h>
#include <qmessagebox.h>

#include "hdrecord.h"

#include <stdio.h>

//-----------------------------------------------------------------------------

RecordListBox::RecordListBox(QWidget *parent,
                        HdRecord *theApplication) : 
		QDialog(parent,"Settings",TRUE)
{
   QPushButton  *CloseButton;
   QBoxLayout   *topLayout;
   QBoxLayout   *SectionLayout;
   QBoxLayout   *ActionButtonLayout;
   QBoxLayout   *LevelMasterBoxLayout;
   QBoxLayout   *LevelBoxLayout;
   QBoxLayout   *ButtonLayout;
   QGridLayout  *GridLayout;
   QWidget      *SectionWidget;
   QWidget      *ActionButtonWidget;
   QWidget      *GridWidget;
   QWidget      *LevelMasterBoxWidget;
   QWidget      *LevelBoxWidget;
   QWidget      *ButtonWidget;
   int           i_CharWidth;
   int           i_EditWidth ;
   
   
   p_TheApplication = theApplication;
   setCaption("KHdRecord Recording List");

   {
      QFontMetrics  p_FontMetrics(this->font());
      
      i_CharWidth = p_FontMetrics.maxWidth();
      
      // 13 of the widest characters
      i_EditWidth = i_CharWidth * 13 ;   
      
   }



   topLayout = new QVBoxLayout( this, 2 );
   SectionWidget = new QWidget(this);
   topLayout->addWidget(SectionWidget);
   SectionLayout = new QHBoxLayout( SectionWidget, 2 );

   GridWidget = new QWidget(SectionWidget);
   SectionLayout->addWidget(GridWidget);
   GridLayout = new QGridLayout( GridWidget, 7, 2, 10 );

   // input fields for date and time
   m_StartLineEdit = new QDateTimeEdit(GridWidget);
   m_StartLineEdit->dateEdit()->setOrder(QDateEdit::YMD);

   m_StopLineEdit  = new QDateTimeEdit(GridWidget);
   m_StopLineEdit->dateEdit()->setOrder(QDateEdit::YMD);

   m_RecordingTypeCombo = new QComboBox(false,GridWidget,"RecordingTypeCombo");
   m_RecordingTypeCombo->insertItem("WAV",0);
   m_RecordingTypeCombo->insertItem("MP3",1);
   m_RecordingTypeCombo->insertItem("OGG",2);

   LevelBoxWidget = new QWidget(GridWidget);
   LevelBoxLayout = new QHBoxLayout( LevelBoxWidget, 2 );
   
   m_SampleLevelSlider = new QSlider(0,100, 			// min/max
                                      1,     			// pageStep, 
                                      50,       		//int value, 
                                      QSlider::Horizontal, 	// Orientation, 
                                      LevelBoxWidget);
   m_SampleLevelLabel       = new QLabel("50",LevelBoxWidget);  
   LevelBoxLayout->addWidget( m_SampleLevelSlider );
   LevelBoxLayout->addWidget( m_SampleLevelLabel );
   
   LevelMasterBoxWidget = new QWidget(GridWidget);
   LevelMasterBoxLayout = new QHBoxLayout( LevelMasterBoxWidget, 2 );
   
   m_SampleLevelMasterSlider = new QSlider(0,100, 			// min/max
                                      1,     			// pageStep, 
                                      50,       		//int value, 
                                      QSlider::Horizontal, 	// Orientation, 
                                      LevelMasterBoxWidget);
   m_SampleLevelMasterLabel  = new QLabel("50",LevelMasterBoxWidget);  
   LevelMasterBoxLayout->addWidget( m_SampleLevelMasterSlider );
   LevelMasterBoxLayout->addWidget( m_SampleLevelMasterLabel );
                                      
                                      
                                      
   m_MP3BitrateCombo = new QComboBox(false,GridWidget,"MP3BitrateCombo");
   m_MP3BitrateCombo->insertItem("128",0);
   m_MP3BitrateCombo->insertItem("160",1);
   m_MP3BitrateCombo->insertItem("192",2);
   m_MP3BitrateCombo->insertItem("256",3);
   m_MP3BitrateCombo->insertItem("40/Mono",4);
   m_MP3BitrateCombo->insertItem("320",5);

   m_InputDeviceCombo = new QComboBox(false,GridWidget,"InputDeviceCombo");
   m_InputDeviceCombo->insertItem("Line"      ,0);
   m_InputDeviceCombo->insertItem("Microphone",1);
   m_InputDeviceCombo->insertItem("CD"        ,2);

   GridLayout->addWidget(new QLabel("Start",GridWidget)         ,0,0);
   GridLayout->addWidget(m_StartLineEdit                          ,0,1);
   GridLayout->addWidget(new QLabel("Stop",GridWidget)          ,1,0);
   GridLayout->addWidget(m_StopLineEdit                           ,1,1);
   GridLayout->addWidget(new QLabel("Recording Type",GridWidget),2,0);
   GridLayout->addWidget(m_RecordingTypeCombo                     ,2,1);
   GridLayout->addWidget(new QLabel("Input Level",GridWidget)   ,3,0);
   GridLayout->addWidget(LevelBoxWidget                         ,3,1);
   GridLayout->addWidget(new QLabel("Master Level",GridWidget)   ,4,0);
   GridLayout->addWidget(LevelMasterBoxWidget                    ,4,1);
   GridLayout->addWidget(new QLabel("Encode Bitrate",GridWidget)   ,5,0);
   GridLayout->addWidget(m_MP3BitrateCombo                        ,5,1);
   GridLayout->addWidget(new QLabel("Input Device",GridWidget)  ,6,0);
   GridLayout->addWidget(m_InputDeviceCombo                       ,6,1);


   ActionButtonWidget = new QWidget(SectionWidget,"ActionButtonWidget");
   SectionLayout->addWidget(ActionButtonWidget);
   ActionButtonLayout = new QVBoxLayout( ActionButtonWidget, 2 );
   m_AddButton = new QPushButton("Add",ActionButtonWidget);
   m_ChangeButton = new QPushButton("Change",ActionButtonWidget);
   m_DeleteButton = new QPushButton("Delete",ActionButtonWidget);
   ActionButtonLayout->addWidget( m_AddButton);   
   ActionButtonLayout->addWidget( m_ChangeButton);   
   ActionButtonLayout->addWidget( m_DeleteButton);   


   m_ListBox = new QListBox(SectionWidget);
   m_ListBox ->setMinimumWidth(  i_EditWidth );
   SectionLayout->addWidget(m_ListBox);

   ButtonWidget = new QWidget(this);
   topLayout->addWidget(ButtonWidget);
   ButtonLayout = new QHBoxLayout( ButtonWidget, 2 );

   m_SaveButton = new QPushButton( "Save to File"  , ButtonWidget );
   m_LoadButton = new QPushButton( "Load from File", ButtonWidget );
   CloseButton  = new QPushButton( "OK"            , ButtonWidget );
   connect( CloseButton, SIGNAL(clicked()), SLOT(ClosePressed()) );


   ButtonLayout->addWidget(m_SaveButton);
   ButtonLayout->addWidget(m_LoadButton);
   ButtonLayout->addWidget(CloseButton);


   connect( m_SampleLevelSlider,  
               SIGNAL(valueChanged(int)), 
               SLOT(DoLevelSliderValueChanged(int)) );
   connect( m_SampleLevelMasterSlider,  
               SIGNAL(valueChanged(int)), 
               SLOT(DoLevelSliderValueChanged(int)) );


   connect( m_StartLineEdit,
               SIGNAL(valueChanged(const QDateTime &)),
               SLOT(DoStartTimeChanged(const QDateTime &)) );

   connect( m_StopLineEdit,
               SIGNAL(valueChanged(const QDateTime &)),
               SLOT(DoStopTimeChanged(const QDateTime &)) );

   connect( m_AddButton,    SIGNAL(clicked()), SLOT(DoAddButtonPressed()) );
   connect( m_ChangeButton, SIGNAL(clicked()), SLOT(DoChangeButtonPressed()) );
   connect( m_DeleteButton, SIGNAL(clicked()), SLOT(DoDeleteButtonPressed()) );
   connect( m_LoadButton,   SIGNAL(clicked()), SLOT(DoLoadButtonPressed()) );
   connect( m_SaveButton,   SIGNAL(clicked()), SLOT(DoSaveButtonPressed()) );
   connect( m_ListBox,      SIGNAL(selectionChanged())  ,SLOT(ListBoxSelectionChanged()) );

   // check, whether daylight saving time is on.
   {
      struct tm *t_TimeStruct;
      time_t t_TheTime;
      t_TheTime= time(NULL);
      t_TimeStruct= localtime (& t_TheTime );
      m_i_DaylightSavingTimeActiv = t_TimeStruct->tm_isdst;
   }




   // set the default values
   SetDefaultValues();


   m_DefaultBackgroundColor = m_StartLineEdit->backgroundColor();
   m_ErrorBackgroundColor   = QColor(255,220,200);
   

   // refresh the output list
   RefreshDisplay();

   // en/disable buttons
   SetButtonSensitivity();
   
   
} // RecordListBox() constructor

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::ClosePressed()
{
   // finish the dialog
   accept();
} // ClosePressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::NextHour (struct tm *t_Time)
{
   time_t NextHour;
   struct tm t_Next;
   t_Time->tm_sec=0;
   t_Time->tm_min=0;
   
   NextHour = mktime(t_Time);
   // add one hour
   NextHour += (60*60);	
   
   memcpy( &t_Next,localtime(&NextHour),sizeof(struct tm));
   memcpy(t_Time,&t_Next,sizeof(struct tm));   


} // NextHour

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::SetDefaultValues()
{
   // set the record mode 
   if(p_TheApplication->e_RecordMode == ModeMp3)
   {
      // set to mp3
      m_RecordingTypeCombo->setCurrentItem(1);
   }
   else if( p_TheApplication->e_RecordMode == ModeWave)
   {
      // set to wav
      m_RecordingTypeCombo->setCurrentItem(0);
   }
   else if( p_TheApplication->e_RecordMode == ModeOgg)
   {
      // set to ogg
      m_RecordingTypeCombo->setCurrentItem(2);
   }
   

   switch(p_TheApplication->m_i_MP3Bitsize)
   {
      case 128:
         // bitrate to 128
         m_MP3BitrateCombo->setCurrentItem( 0);
         break;
      case 160:
         // bitrate to 160
         m_MP3BitrateCombo->setCurrentItem( 1);
         break;
      case 192:
         // bitrate to 192
         m_MP3BitrateCombo->setCurrentItem( 2);
         break;
      case 256:
         // bitrate to 256
         m_MP3BitrateCombo->setCurrentItem( 3);
         break;
      case 40:
         // bitrate to 40
         m_MP3BitrateCombo->setCurrentItem( 4);
         break;
      case 320:
         // bitrate to 320
         m_MP3BitrateCombo->setCurrentItem( 5);
         break;
      default:
         break;
   }
   
   
   switch(p_TheApplication->m_i_MixerInputDevice)
   {
      case SOUND_MIXER_LINE:
         m_InputDeviceCombo->setCurrentItem(0);
         break;
      case SOUND_MIXER_MIC:
         m_InputDeviceCombo->setCurrentItem(1);
         break;
      case SOUND_MIXER_CD:
         m_InputDeviceCombo->setCurrentItem(2);
         break;
   }
   
   // input level 
   m_SampleLevelSlider->setValue(p_TheApplication->m_i_SampleLevel);
   
   // master input level 
   m_SampleLevelMasterSlider->setValue(p_TheApplication->m_i_SampleLevelMaster);
   SetSliderLevelLabel();
   
   

   // set a time
   {

       time_t t_TheTime;
       struct tm t_LocalTime;
       char   ca_TimeOutput[100];
       int i_Hour;
       int i_Minute;
       int i_Second;
       int i_Day;
       int i_Month;
       int i_Year;

       QDateTime q_Datetime;

       time(&t_TheTime);

       memcpy(&t_LocalTime,localtime(&t_TheTime),sizeof(struct tm));

       NextHour(&t_LocalTime);

       q_Datetime.setDate(QDate(t_LocalTime.tm_year+1900,
                                t_LocalTime.tm_mon +1,
				t_LocalTime.tm_mday
                          ));
       q_Datetime.setTime(QTime(t_LocalTime.tm_hour,
                                t_LocalTime.tm_min ,
				t_LocalTime.tm_sec
                          ));


       m_StartLineEdit->setDateTime(q_Datetime);

       // again one hour in future
       NextHour(&t_LocalTime);
       q_Datetime.setDate(QDate(t_LocalTime.tm_year+1900,
                                t_LocalTime.tm_mon +1,
				t_LocalTime.tm_mday
                          ));
       q_Datetime.setTime(QTime(t_LocalTime.tm_hour,
                                t_LocalTime.tm_min ,
				t_LocalTime.tm_sec
                          ));


       m_StopLineEdit->setDateTime(q_Datetime);

       m_b_StartTimeValid = true;
       m_b_StopTimeValid = true;

   }
   
} // SetDefaultValues()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::SetSliderLevelLabel()
{
   int i_Value;
   QString qs_Label;
   
   i_Value = m_SampleLevelSlider->value();
   
   qs_Label.sprintf("%3d",i_Value);
   
   m_SampleLevelLabel->setText(qs_Label);

   i_Value = m_SampleLevelMasterSlider->value();
   
   qs_Label.sprintf("%3d",i_Value);
   
   m_SampleLevelMasterLabel->setText(qs_Label);



} // SetSliderLevelLabel

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::DoLevelSliderValueChanged(int)
{
   SetSliderLevelLabel();
}

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::DoStartTimeChanged(const QDateTime & qs_Time)
{

   m_t_Start = qs_Time.toTime_t();
   m_b_StartTimeValid = true;


   // en/disable buttons
   SetButtonSensitivity();


} // DoStartTimeChanged()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::DoStopTimeChanged(const QDateTime & qs_Time)
{
   m_t_Stop = qs_Time.toTime_t();
   m_b_StopTimeValid = true;

   // en/disable buttons
   SetButtonSensitivity();

} // DoStopTimeChanged()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::DoAddButtonPressed()
{
   RecordDescription *pc_NewRecord;
   bool               b_HasOverlappingTime;
   int                i_Item;

   // start and stop time must be valid
   if(   ! m_b_StopTimeValid
      || ! m_b_StartTimeValid )
   {
      return;
   }
   
   pc_NewRecord = new RecordDescription();

   // get display values and fill into the new record   
   CopyDisplayValuesIntoRecord(pc_NewRecord);
   
   // test for overlapping time
   b_HasOverlappingTime = p_TheApplication->m_p_RecordList.
                                    TestForOverlappingTime(pc_NewRecord);
   if(b_HasOverlappingTime)
   {
      int i_WarningResult;      
      i_WarningResult = QMessageBox::warning( this, 
                                              "KHdRecord:Add Time",
                                "The time overlaps with an already used one\n"
                          "What do you want ?",
                          "Merge Times", "Abort", NULL,
                          0, 1 );
    
      
      if(i_WarningResult == 1)
      {
         // abort
         
         // delete the record
         delete pc_NewRecord;
         
         return;
      }

   }   // block for warning box       
   

   // the record has been created 
   // now add it to the list

   if(b_HasOverlappingTime)
   {
      // merge the times
      p_TheApplication->m_p_RecordList.MergeRecord(pc_NewRecord);
   }   
   else
   {
      // add the record
      p_TheApplication->m_p_RecordList.AddRecord(pc_NewRecord);
   }

  
   // refresh the output list
   RefreshDisplay();

} // DoAddButtonPressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::DoChangeButtonPressed()
{
   RecordDescription * pc_ActualRecord;
   RecordDescription * pc_CopyOfRecord;
   RecordDescription * pc_NewRecord;
   bool               b_HasOverlappingTime;
   int                i_Item;

  
   // get the index of the selected item
   m_i_CurrentSelected = m_ListBox->currentItem();
   
   // get the record
   pc_ActualRecord = p_TheApplication->m_p_RecordList.GetRecord(m_i_CurrentSelected);

   // no current record selected
   if(pc_ActualRecord == NULL)
   {
      // then return
      return;
   }
   pc_CopyOfRecord = new RecordDescription( *pc_ActualRecord);   
   p_TheApplication->m_p_RecordList.DeleteRecord(m_i_CurrentSelected);
   
   
   pc_NewRecord = new RecordDescription();

   // get display values and fill into the new record   
   CopyDisplayValuesIntoRecord(pc_NewRecord);
   
   // test for overlapping time
   b_HasOverlappingTime = p_TheApplication->m_p_RecordList.
                                    TestForOverlappingTime(pc_NewRecord);
   if(b_HasOverlappingTime)
   {
      int i_WarningResult;      
      i_WarningResult = QMessageBox::warning( this, 
                                              "KHdRecord:Add Time",
                                "The time overlaps with an already used one\n"
                          "What do you want ?",
                          "Merge Times", "Abort", NULL,
                          0, 1 );

      if(i_WarningResult == 1)
      {
         // abort
         
         // reinstall the original one
         p_TheApplication->m_p_RecordList.AddRecord(pc_CopyOfRecord);
         
         return;
      }

   }   // block for warning box       
   
   // the copy is not needed anymore
   delete pc_CopyOfRecord;
   

   // the record has been created 
   // now add it to the list

   if(b_HasOverlappingTime)
   {
      // merge the times
      p_TheApplication->m_p_RecordList.MergeRecord(pc_NewRecord);
   }   
   else
   {
      // add the record
      p_TheApplication->m_p_RecordList.AddRecord(pc_NewRecord);
   }

   RefreshDisplay();
   

} // DoChangeButtonPressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::DoDeleteButtonPressed()
{
   int i_DeleteItem;
   
   i_DeleteItem = m_ListBox->currentItem();
   
   if(   i_DeleteItem>=0
      && i_DeleteItem == m_i_CurrentSelected)
   {
      // delete the item
      
      p_TheApplication->m_p_RecordList.DeleteRecord(i_DeleteItem);
      m_ListBox->removeItem(i_DeleteItem);   
      
      // refresh the output list
      // CopyRecordsIntoDisplayList();
   
      // disable the current item
      m_ListBox->clearSelection();
      m_i_CurrentSelected = -1;
   }
   
   // en/disable buttons		
   SetButtonSensitivity();

} // DoDeleteButtonPressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::CopyRecordsIntoDisplayList()
{
   int i;
   int i_Records;
   
   // delete the old contents
   m_ListBox->clear();
   
   // get the number of records
   i_Records = p_TheApplication->m_p_RecordList.GetCount();
   for(i =0; i<i_Records; i++)
   {
       RecordDescription *pc_ActualRecord;
       struct tm         t_LocalTime;
       QString            qs_OutputString;
       char               ca_TimeOutput[40];
       
       // get the record
       pc_ActualRecord = p_TheApplication->m_p_RecordList.GetRecord(i);
       if(pc_ActualRecord == NULL)
       {
          break;
          
       }
       
       // get the start time
       memcpy(&t_LocalTime,
              localtime(&(pc_ActualRecord->m_t_StartTime)),
//              gmtime(&(pc_ActualRecord->m_t_StartTime)),
              sizeof(struct tm));
              
       // convert time to ascii
       strftime(ca_TimeOutput,sizeof(ca_TimeOutput)-1,
                "%Y-%m-%d %H:%M:%S",
                &t_LocalTime);
       
       qs_OutputString.sprintf("%s",ca_TimeOutput);
       
       // put time as item into the list
       m_ListBox->insertItem(qs_OutputString);
       
   } // for i

} // CopyRecordsIntoDisplayList()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::ListBoxSelectionChanged()
{
   RecordDescription * pc_ActualRecord;
   // fprintf(stderr,"Listbox selection changed %d\n",m_ListBox->currentItem());
   
   // get the index of the selected item
   m_i_CurrentSelected = m_ListBox->currentItem();

   
   // get the record
   pc_ActualRecord = p_TheApplication->m_p_RecordList.GetRecord(m_i_CurrentSelected);
   if(pc_ActualRecord != NULL)
   {
      // transfer record data into the display
      SetDisplayValuesFromRecord(pc_ActualRecord);
   }
   
   // change the button sensitivity
   SetButtonSensitivity();

} // ListBoxSelectionChanged()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::SetDisplayValuesFromRecord(RecordDescription *pc_Record)
{
   if(pc_Record == NULL)
   {
      return;
   }
   
   // set the record mode 
   if(pc_Record->m_e_RecordingFormat == ModeMp3)
   {
      // set to mp3
      m_RecordingTypeCombo->setCurrentItem(1);
   }
   else if( pc_Record->m_e_RecordingFormat == ModeWave)
   {
      // set to wav
      m_RecordingTypeCombo->setCurrentItem(0);
   }
   else if( pc_Record->m_e_RecordingFormat == ModeOgg)
   {
      // set to ogg
      m_RecordingTypeCombo->setCurrentItem(2);
   }
   

   switch(pc_Record->m_i_MP3Rate)
   {
      case 128:
         // bitrate to 128
         m_MP3BitrateCombo->setCurrentItem( 0);
         break;
      case 160:
         // bitrate to 160
         m_MP3BitrateCombo->setCurrentItem( 1);
         break;
      case 192:
         // bitrate to 192
         m_MP3BitrateCombo->setCurrentItem( 2);
         break;
      case 256:
         // bitrate to 256
         m_MP3BitrateCombo->setCurrentItem( 3);
         break;
      case 40:
         // bitrate to 40
         m_MP3BitrateCombo->setCurrentItem( 4);
         break;
      case 320:
         // bitrate to 320
         m_MP3BitrateCombo->setCurrentItem( 5);
         break;
      default:
         break;
   }
   
   
   switch(pc_Record->m_i_InputDevice)
   {
      case SOUND_MIXER_LINE:
         m_InputDeviceCombo->setCurrentItem(0);
         break;
      case SOUND_MIXER_MIC:
         m_InputDeviceCombo->setCurrentItem(1);
         break;
      case SOUND_MIXER_CD:
         m_InputDeviceCombo->setCurrentItem(2);
         break;
   }
   
   // input level 
   m_SampleLevelSlider->setValue(pc_Record->m_i_RecordingLevel);
   
   // input level 
   m_SampleLevelMasterSlider->setValue(pc_Record->m_i_RecordingLevelMaster);
   SetSliderLevelLabel();
   
   
   // set a time
   {
       QDateTime q_Datetime;
       time_t t_TheTime;
       struct tm t_LocalTime;
       char   ca_TimeOutput[100];
       int i_Hour;
       int i_Minute;
       int i_Second;
       int i_Day;
       int i_Month;
       int i_Year;

       // transfer the star time
       t_TheTime = pc_Record->m_t_StartTime;


       memcpy(&t_LocalTime,localtime(&t_TheTime),sizeof(struct tm));


       q_Datetime.setDate(QDate(t_LocalTime.tm_year+1900,
                                t_LocalTime.tm_mon +1,
				t_LocalTime.tm_mday
                          ));
       q_Datetime.setTime(QTime(t_LocalTime.tm_hour,
                                t_LocalTime.tm_min ,
				t_LocalTime.tm_sec
                          ));


       m_StartLineEdit->setDateTime(q_Datetime);


       // do the same for the end time
       t_TheTime = pc_Record->m_t_EndTime;


       memcpy(&t_LocalTime,localtime(&t_TheTime),sizeof(struct tm));

       q_Datetime.setDate(QDate(t_LocalTime.tm_year+1900,
                                t_LocalTime.tm_mon +1,
				t_LocalTime.tm_mday
                          ));
       q_Datetime.setTime(QTime(t_LocalTime.tm_hour,
                                t_LocalTime.tm_min ,
				t_LocalTime.tm_sec
                          ));


       m_StopLineEdit->setDateTime(q_Datetime);


       // set the internal time after the record time
       m_t_Start = pc_Record->m_t_StartTime;
       m_t_Stop  = pc_Record->m_t_EndTime;

       // set the time status to be valid
       m_b_StartTimeValid = true;
       m_b_StopTimeValid  = true;

   } // block for the time



} // SetDisplayValuesFromRecord()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::CopyDisplayValuesIntoRecord(RecordDescription *pc_Record)
{
   int i_Item;

   // we need a record
   if(pc_Record == NULL)
   {
       return;
   }


   // start and stop time must be valid
   if(   ! m_b_StopTimeValid
      || ! m_b_StartTimeValid )
   {
      return;
   }
   
   // copy start and end times
   pc_Record->m_t_StartTime = m_t_Start;
   pc_Record->m_t_EndTime   = m_t_Stop ;
   
   
   i_Item = m_RecordingTypeCombo->currentItem();   
   if(i_Item ==0)
   {
      pc_Record->m_e_RecordingFormat = ModeWave;
   }
   else if (i_Item == 1 )
   {
      pc_Record->m_e_RecordingFormat = ModeMp3;
   }
   else if (i_Item == 2 )
   {
      pc_Record->m_e_RecordingFormat = ModeOgg;
   }


   i_Item = m_MP3BitrateCombo->currentItem();
   switch(i_Item)
   {
      case 0:
         pc_Record->m_i_MP3Rate = 128;
         break;
      case 1:
         pc_Record->m_i_MP3Rate = 160;
         break;
      case 2:
         pc_Record->m_i_MP3Rate = 192;
         break;
      case 3:
         pc_Record->m_i_MP3Rate = 256;
         break;
      case 4:
         pc_Record->m_i_MP3Rate = 40;
         break;
      case 5:
         pc_Record->m_i_MP3Rate = 320;
         break;
   }
   
   i_Item = m_InputDeviceCombo->currentItem();
   switch(i_Item)
   {
      case 0:
         pc_Record->m_i_InputDevice = SOUND_MIXER_LINE;
         break;
      case 1:
         pc_Record->m_i_InputDevice = SOUND_MIXER_MIC;
         break;
      case 2:
         pc_Record->m_i_InputDevice = SOUND_MIXER_CD;
         break;
   }

   pc_Record->m_i_RecordingLevel = m_SampleLevelSlider->value();
   pc_Record->m_i_RecordingLevelMaster = m_SampleLevelMasterSlider->value();


} // CopyDisplayValuesIntoRecord


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::DoSaveButtonPressed()
{
   QString qs_Filename;
   FILE    *fp_Outfile;
   int     i;
   int     i_RecordCount;
   
   qs_Filename = GetFilename( false );
   if(qs_Filename.isEmpty() )
   {
      return;
   }
   
   // now save the data
   fp_Outfile = fopen(qs_Filename,"w");
   if(fp_Outfile == NULL)
   {
      // perhaps a warning box ??
      
      return;
   }
   
   fprintf(fp_Outfile,"KHdRecord:RecordingList:Version=1\n");
   
   i_RecordCount = p_TheApplication->m_p_RecordList.GetCount();
   for(i=0; i < i_RecordCount; i++)
   {
      RecordDescription *pc_Record;
      pc_Record = p_TheApplication->m_p_RecordList.GetRecord(i);
      pc_Record->SaveToFile(fp_Outfile);
   }


   // close the file
   fclose(fp_Outfile);
   
} // DoSaveButtonPressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void RecordListBox::DoLoadButtonPressed()
{
   QString   qs_Filename;
   FILE      *fp_Infile;
   char      c_Buffer[200];
   
   qs_Filename = GetFilename( true );
   if(qs_Filename.isEmpty() )
   {
      return;
   }

   // now read the data
   fp_Infile = fopen(qs_Filename,"r");
   if(fp_Infile == NULL)
   {
      // perhaps a warning box ??
      
      return;
   }
   
   // read the first version line
   fgets(c_Buffer,sizeof(c_Buffer),fp_Infile);
   // TODO: check version 
   

   // delete the old data
   p_TheApplication->m_p_RecordList.DeleteAll();

   // read all records
   p_TheApplication->m_p_RecordList.ReadFromFile(fp_Infile);     


   // close the file
   fclose(fp_Infile);

   RefreshDisplay();


} // DoLoadButtonPressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


QString RecordListBox::GetFilename(bool b_LoadFile)
{

   QFileDialog *fileDialog;
   QString     qs_BaseName;
   QString     qs_ReturnValue;
   
   int         i_FileDialogReturn;

   if(b_LoadFile)
   {
         qs_ReturnValue = QFileDialog::getOpenFileName( QString::null, 
         				"Recording List (*.list)", 
         				this ) ;


   }
   else // save
   {
   
      qs_ReturnValue = QFileDialog::getSaveFileName( "KHdRecord.list", 
                                       "Recording List (*.list)", 
                                       this );
   } // else save
   

   return qs_ReturnValue;

} // GetFilename()

//-----------------------------------------------------------------------------
// enable or disable the add, change and delete button depending on 
// several states
//-----------------------------------------------------------------------------

void RecordListBox::SetButtonSensitivity()
{
   int i_DeleteItem;
   
   // the times must be valid, and start before stop
   if(   m_b_StartTimeValid 
      && m_b_StopTimeValid
      && m_t_Start < m_t_Stop)
   {
      m_AddButton   ->setEnabled(true);
      m_ChangeButton->setEnabled(true);
   }
   else
   {
      m_AddButton   ->setEnabled(false);
      m_ChangeButton->setEnabled(false);
   }

   i_DeleteItem = m_ListBox->currentItem();
   
   if(   i_DeleteItem>=0
      && i_DeleteItem == m_i_CurrentSelected)
   {
      m_DeleteButton->setEnabled(true);
   }
   else
   {
      m_DeleteButton->setEnabled(false);
   }
   
   
} // SetButtonSensitivity()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


void RecordListBox::RefreshDisplay()
{
   // refresh the output list
   CopyRecordsIntoDisplayList();
   
   // disable the current item
   m_ListBox->clearSelection();
   m_i_CurrentSelected = -1;

   // en/disable buttons		
   SetButtonSensitivity();

} // RefreshDisplay()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


