// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//
// sample level output widget

#include <stdio.h>


#include <qpainter.h>

#include "samplearea.h"


//=========================================================================
// constructor
//=========================================================================

SampleArea::SampleArea(QWidget *parent)
		: QWidget(parent,"SampleArea")
{
   int i;

   // set the dimension of the area
   setMinimumSize(SAMPLE_AREA_VALUES,32);
   setMaximumSize(SAMPLE_AREA_VALUES,32);
   i_Width = width();
   i_Height = height();
   
   // allocate both colors
   pc_WhiteColor = new QColor(0xff,0xff,0xff);
   pc_BlackColor = new QColor(0x00,0x00,0x00);
   
   // allocate a brush
   pc_Brush      = new QBrush(*pc_WhiteColor);
   
   
   
   for(i=0; i<SAMPLE_AREA_VALUES; i++)
   {
      us_Values[i] = 16;
   }
   
   b_Stopped = false;
}


//=========================================================================
//
//=========================================================================

SampleArea::~SampleArea()
{
   // delete the brush
   pc_Brush      = new QBrush(*pc_WhiteColor);

   // delete the colors
   delete pc_WhiteColor ;
   delete pc_BlackColor ;
   
} // ~SampleArea()

//=========================================================================
// repaint the area.
// if theEvent is NULL, the function was called from the programm
// if theEvent is valid, it is called by the X-System
//=========================================================================
		
void  SampleArea::paintEvent(QPaintEvent *theEvent)
{
   QPainter painter(this);
   int      i;

   // if user redraw, but stopped, do nothing
   if(   theEvent ==NULL 
      && b_Stopped)
   {
      return;
   }
   
   
   // if system redraw (expose event), but stopped
   // draw a cross.
   if(b_Stopped)
   {
      pc_Brush->setColor(*pc_WhiteColor);
      painter.fillRect(0,0,i_Width,i_Height,*pc_Brush);

      pc_Brush->setColor(*pc_BlackColor);
      painter.drawLine(0,0,i_Width,i_Height);
      painter.drawLine(i_Width,0,0,i_Height);

      return;

   }

   // -------- regular case: draw the samples

   // fill the background
   pc_Brush->setColor(*pc_WhiteColor);
   painter.fillRect(0,0,i_Width,i_Height,*pc_Brush);

   // draw a sample line
   pc_Brush->setColor(*pc_BlackColor);
   for(i=0;i<SAMPLE_AREA_VALUES;i++)
   {
      int i_YValue;

      i_YValue = us_Values[i];

      painter.drawLine(i,0,i,i_YValue);
   }

} // paintEvent()

//=========================================================================
//
//=========================================================================


void  SampleArea::resizeEvent(QResizeEvent *theEvent)
{
   i_Width  = width();
   i_Height = height();
}


//=========================================================================
//
//=========================================================================

void  SampleArea::mousePressEvent(QMouseEvent *theEvent)
{

   // on a mouse event toggle the stopped state
   b_Stopped = ! b_Stopped;
   
}


//=========================================================================
// set a new array of sample values
//=========================================================================

void  SampleArea::SetValues(unsigned short *p_Values)
{
   int i;
   for(i=0; i<SAMPLE_AREA_VALUES; i++)
   {
      // adjust to 0..31 
      us_Values[i] = p_Values[i] >> 11;
   }
} // SetValues()

//=========================================================================
//
//=========================================================================
