// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//

#include <stdio.h>
#include <unistd.h>



#include "saveblock.h"



//===========================================================================
// constructor for one saveblock
//===========================================================================

SaveBlock::SaveBlock(size_t size)
{
   if(size >0)
   {
      Buffer = new unsigned char[size];
      if(Buffer != NULL)
      {
         BufferSize = size;
      }
      else
      {
         BufferSize =0;
      }
      BufferFilled =0;
      BufferSavePosition =NULL;
   
   }
   else // zero size, should not happen
   {
      Buffer             = NULL;
      BufferSize         =0;
      BufferFilled       =0;
      BufferSavePosition =NULL;
   }

   // reset the trigger for the last block
   b_FinalBlock = false;
   
   m_i_Level = 5000;
}

//===========================================================================
// delete the saveblocks
//===========================================================================

SaveBlock::~SaveBlock()
{
   if(Buffer != NULL)
   {
      delete[] Buffer;
      Buffer = NULL;
   }
}

//===========================================================================
// write on block into a file
//===========================================================================


int SaveBlock::WriteBlock(int OutFileHandle)
{

   if( OutFileHandle <=0 )
   {
      return 0;
   }
   
   // write the data
   write(OutFileHandle,Buffer,BufferFilled);
   
   // remember: this block is empty now
   BufferFilled=0;
   
   
   return 0;

} // WriteBlock()


//===========================================================================
// check, whether it is the final block
//===========================================================================

bool SaveBlock::IsFinalBlock()
{
   return b_FinalBlock;
}

//===========================================================================
// set the final block flag
//===========================================================================

void SaveBlock::SetFinalBlock()
{
   b_FinalBlock = true;
} // SetFinalBlock()

//===========================================================================
// schwelle is from 0 to 10
//===========================================================================


bool SaveBlock::Analyze(int schwelle, float & f_Level)
{
   unsigned char *uc_InBuffer;
   short int      left;
   short int      right;
   int            i;
   int            i_Total;
   int            i_CompareValue;
   int            i_Divider;
   
   uc_InBuffer = Buffer;

   i_CompareValue = m_i_Level ;
   i_Total=0;  
   
   if(schwelle<=0)
   {
      schwelle =1;
   }
   
   for (i=0; i<BufferFilled; i+=4)
   {
      left =0;
      right=0;

      // get the sample values
      left  |= ((int)( * uc_InBuffer++ )) ;
      left  |= ((int)( * uc_InBuffer++ )) << 8;
      right |= ((int)( * uc_InBuffer++ )) ;
      right |= ((int)( * uc_InBuffer++ )) << 8;
  
      // make the positive
      if(left<0)
      {
          left = -left;
      }
      if(right <0)
      {
         right = -right;
      }
      
      // are they over a level            
      if(    left  > i_CompareValue 
          || right > i_CompareValue)
      {
          // then increase the counter
          i_Total += 1;
      }

   } // for i

   i_Divider = BufferFilled / (4 * schwelle);
   if (i_Divider  <=0)
   {
      m_i_AnalyzeResult = 0;
   }
   else 
   {
      m_i_AnalyzeResult = i_Total / i_Divider;
   }

   // fprintf(stderr,"left = %04x right = %04x total = %d result = %d\n",
   //              left,right,i_Total,m_i_AnalyzeResult);
   {

      // calculate level between 0 and 1.0
      f_Level = (float) i_Total / BufferFilled ;
      f_Level *= 4.0;
      if(f_Level > 1.0) 
      {
         f_Level = 1.0;
      }


      //fprintf(stderr,"left = %04x right = %04x total = %d level = %f\n",
      //           left,right,i_Total,
      //           f_Level
      //           );
   }

   if(m_i_AnalyzeResult > 1 )
   {
      return true;
   }
   
   return false;
   
} // Analyze()


//###########################################################################
// now create an array of safeblocks
//###########################################################################


//===========================================================================
// constructor

//===========================================================================
SaveBlockArray::SaveBlockArray()
{
   int i;
   
   // allocate the buffers
   for( i=0; i < SAVEBLOCKARRAYSIZE; i++)
   {
      // create a new block, size hardcoded 64K
      pc_SaveBlocks[i]= new SaveBlock(SAVEBLOCKBUFFERSIZE);
   }
   
   
   // init the variables
   
   NextRecordBlock = 0;
   NextWriteBlock  = 0;
   
   // create the semaphore for sync of record/write
   sem_init(&BufferSemaphore,
            0,                  // process share
            0                   // initital value
            );
   
} //SaveBlockArray()

//===========================================================================
// destructor
//===========================================================================


SaveBlockArray::~SaveBlockArray()
{
   int i;
   
   // free all blocks
   for(i=0; i<SAVEBLOCKARRAYSIZE; i++)
   {
      if( pc_SaveBlocks[i] != NULL)
      {
         delete pc_SaveBlocks[i];
         pc_SaveBlocks[i] = NULL;
      }
   }
   
   // now delete the semaphore
   sem_destroy(&BufferSemaphore);
   
} // ~SaveBlockArray()

//===========================================================================
// get next record block
// assume, there is always one, so no semaphore is used
//===========================================================================


SaveBlock *SaveBlockArray::GetNextRecordBlock()
{
 
   SaveBlock *pc_NextBlock;
 
   pc_NextBlock = pc_SaveBlocks[NextRecordBlock];
   
   NextRecordBlock += 1;
   if(NextRecordBlock >= SAVEBLOCKARRAYSIZE)
   {
      NextRecordBlock =0;
   }
   
   return pc_NextBlock;
}

//===========================================================================
// release a block for the writing
//===========================================================================


void SaveBlockArray::ReleaseRecordBlock()
{
   // simply increment the semaphore
   sem_post(&BufferSemaphore);
}

//===========================================================================
// get a block for writing.
// because the can be none available, wait with the semaphore for a block
//===========================================================================

SaveBlock *SaveBlockArray::GetNextWriteBlock()
{
 
   SaveBlock *pc_NextBlock;
 
   // wait for the block
   sem_wait(&BufferSemaphore);
   
   // get block
   pc_NextBlock = pc_SaveBlocks[NextWriteBlock];
   
   NextWriteBlock += 1;
   if(NextWriteBlock >= SAVEBLOCKARRAYSIZE)
   {
      NextWriteBlock =0;
   }
   
   
   return pc_NextBlock;
   
} // GetNextWriteBlock()

//===========================================================================
//
//===========================================================================

