// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//

#include "settingsbox.h"
#include <qdialog.h>
#include <qpushbutton.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qdir.h>
#include "hdrecord.h"

#include <stdio.h>

#include <sys/ioctl.h>
#include <linux/soundcard.h>

//-----------------------------------------------------------------------------

SettingsBox::SettingsBox(QWidget *parent,
                        HdRecord *theApplication) : 
		QDialog(parent,"Settings",TRUE)
{
   QPushButton  *CloseButton;
   QPushButton  *CancelButton;
   QBoxLayout   *topLayout;
   QGridLayout  *grid;
   QBoxLayout   *dialogButtons;

   p_TheApplication = theApplication;

   topLayout = new QVBoxLayout( this, 2 );


   // set the titel 
   setCaption("KHdRecord Settings");


   grid = new QGridLayout( topLayout, 5, 2, 10 );

   m_BitrateCombo = new QComboBox(false,this,"BitrateSelection");
#ifdef LOWFI
   m_BitrateCombo->insertItem("8 KBit/sec Mono",0);
   m_BitrateCombo->insertItem("16 KBit/sec Mono",1);
#else
   m_BitrateCombo->insertItem("128 KBit/sec",0);
   m_BitrateCombo->insertItem("160 KBit/sec",1);
   m_BitrateCombo->insertItem("192 KBit/sec",2);
   m_BitrateCombo->insertItem("256 KBit/sec",3);
   m_BitrateCombo->insertItem("40 KBit/sec / Mono",4);
   m_BitrateCombo->insertItem("320 KBit/sec",5);
#endif
   grid->addWidget(new QLabel("Encoding Bitsize",this),0,0);
   grid->addWidget(m_BitrateCombo,0,1);


   m_InputDeviceCombo = new QComboBox(false,this,"InputDevice");
   m_InputDeviceCombo->insertItem("Line",0);
   m_InputDeviceCombo->insertItem("Microphone",1);
   m_InputDeviceCombo->insertItem("CD",2);
   grid->addWidget(new QLabel("Input Device",this),1,0);
   grid->addWidget(m_InputDeviceCombo,1,1);

   m_FilenameChangeModeCombo = new QComboBox(false,this,"InputDevice");
   m_FilenameChangeModeCombo->insertItem("Start with basic name",0);
   m_FilenameChangeModeCombo->insertItem("Always count up",1);
   m_FilenameChangeModeCombo->insertItem("Use date",2);
   grid->addWidget(new QLabel("Filename Change Mode",this),2,0);
   grid->addWidget(m_FilenameChangeModeCombo,2,1);

   m_SoundDeviceEdit = new QLineEdit(this);
   grid->addWidget(new QLabel("Sound Device",this),3,0);
   grid->addWidget(m_SoundDeviceEdit,3,1);

   m_MixerDeviceEdit = new QLineEdit(this);
   grid->addWidget(new QLabel("Mixer Device",this),4,0);
   grid->addWidget(m_MixerDeviceEdit,4,1);


   // --- the cancel and ok buttons
   dialogButtons = new QHBoxLayout( topLayout);

   CancelButton = new QPushButton( "Cancel", this );
   //cancel->setGeometry( 10,60, 100,30 );
   connect( CancelButton, SIGNAL(clicked()), SLOT(reject()) );


   CloseButton = new QPushButton( "OK", this );
   connect( CloseButton, SIGNAL(clicked()), SLOT(ClosePressed()) );

   dialogButtons->addWidget(CloseButton);   
   dialogButtons->addWidget(CancelButton);   



   // set the actual bitsize into the combo box
   switch(p_TheApplication -> m_i_MP3Bitsize)
   {
   case 128:
      m_BitrateCombo->setCurrentItem(0);
      break;
   case 160:
      m_BitrateCombo->setCurrentItem(1);
      break;
   case 192:
      m_BitrateCombo->setCurrentItem(2);
      break;
   case 256:
      m_BitrateCombo->setCurrentItem(3);
      break;
   case 40:
      m_BitrateCombo->setCurrentItem(4);
      break;
   case 320:
      m_BitrateCombo->setCurrentItem(5);
      break;
   case 8:
      m_BitrateCombo->setCurrentItem(0);
      break;
   case 16:
      m_BitrateCombo->setCurrentItem(1);
      break;
   default:
      m_BitrateCombo->setCurrentItem(0);
      break;
      
   }
   
   // set the actual bitsize into the combo box
   switch(p_TheApplication -> m_i_MixerInputDevice)
   {
   case SOUND_MIXER_LINE:
      m_InputDeviceCombo->setCurrentItem(0);
      break;
   case SOUND_MIXER_MIC:
      m_InputDeviceCombo->setCurrentItem(1);
      break;
   case SOUND_MIXER_CD:
      m_InputDeviceCombo->setCurrentItem(2);
      break;
   default:
      m_InputDeviceCombo->setCurrentItem(0);
      break;
   }

   // set the actual bitsize into the combo box
   switch(p_TheApplication -> e_FilenameChangeMode)
   {
   case FilenameChange_StartWithBasename:
      m_FilenameChangeModeCombo->setCurrentItem(0);
      break;
   case FilenameChange_AlwaysIncrement:
      m_FilenameChangeModeCombo->setCurrentItem(1);
      break;
   case FilenameChange_UseDate:
      m_FilenameChangeModeCombo->setCurrentItem(2);
      break;
   default:
      m_FilenameChangeModeCombo->setCurrentItem(0);
      break;
   }
   
   // set the device names into the edit fields
   m_SoundDeviceEdit->setText(p_TheApplication->m_qs_SoundDevice);
   m_MixerDeviceEdit->setText(p_TheApplication->m_qs_MixerDevice);

} // SettingsBox() constructor

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void SettingsBox::ClosePressed()
{
   int i_Bitsize;
   int i_InputDevice;
   int i_FilenameChangeMode;
   
   // get the bitsize setting
   i_Bitsize = m_BitrateCombo->currentItem();
   
   switch(i_Bitsize)
   {
#ifdef LOWFI
   case 0:
      p_TheApplication -> m_i_MP3Bitsize = 8;
      break;
   case 1:
      p_TheApplication -> m_i_MP3Bitsize = 16;
      break;
#else
   case 0:
      p_TheApplication -> m_i_MP3Bitsize = 128;
      break;
   case 1:
      p_TheApplication -> m_i_MP3Bitsize = 160;
      break;
#endif
   case 2:
      p_TheApplication -> m_i_MP3Bitsize = 192;
      break;
   case 3:
      p_TheApplication -> m_i_MP3Bitsize = 256;
      break;
   case 4:
      p_TheApplication -> m_i_MP3Bitsize = 40;
      break;
   case 5:
      p_TheApplication -> m_i_MP3Bitsize = 320;
      break;
   default:
      p_TheApplication -> m_i_MP3Bitsize = 128;
      break;
   }


   // get the bitsize setting
   i_InputDevice = m_InputDeviceCombo->currentItem();
   
   switch(i_InputDevice)
   {
   case 0:
      p_TheApplication -> m_i_MixerInputDevice = SOUND_MIXER_LINE;
      break;
   case 1:
      p_TheApplication -> m_i_MixerInputDevice = SOUND_MIXER_MIC;
      break;
   case 2:
      p_TheApplication -> m_i_MixerInputDevice = SOUND_MIXER_CD;
      break;
   default:
      p_TheApplication -> m_i_MixerInputDevice = SOUND_MIXER_LINE;
      break;
   }
   
   // get the bitsize setting
   i_FilenameChangeMode = m_FilenameChangeModeCombo->currentItem();
   
   switch(i_FilenameChangeMode)
   {
   case 0:
      p_TheApplication -> e_FilenameChangeMode = FilenameChange_StartWithBasename;
      break;
   case 1:
      p_TheApplication -> e_FilenameChangeMode = FilenameChange_AlwaysIncrement;
      break;
   case 2:
      p_TheApplication -> e_FilenameChangeMode = FilenameChange_UseDate;
      break;
   default:
      p_TheApplication -> e_FilenameChangeMode = FilenameChange_StartWithBasename;
      break;
   }
   
   
   // apply the mixer setting to the mixer device
   p_TheApplication ->SetInputMixer();
   
   p_TheApplication->m_qs_SoundDevice = m_SoundDeviceEdit->text();
   p_TheApplication->m_qs_MixerDevice = m_MixerDeviceEdit->text();
   
   
   // finish the dialog
   accept();
} // ClosePressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


