// This sourcefile is created by Peter Jodda / peter@jodda.de
// It is under the Peter Jodda license:
// Everyboy may use it on his own risk. This header must stay in the file!
// If you expand this file, please contact me, that I can integrate the
// enhancemants in the original file.
//

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <time.h>

#include "timedsplitbox.h"
#include <qdialog.h>
#include <qpushbutton.h>
#include <qgrid.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qdir.h>
#include <qmessagebox.h>

#include "hdrecord.h"

#include <stdio.h>

//-----------------------------------------------------------------------------

TimedSplitBox::TimedSplitBox(QWidget *parent,
                        HdRecord *theApplication) : 
		QDialog(parent,"TimedSplitBox",TRUE)
{
   QBoxLayout   *topLayout;
   QBoxLayout   *TimeLayout;
   QBoxLayout   *ButtonLayout;
   QWidget      *TimeWidget;
   QWidget      *ButtonWidget;
   QLabel       *TimeFrontLabel;
   QLabel       *TimeBackLabel;
   
   p_TheApplication = theApplication;
   
   
   m_i_CurrentTime = p_TheApplication->m_i_TimedSplitSeconds;

   setCaption("KHdRecord Timed Split Setup");

   // top layout is vertical box
   topLayout = new QVBoxLayout( this, 2 );
   
   // create a horizontal box for the time input
   TimeWidget = new QWidget(this);
   topLayout->addWidget(TimeWidget);
   TimeLayout = new QHBoxLayout( TimeWidget, 2 );

   TimeFrontLabel = new QLabel("Automtic split all ",TimeWidget);
   m_TimeEdit = new QLineEdit(TimeWidget);
   TimeBackLabel = new QLabel("seconds ",TimeWidget);

   TimeLayout->addWidget(TimeFrontLabel);
   TimeLayout->addWidget(m_TimeEdit);
   TimeLayout->addWidget(TimeBackLabel);

   // create a horizonzal box for the ok and cancel button
   ButtonWidget = new QWidget(this);
   topLayout->addWidget(ButtonWidget);
   ButtonLayout = new QHBoxLayout( ButtonWidget, 2 );

   m_CancelButton  = new QPushButton( "Cancel"            , ButtonWidget );
   connect( m_CancelButton, SIGNAL(clicked()), SLOT(CancelPressed() ) );

   m_OKButton  = new QPushButton( "OK"            , ButtonWidget );
   connect( m_OKButton, SIGNAL(clicked()), SLOT(OKPressed() ) );


   ButtonLayout->addWidget(m_CancelButton);
   ButtonLayout->addWidget(m_OKButton);


   // set a signal on change of the time edit field
   connect( m_TimeEdit,  
               SIGNAL(textChanged(const QString &)), 
               SLOT(DoTimeChanged(const QString &)) );


   // set the default values
   SetDefaultValues();


   // en/disable buttons
   SetButtonSensitivity();
   
   
} // TimedSplitBox() constructor

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void TimedSplitBox::OKPressed()
{
   // send back the time into the application
   p_TheApplication->m_i_TimedSplitSeconds = m_i_CurrentTime;

   // finish the dialog
   accept();

} // OKPressed()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void TimedSplitBox::CancelPressed()
{
   // only finish the dialog
   accept();

} // CancelPressed()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void TimedSplitBox::SetDefaultValues()
{
   char ca_Time[30];
   
   // copy the current tima value into the dialog   
   sprintf(ca_Time,"%d",m_i_CurrentTime);

   m_TimeEdit->setText(ca_Time);
   m_b_TimeValid = true;
   
} // SetDefaultValues()


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------

void TimedSplitBox::DoTimeChanged(const QString & qs_Time)
{
   char c_DummyChar;
   int  i_NewTime;
   int  i_SScanfResult;
   
   // get a time value out of the doalog string   
   i_SScanfResult = sscanf(qs_Time,"%d%c",&i_NewTime,&c_DummyChar);


   // 1 is the preferred result
   if(i_SScanfResult == 1)
   {
      m_b_TimeValid = true;
      m_i_CurrentTime = i_NewTime;
   }
   else
   {
      // wrong characters in input field
      m_b_TimeValid = false;
   }   
   		
   // en/disable buttons		
   SetButtonSensitivity();


} // DoTimeChanged()

//-----------------------------------------------------------------------------
// enable or disable the add, change and delete button depending on 
// several states
//-----------------------------------------------------------------------------

void TimedSplitBox::SetButtonSensitivity()
{

   // only enable the ok button, if the time is valid
   if(m_b_TimeValid)
   {
      m_OKButton->setEnabled(true);
   }
   else
   {
      m_OKButton->setEnabled(false);
   }
   
   
} // SetButtonSensitivity()

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------


